/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>

#include "gdis.h"

#define DEBUG 0
#define DEBUG2 0

/* global pak structures */
extern struct sysenv_pak sysenv;
extern struct elem_pak elements[ATOMDATA];

/* animation globals */
gint anim_loop=0;
GtkWidget *d_spinner;

/* TODO - a stop button for animations (difficult?), plus a way of */
/* specifying that pbc confinement should/should not be performed */
/* on each frame. */

/****************/
/* do animation */
/****************/
void anim_hook(GtkWidget *w)
{
animate(sysenv.active,0);
}

/***********************/
/* configure animation */
/***********************/
void animate_setup_widget()
{
gint id;
GtkWidget *frame, *vbox, *hbox, *button, *label;
GtkAdjustment *adj;
struct dialog_pak *anim_dialog;

if (!sysenv.num_models) 
  return;

if ((id = request_dialog(sysenv.active, ANIM)) < 0)
  return;
anim_dialog = &sysenv.dialog[id];

anim_dialog->win = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW(anim_dialog->win), "Animate");
gtk_signal_connect(GTK_OBJECT(anim_dialog->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

/* Frame 1 - general animation stuff */
frame = gtk_frame_new ("Properties");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(anim_dialog->win)->vbox),frame,TRUE,TRUE,0);
/* create a vbox in the frame */
vbox = gtk_vbox_new (FALSE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);

hbox = gtk_hbox_new(FALSE, 0);
gtk_container_set_border_width(GTK_CONTAINER (hbox), 10);
gtk_box_pack_start(GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

/* TODO - align the adjustment widgets */

/* delay - label */
label = gtk_label_new ("Frame delay (units) ");
/* align left */
gtk_misc_set_alignment(GTK_MISC(label), 0, 0);
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

/* delay - adjustment */
adj = (GtkAdjustment *) gtk_adjustment_new (0, 0, 10, 1, 1, 0);
d_spinner = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(d_spinner), TRUE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(d_spinner), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), d_spinner, FALSE, TRUE, 0);

/* TODO - if file read counted DATE or BIOSYM lines we would */
/*        know how many frames there were */

hbox = gtk_hbox_new (FALSE, 0);
gtk_container_set_border_width (GTK_CONTAINER (hbox), 10);
gtk_box_pack_start (GTK_BOX (vbox), hbox, TRUE, TRUE, 0);

/* frame - label */
label = gtk_label_new ("Display frame ");
/* align left */
gtk_misc_set_alignment (GTK_MISC (label), 0, 0);
gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
/* frame - adjustment */
adj = (GtkAdjustment *) gtk_adjustment_new (0, 0, 10, 1, 1, 0);
d_spinner = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_wrap (GTK_SPIN_BUTTON (d_spinner), TRUE);
gtk_spin_button_set_shadow_type (GTK_SPIN_BUTTON (d_spinner), GTK_SHADOW_OUT);
gtk_box_pack_start (GTK_BOX (hbox), d_spinner, FALSE, TRUE, 0);

/* Last - terminating buttons */
button = gtk_button_new_with_label (" Run ");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(anim_dialog->win)->action_area),
                                      button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(anim_hook),
                          GTK_OBJECT(anim_dialog->win));

button = gtk_button_new_with_label ("Close");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(anim_dialog->win)->action_area),
                                      button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(close_dialog),
                          (gpointer) id);

/* display the dialog */
gtk_widget_show_all(anim_dialog->win);
}

/**********************************/
/* open and configure a data file */
/**********************************/
/* TODO - make mode a struct if more features are implemented */
void animate(gint model, gint mode)
{
FILE *fp;
gint flag;
struct parse_pak parse_data;
struct model_pak *data;

data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* only BIOSYM files accepted */
if (data->id != BIOSYM)
  {
  show_text("Only BIOSYM models can be animated.");
  return;
  }

/* not really needed? */
if (data->num_frames < 2)
  {
  show_text("No extra frames found.");
  return;
  }

show_text("Animating current model...");    
fp = fopen(data->filename,"r");

/* was a file stream opened successfully? */
if (!fp)
  {
  printf("Error, animate(): could not open file!\n");
  return;
  }

/* get appropriate parsing structure */
if (parse_ptr(&parse_data, data->id))
  {
  fclose(fp);
  return;
  }
/* we've already parsed this header on initial load */
parse_data.hdr_skip = 0;

/* animation loop */
flag=0;
rewind(fp);
while(!flag)
  {
/* load coords into the models data space */
  flag = load_data(fp, &parse_data, data);
  if (!flag)
    {
/* refresh the PBC & xlat vectors (in case it changed) */
    init_objs(CELL_ON, data);
    make_xlat(data);
/* calc mol centroids for the new frame */
    init_objs(REDO_COORDS, data);
    calc_bonds(data);
    calc_mols(data);
/* TODO - PBC applied only if desired */
    if (data->periodic)
      pbc_constrain_mols(data);
/* recalc after pbc constrain */
    init_objs(REDO_COORDS, data);
    calc_bonds(data);
    calc_mols(data);
    wire_draw(SINGLE);
    }
  }
show_text("Animation completed.");
fclose(fp);
}

