/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include "setoper.h" 
  /* set operation library header (March 16, 1995 version or later) */
#include "cdddef.h"
#include "cdd.h"
#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <math.h>
#include <string.h>

#include "gdis.h"

/* TODO - get rid of all n_input's - it's always 4 ie len,x,y,z*/
#define NDIM 4

/* CDD main call */
void DDEnumerate(rowrange, colrange, rowrange, colrange, Amatrix, struct model_pak *);

long mprev=0;
double *beta=NULL;
long mlast=0;

/************************/
/* transfer vertex data */
/************************/
#define DEBUG_SAVE_VERTEX 0
void save_vertex(struct model_pak *data)
{
gint j;
RayRecord *RR;

/* setup for placing in GDIS struct */
data->num_vertices = VertexCount;
if (data->num_vertices < 4)
  {
  printf("Insufficient planes to define a closed polyhedron.\n");
/* force bad */
  data->num_vertices=0;
  return;
  }
data->vertices = (struct vertex_pak *) 
                  g_malloc(data->num_vertices * sizeof(struct vertex_pak));

#if DEBUG_SAVE_VERTEX
printf("Found %d vertices.\n",data->num_vertices);
printf("Vertex list:\n");
#endif

RR = FirstRay;
j=0;
while (RR != NULL) 
  {
  if (RR->feasible) 
    {
#if DEBUG_SAVE_VERTEX
printf("[%d]  ",j);
printf("%5.2f %5.2f %5.2f\n",RR->Ray[0]/fabs(RR->Ray[0])
                            ,RR->Ray[1]/fabs(RR->Ray[0])
                            ,RR->Ray[2]/fabs(RR->Ray[0]));
#endif

if (j>=data->num_vertices)
  printf("HELP!\n");
    (data->vertices+j)->x = RR->Ray[1]/fabs(RR->Ray[0]);
    (data->vertices+j)->y = RR->Ray[2]/fabs(RR->Ray[0]);
    (data->vertices+j)->z = RR->Ray[3]/fabs(RR->Ray[0]);
    j++;
    }
  RR = RR->Next;
  }
}

/***************************/
/* transfer adjacency data */
/***************************/
#define DEBUG_SAVE_ADJ 0
void save_adj(struct model_pak *data, Amatrix A)
{
gint i, j, id, m_size, n_size;
RayRecord *RayPtr1, *RayPtr2;
long pos1, pos2, degree;
boolean adj;
node *headnode, *tailnode, *newnode, *prevnode;

headnode=NULL; tailnode=NULL;

m_size = data->num_planes + 1;
n_size = NDIM;

#if DEBUG_SAVE_ADJ
printf("Adjacency list:\n");
#endif

if (RayCount==0)
  return;

LastRay->Next=NULL;
for (RayPtr1=FirstRay, pos1=1;RayPtr1 != NULL; RayPtr1 = RayPtr1->Next, pos1++)
  {
  for (RayPtr2=FirstRay, pos2=1,degree=0; RayPtr2 != NULL; RayPtr2 = RayPtr2->Next, pos2++)
    {
    if (RayPtr1!=RayPtr2)
      {
        CheckAdjacency2(m_size, n_size, A, &RayPtr1, &RayPtr2, &adj);
        if (adj) 
          {
          degree++;
          if (degree==1)
            {
            newnode=(node *)g_malloc(sizeof *newnode);
            newnode->key=pos2;
            newnode->next=NULL;
            headnode=newnode;
            tailnode=newnode;
            }
          else
            {
            newnode=(node *)g_malloc(sizeof *newnode);
            newnode->key=pos2;
            newnode->next=NULL;
            tailnode->next=newnode;
            tailnode=newnode;
            }
          }
        }
      }
/* stupid pascal numbering */
    i = (gint) pos1-1;
    j = (gint) degree;
#if DEBUG_SAVE_ADJ
printf(" Vertex %d  has %d adj :", i, j);
#endif
/* allocate for adjacency */
    (data->vertices+i)->num_adj = j;
    (data->vertices+i)->adj = (gint *) g_malloc(j*sizeof(gint));
    j=0;
    for (newnode=headnode; newnode!=NULL; newnode=newnode->next, free(prevnode))
      {
      prevnode=newnode;
/* stupid pascal numbering */
      id = (gint) newnode->key - 1;
      *((data->vertices+i)->adj+j) =  id;
#if DEBUG_SAVE_ADJ
printf(" %d", id);
#endif
      j++;
      }
#if DEBUG_SAVE_ADJ
printf("\n");
#endif
  }
}

/******************************/
/* compute visible facet data */
/******************************/
#define DEBUG_SAVE_FACET 0
void save_facet(struct model_pak *data)
{
gint i, m, id, j, p, vertex;
gint *facet, *new_id, *old_id;
long elem, zcar;
rowset cset;
RayRecord *RR;

m = data->num_planes+1;

/* FIXME -  are 3 these being overrun somewhere? */
/*          doesn't seem so, but adding a 10 * factor */
/*          helps a bit (but doesn't completely fix) */

/* number of facets <= number of input planes */
facet = (gint *) g_malloc(data->num_planes*sizeof(gint));
for (i=0 ; i<data->num_planes ; i++)
  *(facet+i) = 0;
/* switch between plane & facet indexing */
new_id = (gint *) g_malloc(data->num_planes*sizeof(gint));
old_id = (gint *) g_malloc(data->num_planes*sizeof(gint));

/* pass1 - get sizes for alloc */
#if DEBUG_SAVE_FACET
printf("Planes:\n");
#endif
RR = FirstRay;
i=0;
while (RR != NULL) 
  {
  if (RR->feasible) 
    {
    set_initialize(&cset,m);
    zcar = set_card(RR->ZeroSet);
    if (m - zcar >= zcar) 
      {
#if DEBUG_SAVE_FACET
printf("Vetex %d at junction of %1ld planes: ",i,zcar);
#endif
      for (elem=1 ; elem<=RR->ZeroSet[0] ; elem++)
        if (set_member(elem,RR->ZeroSet))
          {
/* plane number */
          id = (gint) elem-1;
          if (id >= data->num_planes)
            printf("HELP!\n");
#if DEBUG_SAVE_FACET
printf("%d ",id);
#endif
/* facet number id has another vertex */
/* NB: stupid pascal numbering */
          (*(facet+id))++;
          }
      i++;
#if DEBUG_SAVE_FACET
printf("\n");
#endif
      }
    else 
      {
      printf("badly truncated code.\n");
      }
    set_free(cset);
    }
  RR = RR->Next;
  }

/* count up the number of facets that have a non zero number of vertices */
data->num_facets = 0;
for (i=0 ; i<data->num_planes ; i++)
  {
  if (*(facet+i))
    {
/* plane i -> facet list */
    *(new_id+i) = data->num_facets;
/* facet -> plane i */
    *(old_id+data->num_facets) = i;
    data->num_facets++;
    }
  }

/* allocate for facet list */
#if DEBUG_SAVE_FACET
printf("Number of visible facets: %d\n",data->num_facets);
#endif
data->facets = (struct facet_pak *) 
                g_malloc(data->num_facets*sizeof(struct facet_pak));

/* allocate & init for each facet's vertex listing */
i=id=0;
while(id < data->num_facets)
  {
if (i >= data->num_planes)
  printf("HELP\n");

  p = *(facet+i);
  if (p)
    {

if (id >= data->num_facets)
  printf("HELP\n");

#if DEBUG_SAVE_FACET
printf("facet %d has %d vertices.\n",id,p);
#endif
    (data->facets+id)->points = (gint *) g_malloc(p*sizeof(gint));
    (data->facets+id)->num_points = 0;
    (data->facets+id)->lim_points = p;

/* get plane label (miller indices) */
/* remember - normal can't be computed until latmat transformation */
    j = *(old_id+id);
    (data->facets+id)->index[0] = (data->planes+j)->m[0];
    (data->facets+id)->index[1] = (data->planes+j)->m[1];
    (data->facets+id)->index[2] = (data->planes+j)->m[2];
    id++;
    }
  i++;
  }

/* pass2 - construct vertex list for each facet */
RR = FirstRay;
vertex=0;
while (RR != NULL) 
  {
  if (RR->feasible) 
    {
    set_initialize(&cset,m);
    zcar = set_card(RR->ZeroSet);
    if (m - zcar >= zcar) 
      {
      for (elem=1 ; elem<=RR->ZeroSet[0] ; elem++)
        {
        if (set_member(elem,RR->ZeroSet))
          {
/* facet number elem 'contains' current vertex */
/* NB: stupid pascal numbering */
          i = (gint) elem - 1;
          id = *(new_id+i);
          p = (data->facets+id)->num_points;
          *((data->facets+id)->points+p) = vertex;
          (data->facets+id)->num_points++;
          }
        }
      vertex++;
      }
    }
  RR = RR->Next;
  }

/* output summary */
#if DEBUG_SAVE_FACET
printf("-----------------------------\n");
for (i=0 ; i<data->num_facets ; i++)
  printf("facet %d has %d vertices.\n",i,(data->facets+i)->num_points);
#endif

/* done */
g_free(facet);
g_free(new_id);
g_free(old_id);
}

/*****************************************************/
/* Main morphology call for vertex/facet enumeration */
/*****************************************************/
#define DEBUG_EXEC_CDD 0
void exec_cdd(struct model_pak *data)
{
gint i,j;
rowrange m_input, m_size;
colrange n_input;
Amatrix AA;

/* static initialization */
mprev=0;
if (beta != NULL)
  g_free(beta);
mlast=0;

/* setup run flags */
HyperplaneOrder = LexMin;
AdjacencyTest = Combinatorial;
NondegAssumed = FALSE;
RecomputeRowOrder=TRUE;
PreOrderedRun=FALSE;
InitBasisAtBottom = FALSE;
Error=None;
CompStatus=InProgress;
m_input = data->num_planes;
n_input = NDIM;                   /* ineq -> len + h k l */
/* can be Real or Integer, makes no diff (even if input is float, strange) */
Number = Real;
/*
Number = Integer;
*/
RestrictedEnumeration = FALSE;
RelaxedEnumeration = FALSE;

/* feed planes into matrix */
for (i=0 ; i<data->num_planes; i++) 
  {
  AA[i]= (double *) calloc(n_input, sizeof(double));
  AA[i][0] = (double) (data->planes+i)->len;
/* input normalized miller indices */
  for (j=1 ; j<n_input ; j++) 
    AA[i][j] = (double) (data->planes+i)->x[j-1];
  }

/* non homogeneous - add row */
m_size = m_input + 1;
AA[m_size-1] = (double *) calloc(n_input, sizeof(double));
/* artificial row for x_1 >= 0*/
AA[m_size-1][0] = 1.0;
for (j=1 ; j<n_input ; j++) 
  AA[m_size-1][j] = 0.0;

/* more init */
EqualityIndex = (long *) calloc(m_size+1, sizeof(EqualityIndex));
SetInequalitySets(m_size, EqualityIndex);

/* DEBUG - main input matrix */
#if DEBUG_EXEC_CDD
for (i=0 ; i<m_size ; i++)
  {
  for (j=0 ; j<n_input ; j++)
    {
    printf("%f ",AA[i][j]);
    }
  printf("\n");
  }
#endif

/* vertex/facets enumeration - and model_pak storage */
DDEnumerate(m_input, n_input, m_size, n_input, AA, data);

return;
}

