/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <math.h>

#include "gdis.h"

/* data structures */
extern struct sysenv_pak sysenv;
extern struct elem_pak elements[];

/****************/
/* SPECIAL OBJS */
/****************/
#define DEBUG_INIT 0
void init_objs(gint type, struct model_pak *data)
{
gint i, flag;

g_return_if_fail(data != NULL);

switch(type)
  {
/* reference axes */
  case AXES_ON:
    data->axes_on = TRUE;
    make_axes(data);
    break;
  case AXES_OFF:
    data->axes_on = FALSE;
    break;

/* cell wire frame */
  case CELL_ON:
    data->cell_on = TRUE;
    make_cell(data);
    break;
  case CELL_OFF:
    data->cell_on = FALSE;
    break;

/* creation mesh */
  case MESH_ON:
    data->mesh_on = TRUE;
    break;
  case MESH_OFF:
    data->mesh_on = FALSE;
    for (i=data->num_atoms ; i-- ; )
      if ((data->atoms+i)->status == SLITE)
        (data->atoms+i)->status = NORMAL;
    break;

/* set all atoms back to element colour */
  case REFRESH_COLOUR:
    for (i=data->num_atoms ; i-- ; )
      ARR3SET((data->atoms+i)->colour,
              elements[(data->atoms+i)->atom_code].colour);
    for (i=data->num_shells ; i-- ; )
      ARR3SET((data->shells+i)->colour,
              elements[(data->shells+i)->atom_code].colour);
    break;

/* scale all atom colours by their sof */
  case SOF_COLOUR:
    if (data->has_sof)
      for (i=data->num_atoms ; i-- ; )
        {
        VEC3MUL((data->atoms+i)->colour, (data->atoms+i)->sof);
        }
    break;

/* region lighting */
  case REFRESH_LITE:
/* NEW - colourmap dependant region dimming */
    flag = 1;
/* if depth isn't sufficient, don't dim */
/* CURRENT - with new colormap alloc stuff this may not be necessary */
    if (sysenv.depth < 16)
      flag = 0;
/* refresh atom lighting */
    for (i=data->num_atoms ; i-- ; )
      {
      elem_seek(i, ATOM, data);
      if (!data->region[(data->atoms+i)->region])
        {
        if (flag)
          {
          VEC3MUL((data->atoms+i)->colour, 0.5);
          }
        else
          {
          VEC3SET((data->atoms+i)->colour, 65535, 65535, 65535);
          }
        }
      }
/* refresh shell lighting */
    if (data->show_shells)
      {
      for (i=data->num_shells ; i-- ; )
        {
        elem_seek(i, SHELL, data);
        if (!data->region[(data->shells+i)->region])
          {
          if (flag)
            {
            VEC3MUL((data->shells+i)->colour, 0.5);
            }
          else
            {
            VEC3SET((data->shells+i)->colour, 65535, 65535, 65535);
            }
          }
        }
      }
    break; 

/* atomic coordinates processing */
  case INIT_COORDS:
/* repetition here, since there is interdependence */
/* between centroid (cent_coords) and latmat (update_coords) calc */
    cent_coords(data);
    update_coords(INITIAL, data);
  case CENT_COORDS:
    cent_coords(data);
  case REDO_COORDS:
    update_coords(REFRESH, data);
    update_shells(data);
    break;

  default:
    printf("Unknown object type requested!\n");
  }
}

/********/
/* AXES */
/********/
void make_axes(struct model_pak *data)
{
/* axes end points */
data->axes[0].x = 20.0;
data->axes[0].y = 0.0;
data->axes[0].z = 0.0;
data->axes[1].x = 0.0;
data->axes[1].y = 20.0;
data->axes[1].z = 0.0;
data->axes[2].x = 0.0;
data->axes[2].y = 0.0;
data->axes[2].z = 20.0;
/* axes label positions */
data->axes[3].x = 25.0;
data->axes[3].y = 0.0;
data->axes[3].z = 0.0;
data->axes[4].x = 0.0;
data->axes[4].y = 25.0;
data->axes[4].z = 0.0;
data->axes[5].x = 0.0;
data->axes[5].y = 0.0;
data->axes[5].z = 25.0;
}

/********/
/* CELL */
/********/
void make_cell(struct model_pak *data)
{
#define A data->pbc[0]
#define B data->pbc[1]
#define C data->pbc[2]
data->cell[0].x = 0;
data->cell[0].y = 0;
data->cell[0].z = 0;
data->cell[1].x = 0;
data->cell[1].y = B;
data->cell[1].z = 0;
data->cell[2].x = A;
data->cell[2].y = B;
data->cell[2].z = 0;
data->cell[3].x = A;
data->cell[3].y = 0;
data->cell[3].z = 0;
data->cell[4].x = 0;
data->cell[4].y = 0;
data->cell[4].z = C;
data->cell[5].x = 0;
data->cell[5].y = B;
data->cell[5].z = C;
data->cell[6].x = A;
data->cell[6].y = B;
data->cell[6].z = C;
data->cell[7].x = A;
data->cell[7].y = 0;
data->cell[7].z = C;
}

/******************************************/
/* construct the unit translation vectors */
/******************************************/
#define DEBUG_XLAT 0
void make_xlat(struct model_pak *data)
{
gfloat b1, b2, b3, c1, c2, c3;

/* This basically works by using the cosine rule in conjunction */
/* with a few geometric constraints (eg normalized vectors) */
#if DEBUG_XLAT
printf("Constructing for PBC: [%6.2f %6.2f %6.2f] (%6.2f %6.2f %6.2f)\n",
       data->pbc[0], data->pbc[1], data->pbc[2],
       data->pbc[3], data->pbc[4], data->pbc[5]);
#endif

/* TODO - validity checks on the PBC */
/* the unit a vector is always assumed co-linear with x */
data->latmat[0] = 1.0;
data->latmat[3] = 0.0;
data->latmat[6] = 0.0;

/* compute the translation vector for b */
b1 = cos(data->pbc[5]);
b2 = sqrt(1.0 - b1*b1);
b3 = 0.0;              /* constrain a,b to remain on the x,y plane */

/* compute the translation vector for c */
c1 = cos(data->pbc[4]);
c2 = (2.0*cos(data->pbc[3]) + b1*b1 + b2*b2 - 2.0*b1*c1 - 1.0)/(2.0*b2);
/* the fabs shouldn't be necessary... */
c3 = sqrt(fabs(1.0 - c1*c1 - c2*c2));

/* assign - latmat is a column vector matrix! */
data->latmat[1] = b1;
data->latmat[4] = b2;
data->latmat[7] = b3;
data->latmat[2] = c1;
data->latmat[5] = c2;
data->latmat[8] = c3;

#if DEBUG_XLAT
P3MAT("Model xlat matrix:",data->latmat);
#endif

return;
}

/**********/
/* SHELLS */
/**********/
#define DEBUG_SHELLS 0
void update_shells(struct model_pak *data)
{
gint i,n,match,tot_match;
gfloat dx,dy,dz,d2;

tot_match = 0;
for (n=data->num_atoms ; n-- ; )
  {
/* default */
  (data->atoms+n)->has_shell = FALSE;
  match=0;
  for (i=data->num_shells ; i-- ; )
    {
    if ((data->shells+i)->atom_code == (data->atoms+n)->atom_code)
      {
      dx = (data->shells+i)->x - (data->atoms+n)->x;
      dy = (data->shells+i)->y - (data->atoms+n)->y;
      dz = (data->shells+i)->z - (data->atoms+n)->z;
      d2 = dx*dx + dy*dy + dz*dz; 
/* FIXME - better cutoff? */
      if (d2 < 2.25)   /* 1.5^2 */
        {
/* reference each other */
        (data->atoms+n)->has_shell = TRUE;
        (data->atoms+n)->idx_shell = i;
        (data->shells+i)->has_core = TRUE;
        (data->shells+i)->idx_core = n;
        match++;
        tot_match++;
        }
      }
    }
/* TODO - how to handle this? */
#if DEBUG_SHELLS
  if (match > 1)
    printf("update_shells() warning: more than one core per shell.\n");
#endif
  }
/* do some shells float down here? */
#if DEBUG_SHELLS
if (tot_match != data->num_shells)
  printf("update_shells() warning: some shell(s) have no cores.\n");
printf("update_shells(): found %d core-shell connections.\n",tot_match);
#endif
}

/**********/
/* COORDS */
/**********/
#define DEBUG_CENT 0
gint cent_coords(struct model_pak *data)
{
gint i, n;
gfloat vec[3], r, r2, rmax;

/* initialize */
VEC3SET(data->offset, 0, 0, 0);
VEC3SET(data->centroid, 0.0, 0.0, 0.0);
r2 = 0.0;

/* Morphology */
if (data->num_vertices)
  {
/* centroid is always the origin, so just calc. scaling */
  r2 = 0.0;
  for (i=data->num_vertices ; i-- ; )
    {
    vec[0] = (data->vertices+i)->x;
    vec[1] = (data->vertices+i)->y;
    vec[2] = (data->vertices+i)->z;
    r = VEC3MAGSQ(vec);
    if (r > r2)
      r2 = r; 
    }
  }

/* coord centroid calc */
n=0;
for (i=data->num_atoms ; i-- ; )
  {
/* don't include deleted atoms */
  if ((data->atoms+i)->status == DELETED)
    continue;
/* transformed coords */
  VEC3SET(vec, (data->atoms+i)->x, (data->atoms+i)->y, (data->atoms+i)->z);
/* NB: centroid should now always be fractional */
  ARR3ADD(data->centroid, vec);
  n++;
  }
/* calculate centroid */
if (n)
  {
  data->centroid[0] /= n;
  data->centroid[1] /= n;
  data->centroid[2] /= n;
  }

/* if centroid is substantially within pbc, make pbc/2 the centroid */
if (data->periodic)
  {
  for (i=data->periodic ; i-- ; )
    if (fabs(data->centroid[i] - 0.5 * data->pbc[i]) < 0.45*data->pbc[i])
      data->centroid[i] = 0.5 * data->pbc[i]; 
  }

/* molecular surface */
if (data->csurf_on)
  {
/* compensate for the surface drawing offset */
/* FIXME - this is a rough hack */
  data->centroid[data->csurf.ix[2]] += data->csurf.offset;
  }

/* get distance of furtherest displayed item from centroid */
/* check atoms */
for (i=data->num_atoms ; i-- ; )
  {
/* don't include deleted atoms */
  if ((data->atoms+i)->status == DELETED)
    continue;
/* compute distance from centroid to atoms */
  VEC3SET(vec, (data->atoms+i)->x, (data->atoms+i)->y, (data->atoms+i)->z);
/* centeroid removal */
  ARR3SUB(vec, data->centroid);
/* transform */
  vecmat(data->latmat,vec);
/* dist squared */
  r = VEC3MAGSQ(vec);
  if (r > r2)
    r2 = r; 
  }

/* check cell vertices */
if (data->periodic)
  {
  for (i=8 ; i-- ; )
    {
    VEC3SET(vec, data->cell[i].x, data->cell[i].y, data->cell[i].z);
/* centroid removal */
    ARR3SUB(vec, data->centroid);
/* transform */
    vecmat(data->latmat,vec);  
/* dist sq. */
    r = VEC3MAGSQ(vec);
    if (r > r2)
      r2 = r; 
    }
  }

rmax = RMAX_SCALE * sqrt(r2);

/* assign the calculated value, unless 0 */
if (rmax)
  data->rmax = rmax;
else
  data->rmax = RMAX_FUDGE;

/* hack to fit mol/sol surfaces on the screen */
/* FIXME - better (auto calc'd) inc vals */
if (data->solsurf_on && data->mysurf[SOLSURF].num_points)
  data->rmax += 3.0;
else
  if (data->molsurf_on && data->mysurf[MOLSURF].num_points)
    data->rmax += 2.0;

#if DEBUG_CENT
P3VEC("centroid = ",data->centroid);
printf("rmax = %f\n",data->rmax);
printf("cent_coords() done.\n");
#endif

return(0);
}

/*********/
/* BONDS */
/*********/
#define DEBUG_BONDS 0
void calc_bonds(struct model_pak *data)
{
gint i,j,n;
gfloat dx,dy,dz;
gfloat r2,rcut,r2cut;

#if DEBUG_BONDS
printf("update_bonds() start.\n");
#endif

/* clean the number of bonds */
data->num_bonds=0;
for (i=data->num_atoms ; i-- ; )
  (data->atoms+i)->atom_bonds = 0;

/* calculate bonds - unique i,j */
for (i=0 ; i<((data->num_atoms)-1) ; i++)
  {
  for (j=i+1 ; j<data->num_atoms ; j++)
    {
/* only test for a bond between undeleted atoms */
    if ((data->atoms+i)->status != DELETED &&
        (data->atoms+i)->status != HIDDEN &&
        (data->atoms+j)->status != DELETED &&
        (data->atoms+j)->status != HIDDEN)
      { 
/* i-j distance */
/* NEW - use rx (not x) as lattice vectors MUST be applied */
      dx = ((data->atoms+i)->rx - (data->atoms+j)->rx);
      dy = ((data->atoms+i)->ry - (data->atoms+j)->ry);
      dz = ((data->atoms+i)->rz - (data->atoms+j)->rz);
      r2 = dx*dx + dy*dy + dz*dz;

/* error trap */
      if (r2 == 0.0)
        {
#if DEBUG_BONDS
printf("Warning: atoms (%d, %d) have identical coordinates\n",i,j);
#endif
        r2=1.0;
        }

      rcut = elements[(data->atoms+i)->atom_code].cova 
           + elements[(data->atoms+j)->atom_code].cova;
/* bond fudge */
      rcut += BOND_FUDGE*rcut;
      r2cut = rcut*rcut;
/* bonded? */
      if (r2 < r2cut)
        {
/* CURRENT - do not bond v close atoms of same type w/ sof < 1.0 */
        if (data->has_sof)
          if ((data->atoms+i)->atom_code == (data->atoms+j)->atom_code)
            if ((data->atoms+i)->sof < 1.0 || (data->atoms+j)->sof < 1.0)
              continue;
/* bonded atoms - reference each other */
        (data->atoms+i)->bond[(data->atoms+i)->atom_bonds] = j;
        (data->atoms+j)->bond[(data->atoms+j)->atom_bonds] = i;
/* update bond list */
        (data->bonds+data->num_bonds)->atom1 = i;
        (data->bonds+data->num_bonds)->atom2 = j;
        (data->bonds+data->num_bonds)->type = SINGLE;
/* this counting should be ok (ie no repeat), as i,j are unique pairs */
        (data->atoms+i)->atom_bonds++;
        (data->atoms+j)->atom_bonds++;
/* allowed bonds (per atom) check */
        if ((data->atoms+i)->atom_bonds >= MAX_BONDS)
          (data->atoms+i)->atom_bonds = MAX_BONDS-1;
        if ((data->atoms+j)->atom_bonds >= MAX_BONDS)
          (data->atoms+j)->atom_bonds = MAX_BONDS-1;
/* increment number of bonds found */
        data->num_bonds++;
        if (data->num_bonds >= data->bond_limit)
          {
/* how many atom pairs looked at */
/* FIXME - this is a hack, the real value should take j into account */
          n = (data->num_bonds*(data->num_atoms-i))/data->num_atoms;
/* failsafe */
/* FIXME - could this (arbitrary) increment cause delays in big models? */
          if (!n)
            n++;
#if DEBUG_BONDS
printf("Found %d bonds, guessing about %d left.\n",data->num_bonds,n);
#endif
          mem_grow(data, BOND, n);
          }
        }
      }      /* if both undeleted */
    }        /* for j */
  }          /* for i */

/* finished */
#if DEBUG_BONDS
printf("update_bonds() done: %d bonds found.\n", data->num_bonds);
#endif

return;
}

/*************/
/* MOLECULES */
/*************/
#define DEBUG_MOLS 0
void calc_mols(struct model_pak *data)
{
gint i, j, k, n, ix, tmp, new_flag;
gint *mol_list, *mol_start;

/* need to do anything? */
data->num_mols=0;
if (!data->num_atoms)
  return;

/* molecule search scheme */
/* NB: limits like MAX_BOND_SIZE/MAX_MOL_SIZE can cause problems if overrun */
#if DEBUG_MOLS
printf("Generating molecule list.\n");
#endif

/* list of atom indices */
mol_list = (gint *) g_malloc(data->num_atoms*sizeof(gint));
/* 1 -> start */
mol_start = (gint *) g_malloc(data->num_atoms*sizeof(gint));

/* init */
for (i=data->num_atoms ; i-- ; )
  {
  *(mol_list+i) = i;
  *(mol_start+i) = 0;
  }

/* i - reference atom */
/* j - new atom index */
/* ix - position in list where atom j goes if bonded to i */
/* n - true atom # of i */

/* first molecule */
i=ix=0;
*(mol_start+i) = 1;
new_flag=1;
/* search for new atoms in current molecule */
while (i<data->num_atoms)
  {
/* if no substitutions made, and no more atoms in the */
/* molecule to compare with - start a new molecule */
  if (new_flag && ix==i)
    {
    *(mol_start+i) = 1;
    ix=i+1;
    }
/* scheme - test if unique i,j are bonded */
  n = *(mol_list+i);     /* current atom */
  j = ix;                /* compare atom */
/* default - no more bonds bound -> start new molecule */
  new_flag=1;
  while (j<data->num_atoms)
    {
/* are i and j bonded? */
    for (k=0 ; k<(data->atoms+n)->atom_bonds ; k++)
      {
      if ((data->atoms+n)->bond[k] == *(mol_list+j))
        {
/* yes, swap j and ix (substitution) atoms */
        tmp = *(mol_list+ix);
        *(mol_list+ix) = *(mol_list+j);
        *(mol_list+j) = tmp;
        new_flag=0;
        ix++;
        break;
        }
      }
/* next atom to compare with */
    j++;
    }
/* next atom to compare against */ 
  i++;
  }

/* use new molecule markers & molecule list to init mols */
for (i=data->num_atoms ; i-- ; )
  if (*(mol_start+i))
    data->num_mols++;

/* free old list */
g_free((data->mols)->atom);
/* alloc for the new */
data->mols = (struct mol_pak *) g_realloc
             (data->mols, data->num_mols*sizeof(struct mol_pak));

/* fill out the molecular atom lists */
j=0;
for (i=0 ; i<data->num_mols ; i++)
  {
/* k marks start of old - j seeks start of new */
  k=j++;
/* NEW - last j is data->num_atoms -> mol_start overrun */
  while (j<data->num_atoms)
    {
    if (*(mol_start+j) == 1)
      break;
    j++;
    }
/* got size - allocate */
  (data->mols+i)->num_atoms = j-k;
  (data->mols+i)->atom = (gint *) g_malloc((j-k)*sizeof(gint));
/* init */
  VEC3SET((data->mols+i)->centroid, 0.0, 0.0, 0.0);
/* read 'em in */
  for (n=0 ; n<(j-k) ; n++)
    {
    ix = *((data->mols+i)->atom+n) = *(mol_list+k+n);
    (data->mols+i)->centroid[0] += (data->atoms+ix)->x;
    (data->mols+i)->centroid[1] += (data->atoms+ix)->y;
    (data->mols+i)->centroid[2] += (data->atoms+ix)->z;
/* NEW - record assoc. molecule */
    (data->atoms+ix)->molecule = i;
    }
/* calc centroid */
  if (j-k)
    {
    (data->mols+i)->centroid[0] /= (j-k);
    (data->mols+i)->centroid[1] /= (j-k);
    (data->mols+i)->centroid[2] /= (j-k);
    }
  }

/* done */
g_free(mol_list);
g_free(mol_start);

#if DEBUG_MOLS
printf("Found %d molecules\n",data->num_mols);
for (i=0 ; i<data->num_mols ; i++)
  {
  printf("[%d]  ",i);
  for (j=0 ; j<(data->mols+i)->num_atoms ; j++)
    printf("%d ",*((data->mols+i)->atom+j));
  printf("\n");
  }
#endif

return;
}

/**********************/
/* ROTATE A SELECTION */
/**********************/
void rotate_select(gfloat *mat, struct model_pak *data)
{
gint i, atom, shel;
gfloat scent[3], vec[3], invr[9], invl[9];

/* TODO - include this ability for fractional coords */
/* calculate selection's centroid */
VEC3SET(scent, 0.0, 0.0, 0.0);
for (i=0 ; i<data->select_size ; i++)
  {
/* NEW - rotate the *cartesian* version of the coords (back transform after) */
  atom = *(data->select_list+i);
  scent[0] += (data->atoms+atom)->rx;
  scent[1] += (data->atoms+atom)->ry;
  scent[2] += (data->atoms+atom)->rz;
  }
scent[0] /= data->select_size;
scent[1] /= data->select_size;
scent[2] /= data->select_size;

/* get inv of rotation and lattice matrices */
for (i=0 ; i<9 ; i++)
  invl[i] = data->latmat[i];
invmat(invl, 3);
ARR3SET(&invr[0], &data->rotmat[0]);
ARR3SET(&invr[3], &data->rotmat[3]);
ARR3SET(&invr[6], &data->rotmat[6]);
invmat(invr, 3);

for (i=0 ; i<data->select_size ; i++)
  {
  atom = *(data->select_list+i);
  vec[0] = (data->atoms+atom)->rx - scent[0];
  vec[1] = (data->atoms+atom)->ry - scent[1];
  vec[2] = (data->atoms+atom)->rz - scent[2];

/* rotate to get positions */
  vecmat(mat,vec);
  ARR3ADD(vec, scent);

/* CURRENT - non cartesian back transform */
  vecmat(invr, vec);
  vecmat(invl, vec);
  ARR3ADD(vec, data->centroid);

  (data->atoms+atom)->x = vec[0];
  (data->atoms+atom)->y = vec[1];
  (data->atoms+atom)->z = vec[2];

/* shells */
  if ((data->atoms+atom)->has_shell) 
    {
    shel = (data->atoms+atom)->idx_shell;
    vec[0] = (data->shells+shel)->x - scent[0];
    vec[1] = (data->shells+shel)->y - scent[1];
    vec[2] = (data->shells+shel)->z - scent[2];
/* rotate to get on screen positions, and store the values */
    vecmat(mat,vec);
    ARR3ADD(vec, scent);

/* CURRENT - non cartesian back transform */
    vecmat(invr, vec);
    vecmat(invl, vec);
    ARR3ADD(vec, data->centroid);

    (data->shells+shel)->x = vec[0];
    (data->shells+shel)->y = vec[1];
    (data->shells+shel)->z = vec[2];
    }
  }

/* recalc connectivity */
calc_bonds(data);
calc_mols(data);
}

/**********************/
/* CALC SCREEN COORDS */
/**********************/
#define DEBUG_UPDATE_COORDS 0
void update_coords(gint mode, struct model_pak *data)
{
gint i,j,n;
gint ix[3];
gfloat cosa, sina, scale;
gfloat rot[9], g[3], vec[3];

#if DEBUG_UPDATE_COORDS
printf("update_coords() start.\n");
#endif

/* precalc sine & cosine */
cosa = cos(data->da);
sina = sin(data->da);

/* dependant matrix elements */
switch(mode)
  {
  case ROTATE:
    VEC3SET(&rot[0], cosa, 0.0,-sina);
    VEC3SET(&rot[3],  0.0, 1.0,  0.0);
    VEC3SET(&rot[6], sina, 0.0, cosa);
    break;
  case PITCH:
    VEC3SET(&rot[0], 1.0,  0.0, 0.0);
    VEC3SET(&rot[3], 0.0, cosa,-sina);
    VEC3SET(&rot[6], 0.0, sina, cosa);
    break;
  case ROLL:
    VEC3SET(&rot[0], cosa,-sina, 0.0);
    VEC3SET(&rot[3], sina, cosa, 0.0);
    VEC3SET(&rot[6],  0.0,  0.0, 1.0);
    break;
  case INITIAL:
/* called once to set up from scratch, plus non-90 stuff */
    VEC3SET(&data->rotmat[0], 1.0, 0.0, 0.0);
    VEC3SET(&data->rotmat[3], 0.0, 1.0, 0.0);
    VEC3SET(&data->rotmat[6], 0.0, 0.0, 1.0);
/* create lattice translation vectors */
    make_xlat(data);
/* all done - fall through to make rot[][] the identity */
/* FIXME - is this the best place for this??? */
    data->num_orig = data->num_atoms;
  case REFRESH:
/* do nothing but calculate pixel positions */
    VEC3SET(&rot[0], 1.0, 0.0, 0.0);
    VEC3SET(&rot[3], 0.0, 1.0, 0.0);
    VEC3SET(&rot[6], 0.0, 0.0, 1.0);
    break;
  default:
    printf("Unknown mode!\n");
    return;
  }
#if DEBUG_UPDATE_COORDS
P3MAT("rot matrix", rot);
#endif

/* selection only rotatation */
if (sysenv.mode[sysenv.active] == ALTER_SELECT)
  rotate_select(rot, data); 
/* update cummulative rotation matrix */
else 
  matmat(rot,data->rotmat);

/* precalc scale */
scale = data->scale * sysenv.subscale * 0.5 / data->rmax;

/* update periodic image vectors */
/*
if (data->periodic)
*/
for (n=data->periodic ; n-- ; )
  {
/* setup appropriate pbc */
  VEC3SET(vec, 0.0, 0.0, 0.0);
  vec[n] = data->pbc[n];
/* apply lattice vectors */
  vecmat(data->latmat,vec); 
/* apply rotation */
  vecmat(data->rotmat,vec);
/* save */
/* vectors in columns (not rows) */
  for (i=3 ; i-- ; )
    data->piv_real[3*i+n] = vec[i];
/* convert to pix */
 VEC2MUL(vec,scale);
/* TODO - use z coord for perspective view? */
/* save pixel coordinates for on screen selection AND drawing bonds */
  data->piv_pix[2*n] = (gint) vec[0];
  data->piv_pix[2*n+1] = (gint) vec[1];
  }

/* Rotate atoms */
for (n=data->num_atoms ; n-- ; )
  {
  if ((data->atoms+n)->status != DELETED && (data->atoms+n)->status != HIDDEN)
    {
/* get coordinates */
    VEC3SET(vec, (data->atoms+n)->x,(data->atoms+n)->y,(data->atoms+n)->z);
/* CURRENT - center after lattice conversion */
    ARR3SUB(vec, data->centroid);
/* apply lattice vectors */
    vecmat(data->latmat,vec); 
/* apply rotation */
    vecmat(data->rotmat,vec);
/* save rotated coords */
    (data->atoms+n)->rx = vec[0];
    (data->atoms+n)->ry = vec[1];
    (data->atoms+n)->rz = vec[2];
/* scaling */
    VEC2MUL(vec,scale);
/* TODO - use z coord for perspective view? */
/* save pixel coordinates for on screen selection AND drawing bonds */
    (data->atoms+n)->px = (gint) vec[0];
    (data->atoms+n)->py = (gint) vec[1];
    }
  }

/* rotate shells (if any ) */
for (n=data->num_shells ; n-- ; )
  {
  if ((data->shells+n)->status != DELETED && (data->shells+n)->status != HIDDEN)
    {
/* get coordinates */
    VEC3SET(vec, (data->shells+n)->x,(data->shells+n)->y,(data->shells+n)->z);
/* CURRENT - center after lattice conversion */
    ARR3SUB(vec, data->centroid);
/* apply lattice vectors */
    vecmat(data->latmat,vec);
/* rotation */
    vecmat(data->rotmat,&vec[0]);
/* save rotated coords */
    (data->shells+n)->rx = vec[0];
    (data->shells+n)->ry = vec[1];
    (data->shells+n)->rz = vec[2];
/* scaling */
    VEC2MUL(vec,scale);
/* TODO - use z coord for perspective view? */
/* save pixel coordinates for on screen selection AND drawing bonds */
    (data->shells+n)->px = (gint) vec[0];
    (data->shells+n)->py = (gint) vec[1];
    }
  }

/* rotate special objects */

/********/
/* CELL */
/********/
for (n=8 ; n-- ; )
  {
/* get coordinates */
  VEC3SET(vec, data->cell[n].x, data->cell[n].y, data->cell[n].z);
/* CURRENT - center after lattice conversion */
  ARR3SUB(vec, data->centroid);
/* apply translation vectors */
  vecmat(data->latmat,vec);
/* rotation */
  vecmat(data->rotmat,vec);
/* save rotated coords */
  data->cell[n].rx = vec[0];
  data->cell[n].ry = vec[1];
  data->cell[n].rz = vec[2];
/* scaling */
  VEC2MUL(vec,scale);
/* save pixel coordinates for on screen selection AND drawing bonds */
/* generate pixel coordinates */
  data->cell[n].px = (gint) vec[0];
  data->cell[n].py = (gint) vec[1];
  }

/********/
/* AXES */
/********/
/* FIXME - axes orientations won't change during animation */
/* don't do these to the axes */
  for (n=6 ; n-- ; )
    {
/* do cartesian axes */
    vec[0] = data->axes[n].x;
    vec[1] = data->axes[n].y;
    vec[2] = data->axes[n].z;
/* rotation */
    vecmat(data->rotmat,vec);
/* save rotated coords for rendering */
    data->axes[n].rx = vec[0];
    data->axes[n].ry = vec[1];
    data->axes[n].rz = vec[2];
/* save pixel coordinates for on screen selection AND drawing bonds */
    data->axes[n].px = (gint) vec[0];
    data->axes[n].py = (gint) vec[1];
/* NEW - do pbc axes */
    vec[0] = data->axes[n].x;
    vec[1] = data->axes[n].y;
    vec[2] = data->axes[n].z;
/* apply translation vectors */
    vecmat(data->latmat,vec);
/* rotation */
    vecmat(data->rotmat,vec);
/* save pixel coordinates for on screen selection AND drawing bonds */
    data->xlat[n].px = (gint) vec[0];
    data->xlat[n].py = (gint) vec[1];
    }

/*****************/
/* CREATION MESH */
/*****************/
/* only doing highlighting here */
if (data->mesh_on)
  {
  for (n=0 ; n<data->num_atoms ; n++)
    {
/* not concerned with deleted atoms */
    if ((data->atoms+n)->status == DELETED || (data->atoms+n)->status == HIDDEN)
      continue;
/* if inactive, atoms defaults to normal */
    (data->atoms+n)->status = NORMAL;
/* cutoff too arbitrary? */
    if (fabs((data->atoms+n)->rz) < 0.1)
      (data->atoms+n)->status = SLITE;
    }
  }

/******************/
/* MYSURF updates */
/******************/
if (data->mysurf[SOLSURF].num_points)
  {
  for (i=0 ; i<data->mysurf[SOLSURF].num_points ; i++)
    {
    vec[0] = data->mysurf[SOLSURF].x[i];
    vec[1] = data->mysurf[SOLSURF].y[i];
    vec[2] = data->mysurf[SOLSURF].z[i];
/* centering */
    ARR3SUB(vec, data->centroid);
/* apply translation vectors */
    vecmat(data->latmat,vec); 
/* apply rotation */
    vecmat(data->rotmat,vec);
/* scaling */
    VEC2MUL(vec, scale);
/* save pixel coordinates for on screen selection AND drawing bonds */
    data->mysurf[SOLSURF].px[i] = (gint) vec[0];
    data->mysurf[SOLSURF].py[i] = (gint) vec[1];
    }
  }
if (data->mysurf[MOLSURF].num_points)
  {
  for (i=0 ; i<data->mysurf[MOLSURF].num_points ; i++)
    {
    vec[0] = data->mysurf[MOLSURF].x[i];
    vec[1] = data->mysurf[MOLSURF].y[i];
    vec[2] = data->mysurf[MOLSURF].z[i];
/* centering */
    ARR3SUB(vec, data->centroid);
/* apply translation vectors */
    vecmat(data->latmat,vec); 
/* apply rotation */
    vecmat(data->rotmat,vec);
/* scaling */
    VEC2MUL(vec, scale);
/* save pixel coordinates for on screen selection AND drawing bonds */
    data->mysurf[MOLSURF].px[i] = (gint) vec[0];
    data->mysurf[MOLSURF].py[i] = (gint) vec[1];
    }
  }

/***************/
/* VDW SURFACE */
/***************/
if (data->csurf_on)
  {
  n=0;
  ARR3SET(ix, data->csurf.ix);
  for (i=0 ; i<data->csurf.grid ; i++)
    {
    for (j=0 ; j<data->csurf.grid ; j++)
      {
/* get coords of surface (+ offset to 'height') */
      g[0] = (gfloat) i*data->csurf.step[ix[0]];
      g[1] = (gfloat) j*data->csurf.step[ix[1]];
      g[2] = 0.0;
      grid2cart(g,vec);
      vec[0] += data->csurf.min[ix[0]];
      vec[1] += data->csurf.min[ix[1]];
      vec[2] = *(data->csurf.h+n) + data->csurf.offset;
      ARR3SUB(vec, data->centroid);
/* apply translation vectors */
      vecmat(data->latmat,vec); 
/* apply rotation */
      vecmat(data->rotmat,vec);
/* save rotated coords */
      *(data->csurf.rx+n) = vec[0];
      *(data->csurf.ry+n) = vec[1];
      *(data->csurf.rz+n) = vec[2];
/* scaling */
      VEC2MUL(vec, scale);
/* save pixel coordinates for on screen selection AND drawing bonds */
      *(data->csurf.px+n) = (gint) vec[0];
      *(data->csurf.py+n) = (gint) vec[1];
      n++;
      }
    }
  }

/***************/
/* MORPH MODEL */
/***************/
  for (n=data->num_vertices ; n-- ; )
    {
/* do cartesian axes */
    VEC3SET(vec, (data->vertices+n)->x, (data->vertices+n)->y,
                                        (data->vertices+n)->z);
/* apply translation vectors */
    vecmat(data->latmat,vec);
/* CURRENT - center after lattice conversion */
    ARR3SUB(vec, data->centroid);
/* rotation */
    vecmat(data->rotmat,vec);
/* save (for povray) */
    (data->vertices+n)->rx = vec[0];
    (data->vertices+n)->ry = vec[1];
    (data->vertices+n)->rz = vec[2];
/* scaling */
    VEC2MUL(vec, scale);
/* save pixel coordinates for on screen drawing */
    (data->vertices+n)->px = (gint) vec[0];
    (data->vertices+n)->py = (gint) vec[1];
    }
/* NEW - facet pixel center calc. */
  for (n=data->num_facets ; n-- ; )
    {
/* set visible flag */
    if (facet_visible(data,n))
      (data->facets+n)->visible = TRUE;
    else
      (data->facets+n)->visible = FALSE;
/* pixel coords of center */
    vec[0] = vec[1] = vec[2] = 0.0;
    g[0] = g[1] = 0.0;
    for (i=0 ; i<(data->facets+n)->num_points ; i++)
      {
      j = *((data->facets+n)->points+i);
/* scan all facets connected to vertex */
/* NEW - screen pos and real pos (for povray) */
      vec[0] += (data->vertices+j)->rx;
      vec[1] += (data->vertices+j)->ry;
      vec[2] += (data->vertices+j)->rz;
      g[0] += (gfloat) (data->vertices+j)->px;
      g[1] += (gfloat) (data->vertices+j)->py;
      }
/* average the summation */
    vec[0] /= (data->facets+n)->num_points;
    vec[1] /= (data->facets+n)->num_points;
    vec[2] /= (data->facets+n)->num_points;
    g[0] /= (data->facets+n)->num_points;
    g[1] /= (data->facets+n)->num_points;
/* assign */
    (data->facets+n)->rx = vec[0];
    (data->facets+n)->ry = vec[1];
    (data->facets+n)->rz = vec[2];
    (data->facets+n)->px = (gint) g[0];
    (data->facets+n)->py = (gint) g[1];
    }

/* NEW - update printed coord of selected atom in atom info mode */
if (sysenv.mode[data->number] == ATOM_INFO)
  if ((data->atoms+data->atom_info)->status == SLITE)
    print_atom(data, data->atom_info);

#if DEBUG_UPDATE_COORDS
printf("update_coords() done.\n");
#endif
}

/*********************************/
/* unified atom location routine */
/*********************************/
/* NB: should always return -1 if no atom was found */
gint seek_atom(gint px, gint py, struct model_pak *data)
{
gint i, j, dx, dy, dr2;

/* get the active subwin */
j=-1;
for (i=0 ; i<sysenv.num_displayed ; i++)
  if (sysenv.displayed[i] == sysenv.active)
    j=i;

/* failsafe */
if (j < 0)
  {
  printf("seek_atom() error: invalid subwin.\n");
  return(-1);
  }

/* NB: assume subwin is always the active one */
px -= sysenv.subcenx[j];
py -= sysenv.subceny[j];

for (i=0 ; i<data->num_atoms ; i++)
  {
  if ((data->atoms+i)->status != DELETED && (data->atoms+i)->status != HIDDEN)
    {
    dx = ((data->atoms+i)->px - px) + data->offset[0]; 
    dy = ((data->atoms+i)->py - py) + data->offset[1]; 
    dr2 = dx*dx + dy*dy;
/* FIXME - cutoff too arbitrary? */
    if (dr2 < 36)
      return(i);
    }
  }

/* failed to find an atom at px,py */
return(-1);
}

/***************************************/
/* unified pixel to coordinate routine */
/***************************************/
#define DEBUG_PCM 0
void pixel_coord_map(gint px, gint py, struct model_pak *data, gint atom)
{
gint i, model;
gfloat r[3], inv[9];

/* this could lead to trouble */
g_return_if_fail(sysenv.active == data->number);

/* find active subwin */
i=0;
while (i<sysenv.num_displayed)
  {
  if (sysenv.displayed[i] == sysenv.active)
     break;
  i++;
  }

/* the model should be displayed for this to make sense */
g_return_if_fail(i != sysenv.num_displayed);
model = sysenv.displayed[i];
g_return_if_fail(model == sysenv.active);

/* hack for selection altering */
if (sysenv.mode[model] != ALTER_SELECT)
  {
#if DEBUG_PCM
printf("(*) ");
#endif
/* remove subwin centre */
  px -= sysenv.subcenx[i];
  py -= sysenv.subceny[i];
/* account for user translation */
  px -= data->offset[0];
  py -= data->offset[1];
  }

#if DEBUG_PCM
printf("center [%d,%d] : offset [%d,%d] : pixel pos [%d,%d]\n",
  sysenv.subcenx[i],sysenv.subceny[i],data->offset[0],data->offset[1],px,py);
#endif

/* we know this for sure :-) */
(data->atoms+atom)->px = px;
(data->atoms+atom)->py = py;

/* convert pixel coords to screen values */
r[0] = (gfloat) px; 
r[1] = (gfloat) py; 
/* go up */
r[0] *= 2.0*data->rmax;
r[1] *= 2.0*data->rmax;
/* go down */
r[0] /= sysenv.subscale;
r[1] /= sysenv.subscale;
r[0] /= data->scale;
r[1] /= data->scale;
/* leave z where it is, make sure it's defined!!! */
r[2] = (data->atoms+atom)->rz;
/* assign the rotated vectors */
(data->atoms+atom)->rx = r[0];
(data->atoms+atom)->ry = r[1];

/* mult by inverse rotation matrix */
ARR3SET(&inv[0], &data->rotmat[0]);
ARR3SET(&inv[3], &data->rotmat[3]);
ARR3SET(&inv[6], &data->rotmat[6]);
invmat(inv,3);
vecmat(inv,r);

/* mult by inverse lattice matrix */
if (data->periodic)
  {
  for (i=0 ; i<9 ; i++)
    inv[i] = data->latmat[i];
  invmat(inv,3);
  vecmat(inv,r);
  }

/* generate and assign the unrotated vectors */
(data->atoms+atom)->x = data->centroid[0] + r[0];
(data->atoms+atom)->y = data->centroid[1] + r[1];
(data->atoms+atom)->z = data->centroid[2] + r[2];

/* NEW - update fractional coords */
if (data->periodic)
  {
  (data->atoms+atom)->fx = (data->atoms+atom)->x / data->pbc[0];
  (data->atoms+atom)->fy = (data->atoms+atom)->y / data->pbc[1];
  if (data->periodic == 3)
    (data->atoms+atom)->fz = (data->atoms+atom)->z / data->pbc[2];
  else
    (data->atoms+atom)->fz = (data->atoms+atom)->z;
  }
}

/***********************************/
/* print atom coord data primitive */
/***********************************/
void print_atom(struct model_pak *data, gint i)
{
gchar txt[90];

/* check */
if (i < 0 || i > data->num_atoms)
  return;

/* make string */
if (data->fractional)
  {
  g_snprintf(txt,77,
  "[%03d] %2s at (%7.2f,%7.2f,%7.2f), originally (%7.4f,%7.4f,%7.4f).",
           i,
           (data->atoms+i)->element,
           data->centroid[0] + (data->atoms+i)->rx,
           data->centroid[1] + (data->atoms+i)->ry,
           data->centroid[2] + (data->atoms+i)->rz,
           (data->atoms+i)->fx,
           (data->atoms+i)->fy,
           (data->atoms+i)->fz);
  }
else
  {
  g_snprintf(txt,77,
  "[%03d] %2s at (%7.2f,%7.2f,%7.2f), originally (%7.4f,%7.4f,%7.4f).",
           i,
           (data->atoms+i)->element,
           data->centroid[0] + (data->atoms+i)->rx,
           data->centroid[1] + (data->atoms+i)->ry,
           data->centroid[2] + (data->atoms+i)->rz,
           (data->atoms+i)->x,
           (data->atoms+i)->y,
           (data->atoms+i)->z);
  }

/* done - print it */
show_text(txt);
}

/*****************/
/* info on atoms */
/*****************/
void info_atom(gint x, gint y, struct model_pak *data)
{
gint j;
static gint i=-1;

j = seek_atom(x, y, data);
if (j >= 0)
  {
  print_atom(data, j);
/* remove highlighting of old atom */
/* FIXME - properly cleaned (ie =-1) when mode switching??? */
  if (i>=0 && i<data->num_atoms)
    (data->atoms+i)->status = NORMAL;
/* highlight new */
  (data->atoms+j)->status = SLITE;
/* NEW - save for continuous update */
  data->atom_info = j;
  i=j;
  }

/* update - ie show the new highlighting */
wire_draw(SINGLE);
}

