/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <math.h>
#include <stdio.h>
#include <string.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <sys/param.h>
#ifdef __sun
#include <sys/dirent.h>
#else
#include <sys/dir.h>
#endif

#include "gdis.h"

#define DEBUG_MORE 0

/* main structures */
extern struct sysenv_pak sysenv;
extern struct elem_pak elements[ATOMDATA];

#ifdef __sun
#include "scandir.c"
#endif

/**************************************/
/* get a non trivial line from a file */
/**************************************/
gint fgetline(FILE *fp, gchar *line)
{
gint linlen;

for(;;)
  {
/* get a line */
  if (fgets(line, LINELEN, fp) == NULL)
    return(1);
  linlen = strlen(line);
/* comment/blank line skipping */
  if (line[0] != '#' && linlen)
    break;
  }

/* all clear */
return(0);
}

/*********************/
/* write a GULP file */
/*********************/
#define DEBUG_WRITE_GULP 0
gint write_gulp(gchar *file, struct model_pak *data)
{
gint i, j, maxcyc;
gchar *cmd, *tail, *libfile, *name, fake_tail[] = "";
gfloat vec[3];
GString *line;
FILE *fp;

g_return_val_if_fail(data != NULL, 1);

/* test for library existance */
libfile = data->gulp.libfile;
if (!access(libfile, R_OK))
  {
#if DEBUG_WRITE_GULP
printf("Found: %s ",libfile);
#endif
  name = g_strdup(g_basename(libfile));
/* was it in cwd or somewhere else? */
  if (!access(name, R_OK))
    {
#if DEBUG_WRITE_GULP
printf("in cwd.\n");
#endif
    }
  else
    {
/* elsewhere - copy it to cwd */
#if DEBUG_WRITE_GULP
printf("elsewhere.\n");
#endif
    cmd = g_strdup_printf("cp %s %s",libfile,name);
    system(cmd);
    g_free(cmd);
    }
  }
else
  {
/* missing library - copy default to cwd */
  printf("Warning: %s not found, using default library.\n", libfile);
  name = g_strdup(LIBRARY);
  cmd = g_strdup_printf("cp %s/%s .",INSTALL,LIBRARY);
  system(cmd);
  g_free(cmd);
  }

#if DEBUG_WRITE_GULP
printf("Library: %s\n",name);
#endif

/* off we go */
fp = fopen(file,"wt");

/* NEW - user define choice of run type */
line = g_string_new(NULL);
switch(data->gulp.run)
  {
  case ENERGY:
    g_string_assign(line,"single");
    break;
  case MINIMIZE:
    g_string_assign(line,"opti");
    break;
  default:
    printf("Warning: bad GULP init.\n");
    g_string_assign(line,"single");
  }

switch(data->gulp.constant)
  {
  case PRESSURE:
    g_string_append(line," conp");
    break;
  case VOLUME:
    g_string_append(line," conv");
    break;
  default:
    printf("Warning: bad GULP init.\n");
    g_string_assign(line,"conp");
  }

switch(data->gulp.coulomb)
  {
  case MOLE:
    g_string_append(line," mole");
    break;
  case MOLMEC:
    g_string_append(line," molmec");
    break;
  case MOLQ:
    g_string_append(line," molq");
    break;
  default:
    printf("Warning: bad GULP init.\n");
    g_string_assign(line,"mole");
  }
fprintf(fp,"%s\n\n",line->str);

maxcyc = data->gulp.maxcyc;
fprintf(fp,"maxcyc %d\n\n",maxcyc);

if (!data->gulp.num_potentials)
  fprintf(fp,"library %s\n\n",name);
/* periodic/isolated */
fprintf(fp,"name %s\n",data->basename);
/*
if (data->periodic == 3)
*/
if (data->periodic > 1)
  {
  fprintf(fp,"cell\n");
  fprintf(fp,"%f %f %f  %f %f %f\n",data->pbc[0],data->pbc[1],data->pbc[2],
              data->pbc[3]*180/PI, data->pbc[4]*180/PI, data->pbc[5]*180/PI);
  }
/* new - some models may be 2D periodic & fractional (eg 2D xtl) */
/* which will cause problems with z & 0 pbc -> save as cartesian */
if (data->fractional && data->periodic == 3)
  fprintf(fp,"fractional\n");
else
  fprintf(fp,"cartesian\n");

/* write coords */
for (i=0 ; i<data->num_atoms ; i++)
  {
  if ((data->atoms+i)->status == DELETED || (data->atoms+i)->status == HIDDEN)
    continue;

/* only print tail data if GULP style, not (eg) BIOSYM */
  if (data->id != GULP)
    tail = fake_tail;
  else
    tail = (data->atoms+i)->tail;

/* periodic/isolated */
  if (data->fractional)
    {
    if (!(data->atoms+i)->primary)
      continue;
/* NEW - handle the saving of 2D fractional model */
    switch (data->periodic)
      {
      case 3:
        fprintf(fp,"%-4s core %8.5f %8.5f %8.5f %s\n",
                                      (data->atoms+i)->element,
                                      (data->atoms+i)->fx,
                                      (data->atoms+i)->fy,
                                      (data->atoms+i)->fz,
                                      tail);
        break;
      case 2:
/* get coordinates */
        VEC3SET(vec, (data->atoms+i)->x,(data->atoms+i)->y,(data->atoms+i)->z);
/* apply lattice vectors (converting frac -> cart) */
        vecmat(data->latmat,vec); 
        fprintf(fp,"%-4s core %10.5f %10.5f %10.5f %s\n",
                                      (data->atoms+i)->element,
                                      vec[0], vec[1], vec[2],
                                      tail);
        break;
      default:
        printf("save_gulp() error, bad fractional model.\n");
      }

    if ((data->atoms+i)->has_shell)
      {
      j = (data->atoms+i)->idx_shell;
      if (data->id != GULP)
        tail = fake_tail;
      else
        tail = (data->shells+j)->tail;
      fprintf(fp,"%-4s shel %8.5f %8.5f %8.5f %s\n",
                                        (data->shells+j)->element,
                                        (data->shells+j)->fx,
                                        (data->shells+j)->fy,
                                        (data->shells+j)->fz,
                                        tail);
      } 
    }
  else
    {
    fprintf(fp,"%-4s core %10.5f %10.5f %10.5f %s\n",
                                        (data->atoms+i)->element,
                                        (data->atoms+i)->x,
                                        (data->atoms+i)->y,
                                        (data->atoms+i)->z,
                                        tail);
    if ((data->atoms+i)->has_shell)
      {
      j = (data->atoms+i)->idx_shell;
      if (data->id != GULP)
        tail = fake_tail;
      else
        tail = (data->shells+j)->tail;
      fprintf(fp,"%-4s shel %10.5f %10.5f %10.5f %s\n",
                                          (data->shells+j)->element,
                                          (data->shells+j)->x,
                                          (data->shells+j)->y,
                                          (data->shells+j)->z,
                                          tail);
      } 
    }
  }

/* makes it easier for GDIS to read back in */
fprintf(fp,"\n\n");

/* periodic/isolated */
if (data->periodic > 1)
  {
  fprintf(fp,"space\n%s\n\n",g_strstrip(data->sginfo.spacename));
/* this allows saving of asym cell, but is a pain when you want */
/* to run a gulp job WITH the proper sg name */
/*
  if (data->asym_on)
    fprintf(fp,"space\n%s\n\n",g_strstrip(data->sginfo.spacename));
  else
    fprintf(fp,"space\nP 1\n\n");
*/
  }

/* extra data */
if (data->gulp.num_elem)
  {
  fprintf(fp,"elements\n");
  for (i=0 ; i<data->gulp.num_elem ; i++)
    fprintf(fp,"%s\n",*(data->gulp.elem+i));
  fprintf(fp,"end\n\n");
  }
if (data->gulp.num_species)
  {
  fprintf(fp,"species\n");
  for (i=0 ; i<data->gulp.num_species ; i++)
    fprintf(fp,"%s\n",*(data->gulp.species+i));
  fprintf(fp,"end\n\n");
  }
for (i=0 ; i<data->gulp.num_potentials ; i++)
  {
  fprintf(fp,"%s\n",*(data->gulp.pot1+i));
  fprintf(fp,"%s\n",*(data->gulp.pot2+i));
  }

fprintf(fp,"\nprint 1\n");

/* done */
fclose(fp);
g_free(name);
g_string_free(line, TRUE);
return(0);
}

/**********************************/
/* open and configure a data file */
/**********************************/
gint file_action(gchar *inp_file, gint file_mode, gint model)
{
gint id;
struct model_pak *data;

id = model_type(inp_file);
if (id < 0)
  return(1);

/* setup appropriate file pointer/stream */
switch (file_mode)
  {
  case FILE_SAVE_AS:
    data = model_ptr(model, RECALL);
    g_return_val_if_fail(data != NULL,1);
/* overwrite the old filenames with the new */
    if (data)
      {
      strcpy(data->filename, inp_file);
      g_free(data->basename);
      data->basename = strdup_no_extension(g_basename(inp_file));
      }

  case FILE_SAVE:
/* checks */
    data = model_ptr(model, RECALL);
    g_return_val_if_fail(data != NULL,1);
/* special saves/supported types only */
    switch(id)
      {
      case GULP:
        if (write_gulp(inp_file, data))
          {
          show_text("Save failed.");    
          return(1);
          }
        break;
      case MARVIN:
        save_marvin(inp_file, data);
        break;
      case BIOSYM:
      case XYZ:
        if (create_file(inp_file, id, data))
          {
          show_text("Save failed.");    
          return(1);
          }
        show_text("File saved.");    
        break;
      default:
        show_text("Unsupported file type.");    
      }
    break;
  default:
    show_text("Unknown file mode.");
    return(1);
  }

/* updates */
new_tree_item(model, REPLACE);
wire_draw(SINGLE);

return(0);
}

/*****************************/
/* Setup model for file read */
/*****************************/
#define DEBUG_LOAD_MODEL 0
gint load_model(gint model, gint type, gchar *inp_file)
{
gint i, status=0;
gchar line[LINELEN];
struct model_pak *data;
struct parse_pak parse_data;
FILE *fp;

/* can we open the file */
fp = fopen(inp_file,"r");
if (!fp)
  {
  show_text("Error: bad or missing filename.");
  return(1);
  }
/* memory assignment for the model (and initial assignment for its data) */
data = model_ptr(model, ASSIGN);
if (!data)
  return(1);
data->id = type;

/* get the appropriate parsing structure */
if (parse_ptr(&parse_data, type))
  {
  show_text("Error: unknown parse type requested.\n");
  sysenv.num_models--;
  return(1);
  }

/* TODO - all unique? */
status = load_data(fp, &parse_data, data);

/* FIXME - messy hack */
/* count the frames in BIOSYM files */
if (data->id == BIOSYM)
  {
/* already read in one frame */
  i=1;
  while(!fgetline(fp,line))
    if (g_strncasecmp("!DATE", line, 5) == 0)
      i++;
  data->num_frames = i;
  }
#if DEBUG_LOAD_MODEL
if (data->num_frames)
  printf("Found %d frames.\n",data->num_frames);
#endif

if (status)
  {
  fclose(fp);
#if DEBUG_LOAD_MODEL
  printf("Load failed, error code: %d\n",status);
#endif
/* FIXME - dodgy cleanup if status=6 ??? */
  model_ptr(model, RELEASE);
  display_shuffle();
  wire_draw(ALL);
  return(1);
  }

/* save details */
strcpy(data->filename, inp_file);
g_free(data->basename);
data->basename = strdup_no_extension(g_basename(inp_file));

/* configure for displaying */
sysenv.mode[model] = FREE;
sysenv.active = model;
init_objs(AXES_ON, data);
/* NB: CELL must be initialized b4 coords, for centering purposes */
/* non 90 axes shifting is done by update_coords(INITIAL) */
if (data->periodic)
  {
  init_objs(CELL_ON, data);
  data->axes_type = OTHER;
/* space group init here */
  update_shells(data);
  if (!data->sginfo.spacenum)
    {
#if DEBUG_LOAD_MODEL
printf("No Space Group found: assuming P1.\n");
#endif
    data->sginfo.spacenum=1;
    }
  if (genpos(data))
    printf("Error in Space Group lookup.\n");
  }

init_objs(INIT_COORDS, data);
calc_bonds(data);
calc_mols(data);

/* update any existing dialogue boxes */
pick_model(model);
new_tree_item(model, APPEND);

fclose (fp);
return(0);
}

/*************************/
/* gulp input file parse */
/*************************/
#define DEBUG_LOAD_GULP 0
gint load_gulp(FILE *fp, gint model, gchar *inp_file)
{
gint i,j;
gint current,data_current,completed, num_potentials, num_species, num_elem;
gint data_start, space_start, species_start, elem_start, type;
gint flag, auflag, data_flag, space_flag, cell_flag, species_flag, elem_flag;
gint origin;
gchar line[LINELEN], *tmp, **key, **buff;
gchar *pot1[MAX_POTENTIALS], *pot2[MAX_POTENTIALS];
gchar *species[MAX_POTENTIALS], *elem[MAX_POTENTIALS];
struct model_pak *data, *ptr;

/* setup */
current = model;
data_current = 0;
num_potentials = 0;
num_species = 0;
num_elem = 0;
rewind(fp);
/* initial read to start the do ... while loop off */
if (fgetline(fp,line))
  return(1);

/* main loop for getting all listed structures */
do
  {
  auflag = data_flag = space_flag = cell_flag = species_flag = elem_flag = 0;
  data = model_ptr(current, ASSIGN);
  if (data == NULL)
    return(2);
  data->id = GULP;
  g_free(data->basename);
  data->basename = g_strdup_printf("structure %-2d",current-model+1);
  g_snprintf(data->filename,13,"structure_%-2d",current-model+1);

/* do ... while more lines are available */
  completed=0;
  do
    {
/* keyword/option search */
    data_start = space_start = species_start = elem_start = 0;
    key = get_tokens(line, 2);

/* NEW - take default run type from keywords */
/* FIXME - single/opti must be the 1st or 2nd item for this to work */
    if (g_strncasecmp("single", *(key+0), 6) == 0 || 
        g_strncasecmp("single", *(key+1), 6) == 0 || 
        g_strncasecmp("opti",   *(key+0), 4) == 0 || 
        g_strncasecmp("opti",   *(key+1), 4) == 0)
      {
      buff = get_tokens(line, 6);
      for (i=0 ; i<6 ; i++)
        {
/* currently recognized/utilized keywords */
        if (g_strncasecmp("single",*(buff+i),6) == 0) 
          data->gulp.run = ENERGY;
        if (g_strncasecmp("opti",*(buff+i),4) == 0) 
          data->gulp.run = MINIMIZE;
        if (g_strncasecmp("conp",*(buff+i),4) == 0) 
          data->gulp.constant = PRESSURE;
        if (g_strncasecmp("conv",*(buff+i),4) == 0) 
          data->gulp.constant = VOLUME;
        if (g_strncasecmp("mole",*(buff+i),4) == 0) 
          data->gulp.coulomb = MOLE;
        if (g_strncasecmp("molmec",*(buff+i),4) == 0) 
          data->gulp.coulomb = MOLMEC;
        if (g_strncasecmp("molq",*(buff+i),4) == 0) 
          data->gulp.coulomb = MOLQ;
        }
      g_strfreev(buff);
      }
/* optimization cycles */
/* FIXME - check? */
    if (g_strncasecmp("maxcyc", *key, 6) == 0) 
      sscanf(*(key+1),"%d",&(data->gulp.maxcyc));

/* get name - if any */
    if (g_strncasecmp("name",*key,4) == 0)
      {
/* copy_items(..ALL) replacement */
      g_free(data->basename);
      data->basename = g_strstrip(g_strdup(get_token_pos(line,1)));
      strcpy(data->filename, data->basename);
      }

/* pbc */
    if (g_strncasecmp("cell",*key,4) == 0 && !cell_flag)
      {
/* found it - get next line (if possible) */
      if (fgetline(fp,line))
        return(3);
      buff = get_tokens(line, 6);
      sscanf(*(buff+0),"%f",&(data->pbc[0]));
      sscanf(*(buff+1),"%f",&(data->pbc[1]));
      sscanf(*(buff+2),"%f",&(data->pbc[2]));
      sscanf(*(buff+3),"%f",&(data->pbc[3]));
      sscanf(*(buff+4),"%f",&(data->pbc[4]));
      sscanf(*(buff+5),"%f",&(data->pbc[5]));
      g_strfreev(buff);
/* hack to determine if 2D or 3D */
/* FIIXME - possible for pbc[3] or pbc[4] to be 0 instead? */
      if (fabs(data->pbc[2]) < FRACTION_TOLERANCE)
        data->periodic = 2;
      else
        data->periodic = 3;
/* degrees -> radians */
      data->pbc[3] *= PI/180.0;
      data->pbc[4] *= PI/180.0;
      data->pbc[5] *= PI/180.0;
      cell_flag++;
      }

/* cartesian coord data */
/* FIXME - only data from 1st structure allowed at present! */
    if (g_strncasecmp("cart",*key,4) == 0 && !data_flag)
      {
      data->fractional = FALSE;
      if (g_strncasecmp("au",*(key+1),2) == 0)
        auflag=1;
      data_start++;
      }

/* fractional coord data */
/* FIXME - only data from 1st structure allowed at present! */
    if (g_strncasecmp("frac",*key,4) == 0 && !data_flag)
      {
      data->fractional = TRUE;
      data_start++;
      }

/* while file and coord data are available */
/* FIXME - only data from 1st structure allowed at present! */
    if (data_start)
      {
/* core,shell indices */
      i=j=0;
      while (!fgetline(fp,line))
        {
/* end on space/species - but set flag to read later */
/* TODO - distinguish between symbol & number representations */
/* TODO - call sgInfo (get number if symbol given & matrices) */
        buff = get_tokens(line,7);
        if (g_strncasecmp("space",*buff,5) == 0)
          {
          space_start=1;
          break;
          }
        if (g_strncasecmp("elem",*buff,4) == 0)
          {
          elem_start=1;
          break;
          }
        if (g_strncasecmp("species",*buff,7) == 0)
          {
          species_start=1;
          break;
          }
/* end on anything with less than 4 (atom x y z) items */
/* unless no core/shells are read -> number of fractional coords */
        if (!strlen(*(buff+3)))
          {
          if (i || j)
            break;
          else
           continue;
          }
/* if an atom type (ie core/shell) is specified - adjust data positions */
        type = 0;
        if (g_strncasecmp(*(buff+1),"core",4) == 0)
          type=1;
        if (g_strncasecmp(*(buff+1),"shel",4) == 0)
          type=2;
/* read appropriately */
        switch(type)
          {
          case 0:
          case 1:
/* new core */
            if (i >= data->atom_limit)
              mem_grow(data, ATOM, 10);
/* setup */
            (data->atoms+i)->status = NORMAL;
            (data->atoms+i)->primary = TRUE;
            (data->atoms+i)->orig = TRUE;
            (data->atoms+i)->region = REGION1A;
            (data->atoms+i)->offx = 0;
            (data->atoms+i)->offy = 0;
/* read the atom type & corresponding coords */
/* NB: assumes if there is core/shell l_pos = 1 always */
            sscanf(*buff,"%s",(data->atoms+i)->element);
            sscanf(*buff,"%s",(data->atoms+i)->label);
/* cope with gulp's fractional representation */
            (data->atoms+i)->x = str_to_float(*(buff+1+type));
            (data->atoms+i)->y = str_to_float(*(buff+2+type));
            (data->atoms+i)->z = str_to_float(*(buff+3+type));

/* CURRENT - colourize & special bond calc if partial occ. (shells?) */
            if (strlen(*(buff+5+type)))
              {
              data->has_sof = TRUE;
              (data->atoms+i)->sof = str_to_float(*(buff+5+type));
              }
            else
              (data->atoms+i)->sof = 1.0;

/* copy_items(..ALL) replacement */
            tmp = g_strdup(get_token_pos(line,4+type));
            strcpy((data->atoms+i)->tail, g_strstrip(tmp));
            g_free(tmp);
/* fractional? */
            if (data->fractional)
              {
/* store in case needed in the future */
              (data->atoms+i)->fx = (data->atoms+i)->x;
              (data->atoms+i)->fy = (data->atoms+i)->y;
              (data->atoms+i)->fz = (data->atoms+i)->z;
/* these are the primary coords (for the purpose of display) */
              (data->atoms+i)->x *= data->pbc[0];
              (data->atoms+i)->y *= data->pbc[1];
              (data->atoms+i)->z *= data->pbc[2];
              }
/* atomic units used? */
            if (auflag)
              {
              (data->atoms+i)->x *= AU2ANG;
              (data->atoms+i)->y *= AU2ANG;
              (data->atoms+i)->z *= AU2ANG;
              }
/* NEW - this does setup (colour, code etc.) */
            elem_seek(i, ATOM, data);

#if DEBUG_LOAD_GULP
/*
printf("%s %f %f %f [%d]\n", (data->atoms+i)->element
                           , (data->atoms+i)->x
                           , (data->atoms+i)->y
                           , (data->atoms+i)->z
                           , (data->atoms+i)->atom_code);
*/
#endif
            i++;
            break;
          case 2:
/* new shell */
            if (j >= data->shell_limit)
              mem_grow(data, SHELL, 10);
/* setup the shell */
            (data->shells+j)->status = NORMAL;
            (data->shells+j)->primary = TRUE;
            (data->shells+j)->orig = TRUE;
            (data->shells+j)->region = REGION1A;
            (data->shells+j)->offx = 0;
            (data->shells+j)->offy = 0;
/* read the atom type & corresponding coords */
/* NB: assumes its always [atom] [core/shell] x y z */
            sscanf(*buff,"%s",(data->shells+j)->label);
            sscanf(*buff,"%s",(data->shells+j)->element);
            (data->shells+j)->x = str_to_float(*(buff+2));
            (data->shells+j)->y = str_to_float(*(buff+3));
            (data->shells+j)->z = str_to_float(*(buff+4));
/* CURRENT - copy_items(..ALL) replacement */
            tmp = g_strdup(get_token_pos(line,5));
            strcpy((data->shells+j)->tail, g_strstrip(tmp));
            g_free(tmp);
/* fractional? */
            if (data->fractional)
              {
/* store in case needed in the future */
              (data->shells+j)->fx = (data->shells+j)->x;
              (data->shells+j)->fy = (data->shells+j)->y;
              (data->shells+j)->fz = (data->shells+j)->z;
/* these are the primary coords (for the purpose of display) */
              (data->shells+j)->x *= data->pbc[0];
              (data->shells+j)->y *= data->pbc[1];
              (data->shells+j)->z *= data->pbc[2];
              }
/* GULP atomic units hack */
            if (auflag)
              {
              (data->shells+j)->x *= AU2ANG;
              (data->shells+j)->y *= AU2ANG;
              (data->shells+j)->z *= AU2ANG;
              }
            elem_seek(j, SHELL, data);
            j++;
            break;
          }
        g_strfreev(buff);
        }
/* got them all */
      data->num_asym = data->num_orig = data->num_atoms = i;
      data->num_shells = j;
      data_flag++;
      }

/* space group */
    if (!space_flag)
      {
      if (g_strncasecmp("space",*key,5) == 0 || space_start)
        {
/* CURRENT - is space # possible? The following code assumes space\n# */
        fgetline(fp,line);
/*
        copy_items(buff,line,1,ALL);
*/
        flag=0;
        for (i=0 ; i<strlen(line) ; i++)
          {
/* any alphabetic chars => space *name* */
          if (isalpha(*(line+i)))
            {
            g_free(data->sginfo.spacename);
            data->sginfo.spacename = g_strdup_printf("%s",line);
/* indicate that name should used in lookup */
            data->sginfo.spacenum = -1;
            flag++;
            break;
            }
          }
/* no alphabetic chars, assume space group number */
        if (!flag)
          sscanf(line,"%3d",&data->sginfo.spacenum);
        space_flag++;
        }
      }

/* scan for origin */
    if (g_strncasecmp("origin",*key,6) == 0)
      {
/* FIXME - distinguish between cell choice or origin coords */
/* if nothing on the origin line, hopefully it's on the next */
      if (!strlen(*(key+1)))
        {
        if (fgetline(fp,line))
          return(4);
        buff = get_tokens(line,2);
        sscanf(*buff,"%d",&origin);
        g_strfreev(buff);
        }
      else
        sscanf(*(key+1),"%d",&origin);
/* belongs to previous model or current? */
      if (!space_flag)
        ptr = model_ptr(current-1,RECALL);
      else
        ptr = data;
      ptr->sginfo.cellchoice = abs(origin);
      }

/* element data */
    if (g_strncasecmp("element",*key,7) == 0 || elem_start)
      {
      while (!fgetline(fp,line))
        {
        buff = get_tokens(line,1);
        if (g_strncasecmp("end",*buff,3) == 0)
          break;
        *(elem+num_elem) = g_strdup(g_strstrip(line));
        num_elem++;
        g_strfreev(buff);
        }
      elem_flag=1;
      }

/* species data */
    if (g_strncasecmp("species",*key,5) == 0 || species_start)
      {
      while (!fgetline(fp,line))
        {
        buff = get_tokens(line, 1);
/* is the 1st item an element? */
        i = elem_type(*buff);
        if (i)
          {
          *(species+num_species) = g_strdup(g_strstrip(line));
          num_species++;
          }
        else
          break;
        g_strfreev(buff);
        }
      species_flag=1;
      }

/* potentials - TODO - other types */
    if (g_strncasecmp("buck",*key,4) == 0 ||
        g_strncasecmp("harm",*key,4) == 0 ||
        g_strncasecmp("lenn",*key,4) == 0 ||
        g_strncasecmp("morse",*key,5) == 0 ||
        g_strncasecmp("three",*key,5) == 0 ||
        g_strncasecmp("tors",*key,4) == 0 ||
        g_strncasecmp("outofplane",*key,10) == 0 ||
        g_strncasecmp("spring",*key,6) == 0)
/* etc. */
      {
      if (num_potentials < MAX_POTENTIALS-1)
        {
        *(pot1+num_potentials) = g_strdup(g_strstrip(line));
/* loop until EOF or get a non blank line */
        while (!fgetline(fp,line))
          if (strlen(line))
            break;
/* if missing last line - don't include */
        if (!strlen(line))
          g_free(*(pot1+num_potentials));
        else
          {
          *(pot2+num_potentials) = g_strdup(g_strstrip(line));
          num_potentials++;
          }
        }
      }

/* have we got all of one structure? */
    if (data_flag)
      {
      if (!data->periodic)
        break;
      if ((data->periodic==3 && space_flag) || (data->periodic && species_flag))
        break; 
      }

    g_strfreev(key);
    } /* end main file processing loop */
  while (!(completed = fgetline(fp,line)));

/* did we go right through the file? */
  if (completed)
    {
/* if last pass found nothing - release the allocated model pointer */
    if (!data_flag)
      delete_model(current);
    else
      current++;    /* we found something */
    }
/* not at the end yet, setup for another pass */
  else
    {
    current++;
    data_current++;
    }

#if DEBUG_LOAD_GULP
printf("Found %d cores and %d shells.\n",data->num_atoms,data->num_shells);
printf("Flags: %d %d %d [%d]\n", data_flag, space_flag, cell_flag, completed);
printf("Read model: %d, data number: %d\n",current,data_current);
#endif

/* fudge for saving as marvin restart */
  data->region_empty[REGION1A] = FALSE;
  }
while (!completed);

#if DEBUG_LOAD_GULP
printf("-------------------------------------\n");
printf("  Structures found: %d [%d,%d]\n",current-model,model,current);
printf("       Periodicity: %d\n",data->periodic);
printf("  Potentials found: %d\n",num_potentials);
printf("Species data found: %d\n",num_species);
printf("Element data found: %d\n",num_elem);
printf("-------------------------------------\n");
#endif

/* nothing found (NB: pointers already released) */
if (!(current-model))
  {
  show_text("No data found.");
  return(5);
  }

/* post read init */
for (i=model ; i<current ; i++)
  {
#if DEBUG_LOAD_GULP
printf("Initializing model %d.\n",i);
#endif
/* retrieve data pointer */
  sysenv.mode[i] = FREE;
  data = model_ptr(i, RECALL);
  if (data == NULL)
    return(7);

/* NEW - attach any potential/species/element data found */
  if (num_potentials)
    {
    data->gulp.num_potentials = num_potentials;
    data->gulp.pot1 = g_malloc(num_potentials*sizeof(gchar *));
    data->gulp.pot2 = g_malloc(num_potentials*sizeof(gchar *));
    for (j=0 ; j<num_potentials ; j++)
      {
      *(data->gulp.pot1+j) = g_strdup(*(pot1+j));
      *(data->gulp.pot2+j) = g_strdup(*(pot2+j));
      }
    }
  if (num_elem)
    {
    data->gulp.num_elem = num_elem;
    data->gulp.elem = g_malloc(num_elem*sizeof(gchar *));
    for (j=0 ; j<num_elem ; j++)
      *(data->gulp.elem+j) = g_strdup(*(elem+j));
    }
  if (num_species)
    {
    data->gulp.num_species = num_species;
    data->gulp.species = g_malloc(num_species*sizeof(gchar *));
    for (j=0 ; j<num_species ; j++)
      *(data->gulp.species+j) = g_strdup(*(species+j));
    }

/* special obj & coord init */
  init_objs(AXES_ON, data);
  if (data->periodic)
    {
    init_objs(CELL_ON, data);
    data->axes_type = OTHER;
/* space group init here */
    update_shells(data);
    if (!data->sginfo.spacenum)
      {
#if DEBUG_LOAD_GULP
printf("No Space Group found: assuming P1.\n");
#endif
      data->sginfo.spacenum=1;
      }
    if (genpos(data))
      printf("Error in Space Group lookup.\n");
    }
  init_objs(INIT_COORDS, data);
  calc_bonds(data);
  calc_mols(data);
  new_tree_item(i, APPEND);
  }

/* attached data to all model, now we can free it */
for (i=0 ; i<num_potentials ; i++)
  {
#if DEBUG_LOAD_GULP
printf("%s\n",*(pot1+i));
printf("%s\n",*(pot2+i));
#endif
  g_free(*(pot1+i));
  g_free(*(pot2+i));
  }
for (i=0 ; i<num_elem ; i++)
  {
#if DEBUG_LOAD_GULP
printf("%s\n",*(elem+i));
#endif
  g_free(*(elem+i));
  }
for (i=0 ; i<num_species ; i++)
  {
#if DEBUG_LOAD_GULP
printf("%s\n",*(species+i));
#endif
  g_free(*(species+i));
  }

/* done */
pick_model(current-1);
return(0);
}

/****************************/
/* marvin output file parse */
/****************************/
#define DEBUG_LOAD_MVNOUT 0
gint load_mvnout(FILE *fp, gint model, gchar *inp_file)
{
gchar line[LINELEN], **key, **buff;
gint i, j, k, current_block, region, status, offset;
gint block[4];
gfloat veca[2], vecb[2];
gfloat version;
struct model_pak *data;

data = model_ptr(model, ASSIGN);
if (data == NULL)
  return(1);
 
/* setup */
data->id = MVNOUT;
data->periodic = 2;
strcpy(data->filename, inp_file);
g_free(data->basename);
data->basename = strdup_no_extension(g_basename(inp_file));

/* starting locations */
block[0] = -1;
block[1] = -1;
block[2] = -1;
block[3] = -1;
current_block = 0;

/*****************************************/
/* parse 1 - get location of data blocks */
/*****************************************/
/* scheme - ignore all but the last block of a particular region */
/* read until EOF all coordinate BLOCK data */
status=offset=0;
while (!fgetline(fp,line))
  {
  key = get_tokens(line, 7);
/* YAMH - get version to determine a parsing parameter */
  if (g_strncasecmp(line,"                    Version",27) == 0)
    {
    sscanf(*(key+1),"%f",&version);
    offset = (version < 1.99) ? 0 : 1;
#if DEBUG_LOAD_MVNOUT
printf("Marvin version %f, offset = %d\n",version,offset);
#endif 
    }
/* get surface vectors */
  if (g_strncasecmp(line,"Surface Vectors:",16) == 0)
    {
/* read them in */
    sscanf(*(key+2),"%f",&veca[0]);
    sscanf(*(key+3),"%f",&veca[1]);
    if (fgetline(fp,line))
      return(3);
    buff = get_tokens(line, 2);
    sscanf(*buff,"%f",&vecb[0]);
    sscanf(*(buff+1),"%f",&vecb[1]);
    g_strfreev(buff);
/* find lengths */
    data->pbc[0] = sqrt(veca[0]*veca[0] + veca[1]*veca[1]);
    data->pbc[1] = sqrt(vecb[0]*vecb[0] + vecb[1]*vecb[1]);
/* FIXME - reverse direction of -ve surfvecs, */
/* since gdis can't handle them properly */
    if (veca[0] < 0.0)
      {
      veca[0] *= -1.0;
      veca[1] *= -1.0;
      data->pbc[0] *= -1.0;
      }
    if (vecb[1] < 0.0)
      {
      vecb[0] *= -1.0;
      vecb[1] *= -1.0;
      data->pbc[1] *= -1.0;
      }
/* use via to determine angle between them */
    data->pbc[5] = via(veca, vecb, 2);
    status++;
    continue;
    }
/* NB: disallow image creation in this direction */
data->pbc[2] = 0.0;
/*
  if (g_strncasecmp(line,"Layer Thickness:",16) == 0)
    {
    copy_items(buff,line,3,SINGLE);
    sscanf(buff,"%f",&(data->pbc[2]));
    continue;
    }
*/

/* else, BLOCK (data) keyword check */
/* NB: marvin has block and BLOCK littered about, want BLOCK only */
  if (strncmp(*key,"BLOCK",5) == 0)
    {
    current_block++;
/* is it region data? */
    if (g_strncasecmp(*(key+2),"region",5) != 0)
      continue;
/* get region type */
    if (g_strncasecmp(*(key+5),"1a",2) == 0)
      region = REGION1A;
    else if (g_strncasecmp(*(key+5),"1b",2) == 0)
      region = REGION1B;
    else if (g_strncasecmp(*(key+5),"2a",2) == 0)
      region = REGION2A;
    else if (g_strncasecmp(*(key+5),"2b",2) == 0)
      region = REGION2B;
    else
      continue;
/* is it coordinate data? (don't want gradients) */
    fgetline(fp,line);
    fgetline(fp,line);

    buff = get_tokens(line, 4);
    if (g_strncasecmp(*(buff+3),"coordinates",11) != 0)
      continue;
    block[region] = current_block;
    g_strfreev(buff);

#if DEBUG_LOAD_MVNOUT
printf("Found region type: [%d], block id: %d\n",region,current_block);
#endif
    data->region_empty[region] = FALSE;
    status++;
    }
  g_strfreev(key);
  }

/* checks, return bad status if a key item was missing */
if (status < 2)
  return(status);

/**************************/
/* parse 2 - get the data */
/**************************/
rewind(fp);
current_block = 0;
/* core/shell counters */
i=j=0;
while (!fgetline(fp,line))
  {
/* go through the regions and see which block match should be read */
  if (strncmp(line,"BLOCK",5) == 0)
    {
    current_block++;
    region = -1;
    for (k=REGION1A ; k<REGION2B ; k++)
      if (block[k] == current_block)
        region = k;
    if (region == -1)
      continue;
/* skip straight to data */
    fgetline(fp,line);
    fgetline(fp,line);
    fgetline(fp,line);
    fgetline(fp,line);
/* read it in */
#if DEBUG_LOAD_MVNOUT
printf("Block id: %d, reading region type: %d\n",current_block,region);
#endif
    while(g_strncasecmp(line,"-------",7) != 0)
      {
      buff = get_tokens(line, 7);
/* NB: marvin 1.8 & 2.0 have this at different positions! */
      if (g_strncasecmp(*(buff+offset+2),"cor",3) == 0)
        {
/* new core */
        if (i >= data->atom_limit)
          mem_grow(data, ATOM, 10);
/* setup */
        (data->atoms+i)->status = NORMAL;
        (data->atoms+i)->region = region;
        (data->atoms+i)->primary = TRUE;
        (data->atoms+i)->orig = TRUE;
        (data->atoms+i)->offx = 0;
        (data->atoms+i)->offy = 0;
/* read the atom type */
        sscanf(*(buff+offset+1),"%s",(data->atoms+i)->element);
        sscanf(*(buff+offset+1),"%s",(data->atoms+i)->label);
/* get coords */
        sscanf(*(buff+offset+3),"%f",&((data->atoms+i)->x));
        sscanf(*(buff+offset+4),"%f",&((data->atoms+i)->y));
        sscanf(*(buff+offset+5),"%f",&((data->atoms+i)->z));
/* NEW - occupancy */
        (data->atoms+i)->sof = 1.0;
        *(data->atoms+i)->tail = '\0';
/* do element setup */
        elem_seek(i, ATOM, data);
        i++;
        }
      else if (g_strncasecmp(*(buff+offset+2),"she",3) == 0)
        {
/* new shell */
        if (j >= data->shell_limit)
          mem_grow(data, SHELL, 10);
/* setup the shell */
        (data->shells+j)->status = NORMAL;
        (data->shells+j)->region = region;
        (data->shells+j)->primary = TRUE;
        (data->shells+j)->orig = TRUE;
        (data->shells+j)->offx = 0;
        (data->shells+j)->offy = 0;
/* read the atom type */
        sscanf(*(buff+offset+1),"%s",(data->shells+j)->element);
        sscanf(*(buff+offset+1),"%s",(data->shells+j)->label);
/* get coords */
        sscanf(*(buff+offset+3),"%f",&((data->shells+j)->x));
        sscanf(*(buff+offset+4),"%f",&((data->shells+j)->y));
        sscanf(*(buff+offset+5),"%f",&((data->shells+j)->z));
        *(data->shells+j)->tail = '\0';
/* do element setup */
        elem_seek(j, SHELL, data);
        j++;
        }
      else
        {
        printf("load_mvnout(): parse error.\n");
printf("[%s]\n",line);
        break;
        }
/* get next line, terminate on EOF */
      if (fgetline(fp,line))
        {
        printf("Warning: EOF in data block.\n");
        break;
        }
      g_strfreev(buff);
      }
    }
  }

/* got them all */
data->num_asym = data->num_orig = data->num_atoms = i;
data->num_shells = j;

/* init lighting */
j=0;
for (i=REGION1A ; i<REGION2B ; i++)
  {
  if (!data->region_empty[i] && !j)
    {
    data->region[i] = TRUE;
    j++;
    }
  else
    data->region[i] = FALSE;
  }
   
#if DEBUG_LOAD_MVNOUT
printf("model loaded: found %d atom(s) ",data->num_atoms);
printf("and %d shell(s).\n",data->num_shells);
printf("Surface vectors: %f x %f\n",data->pbc[0],data->pbc[1]);
#endif

/* FIXME - force non periodic in z (marvin is 2D) */
/* FIXME - update image replication code accordingly */
data->pbc[2] = 0;

/* post read init */
sysenv.mode[model] = FREE;
sysenv.active = model;
init_objs(AXES_ON, data);
if (data->periodic)
  {
  init_objs(CELL_ON, data);
  data->axes_type = OTHER;
/* space group init here */
  update_shells(data);
  if (!data->sginfo.spacenum)
    data->sginfo.spacenum=1;
  if (genpos(data))
    printf("Error in Space Group lookup.\n");
  }
/* setup latmat */
init_objs(INIT_COORDS, data);
calc_bonds(data);
calc_mols(data);

pick_model(model);
new_tree_item(model, APPEND);

return(0);
}

/*******************/
/* data file parse */
/*******************/
#define DEBUG_LOAD 0
gint load_data(FILE *fp, struct parse_pak *parse_data, struct model_pak *data)
{
gchar line[LINELEN], key[LINELEN], buff[LINELEN];
gint i, j, start, type, region, status;
gint space_flag=0;

/* skip any initial rubbish */
i=0;
while(i<parse_data->hdr_skip)
  {
/* EOF check */
  if (fgetline(fp,line))
    return(2);
/* flag if file has a PBC */
  sscanf(line,"%s",key); 
  if (g_strncasecmp(key,"PBC=ON",6) == 0)
    data->periodic = 3;
  i++;
  }

#if DEBUG_LOAD
printf("Parsing header...\n");
#endif

switch(parse_data->id)
  {
/* if BIOSYM and periodic - start of coords is after PBC line (not !DATE) */
  case BIOSYM:
    if (data->periodic)
      {
      i=0;
      while(g_strncasecmp("pbc ", line, 4) != 0 && !i)
        i = fgetline(fp,line);
/* EOF check */
      if (i)
        return(3);
      copy_items(buff,line,2,SINGLE);
      sscanf(buff,"%f",&(data->pbc[0]));
      copy_items(buff,line,3,SINGLE);
      sscanf(buff,"%f",&(data->pbc[1]));
      copy_items(buff,line,4,SINGLE);
      sscanf(buff,"%f",&(data->pbc[2]));
      copy_items(buff,line,5,SINGLE);
      sscanf(buff,"%f",&(data->pbc[3]));
      copy_items(buff,line,6,SINGLE);
      sscanf(buff,"%f",&(data->pbc[4]));
      copy_items(buff,line,7,SINGLE);
      sscanf(buff,"%f",&(data->pbc[5]));
/* degrees -> radians */
      data->pbc[3] *= PI/180.0;
      data->pbc[4] *= PI/180.0;
      data->pbc[5] *= PI/180.0;
/* don't search for coord start string */
      parse_data->coord_start1_len = 0;
/* prevent the warning message (keyword corruption) being printed */
      strcpy(parse_data->coord_start1,"");
      }
    break;

  case XTL:
    data->periodic = 3;
    data->fractional = TRUE;
/* seek dimension keyword */
    while (!fgetline(fp,line))
      {
      if (g_strncasecmp("dimension", line, 9) == 0)
        {
        copy_items(buff,line,2,SINGLE);
        sscanf(buff,"%d",&data->periodic);
        break;
        }
      }
    rewind(fp);
/* seek cell keyword */
    while (!fgetline(fp,line))
      {
      if (g_strncasecmp(parse_data->special_start, line,
                        parse_data->special_start_len) == 0)
        break;
      }
/* get next line (if possible) */
    if (fgetline(fp,line))
      return(4);
    copy_items(buff,line,1,SINGLE);
    sscanf(buff,"%f",&(data->pbc[0]));
    copy_items(buff,line,2,SINGLE);
    sscanf(buff,"%f",&(data->pbc[1]));
/* dimension dependent parsing */
    if (data->periodic == 2)
      {
      copy_items(buff,line,3,SINGLE);
      sscanf(buff,"%f",&(data->pbc[5]));
      data->pbc[2] = 0.0;
      data->pbc[3] = 90.0;
      data->pbc[4] = 90.0;
      }
    else
      {
      copy_items(buff,line,3,SINGLE);
      sscanf(buff,"%f",&(data->pbc[2]));
      copy_items(buff,line,4,SINGLE);
      sscanf(buff,"%f",&(data->pbc[3]));
      copy_items(buff,line,5,SINGLE);
      sscanf(buff,"%f",&(data->pbc[4]));
      copy_items(buff,line,6,SINGLE);
      sscanf(buff,"%f",&(data->pbc[5]));
      }
/* degrees -> radians */
    data->pbc[3] *= PI/180.0;
    data->pbc[4] *= PI/180.0;
    data->pbc[5] *= PI/180.0;

/* try to get symmetry info */
/* FIXME - yet another unholy file parsing mess */
    if (fgetline(fp,line))
      return(5);
    copy_items(buff,line,1,SINGLE);
    while(g_strncasecmp(buff,"symmetry",8) == 0 && !space_flag)
      {
      i=2;
      copy_items(buff,line,i,SINGLE);
      while(strlen(buff) && !space_flag)
        {
/* seek space group label */
        if (g_strncasecmp(buff,"label",5) == 0)
          {
          copy_items(buff,line,i+1,SINGLE);
          g_free(data->sginfo.spacename);
          data->sginfo.spacename = g_strdup_printf("%s",buff);

/* indicate that name should used in lookup */
          data->sginfo.spacenum = -1;
          space_flag++;
          }
/* seek space group number */
        if (g_strncasecmp(buff,"number",6) == 0 && data->sginfo.spacenum > -1)
          {
          copy_items(buff,line,i+1,SINGLE);
/* do lookup via this number only if no label can be found */
          sscanf(buff,"%d",&data->sginfo.spacenum);
          }
        i++;
        copy_items(buff,line,i,SINGLE);
        }
      if (!space_flag)
        {
        if (fgetline(fp,line))
          return(6);
        copy_items(buff,line,1,SINGLE);
        }
      }
    break;
  }

#if DEBUG_LOAD
printf("Searching for data...\n");
#endif

/* shouldn't happen any more ??? */
if (strlen(parse_data->coord_start1) != parse_data->coord_start1_len)
  printf("Warning: corrupted keyword [%s] (%d)\n",parse_data->coord_start1,
                                                  parse_data->coord_start1_len);
/* find start of coords */
i=0;
while(!i && parse_data->coord_start1_len)
  {
/* get lines */
  if (fgetline(fp,line))
    return(7);
/* get 1st item */
  copy_items(key,line,1,SINGLE);
/* keyword scan */
  if (strlen(key) < 3)
    continue;
  if (g_strncasecmp(key,parse_data->coord_start1,parse_data->coord_start1_len) == 0)
    {
    i=1;
    }
/* terminate on match2 (if there is one) - set any special flags */
  if (parse_data->coord_start2_len && 
      g_strncasecmp(key,parse_data->coord_start2,parse_data->coord_start2_len) == 0)
    {
    i=1;
    }
  }

#if DEBUG_LOAD
printf("found!\n");
#endif

/* marvin region parsing */
data->region_empty[REGION1A] = TRUE;
data->region_empty[REGION2A] = TRUE;
data->region_empty[REGION1B] = TRUE;
data->region_empty[REGION2B] = TRUE;
if (data->id == MARVIN)
  {
/* get region type */
  copy_items(key,line,2,ALL);
  if (g_strncasecmp(key,"1 b",3) == 0 || g_strncasecmp(key,"1b",2) == 0)
    region = REGION1B;
  else if (g_strncasecmp(key,"2 a",3) == 0 || g_strncasecmp(key,"2a",2) == 0)
    region = REGION2A;
  else if (g_strncasecmp(key,"2 b",3) == 0 || g_strncasecmp(key,"2b",2) == 0)
    region = REGION2B;
  else
    region = REGION1A;      /* default */
/* we have data for some region */
  data->region_empty[region] = FALSE;
  }
/* allows saving as mvn-r */
else
  {
  data->region_empty[REGION1A] = FALSE;
  region = REGION1A;
  }

/* init for data reading */
i=j=0;                  /* atoms & shells counters */
status=0;               /* from now on an EOF is not an error */

/* MAIN - parse lines of data */
start=0;
while (!fgetline(fp,line))
  {
/* NEW - wait until 1st valid line is reached */
/* ie skip (but don't end!) on anything with less than 4 (atom x y z) items */
  copy_items(buff,line,4,SINGLE);
  if (!strlen(buff) && !start)
    continue;
  else
    start=1;

/* set up for keyword parse */
  copy_items(key,line,1,SINGLE);
/* test for a new start of data keyword */
  if (parse_data->coord_start1_len &&
      g_strncasecmp(key,parse_data->coord_start1,parse_data->coord_start1_len) == 0)
    {
/* do we have anything to do? */
    switch(data->id)
      {
      case MARVIN:
        copy_items(key,line,2,ALL);
/* get region type */
        if (g_strncasecmp(key,"1 b",3) == 0 ||
            g_strncasecmp(key,"1b",2) == 0)
          region = REGION1B;
        else if (g_strncasecmp(key,"2 a",3) == 0 || 
                 g_strncasecmp(key,"2a",2) == 0)
          region = REGION2A;
        else if (g_strncasecmp(key,"2 b",3) == 0 || 
                 g_strncasecmp(key,"2b",2) == 0)
          region = REGION2B;
        else
          region = REGION1A;      /* default */
/* we have data for some region */
        data->region_empty[region] = FALSE;
        break;
      }
/* ok, get next line (actual data) */
    if (fgetline(fp,line))
      break;
/* set up for keyword parse */
    copy_items(key,line,1,SINGLE);
    }

/* test for end keyword */
  if (g_strncasecmp(key,parse_data->coord_end1,parse_data->coord_end1_len) == 0)
    {
/* everything else breaks on this */
    if (data->id != BIOSYM)
      break;
/* parse for things like end end */
    if (parse_data->coord_end2_len)
      {
      if (fgetline(fp,line))
        break;
      copy_items(key,line,1,SINGLE);
/* terminate on end end (biosym) */
      if (g_strncasecmp(key,parse_data->coord_end2,parse_data->coord_end2_len) == 0)
        break;
      }
    else
      break;
    }
/* NEW - end on anything with less than 4 (atom x y z) items */
/* unless MARVIN, since it may be a region changeover */
  copy_items(buff,line,4,SINGLE);
  if (!strlen(buff))
    {
    if (data->id == MARVIN)
      continue;
    else
      break;
    }

/* if an atom type (ie core/shell) is specified - adjust data positions */
  type = 0;
  copy_items(buff,line,parse_data->x_pos,SINGLE);
  if (g_strncasecmp(buff,"core",4) == 0)
    type=1;
  if (g_strncasecmp(buff,"shel",4) == 0)
    type=2;
/* read appropriately */
  switch(type)
    {
    case 0:
    case 1:
/* new core */
      if (i >= data->atom_limit)
        mem_grow(data, ATOM, 10);
/* setup */
      (data->atoms+i)->status = NORMAL;
      (data->atoms+i)->primary = TRUE;
      (data->atoms+i)->orig = TRUE;
      (data->atoms+i)->region = region;
      (data->atoms+i)->offx = 0;
      (data->atoms+i)->offy = 0;
/* NB: assumes if there is core/shell a_pos = 1 always */
      copy_items(buff,line,parse_data->a_pos,SINGLE);
/* read the atom type & corresponding coords */
      sscanf(buff,"%s",(data->atoms+i)->element);
/* NB: assumes if there is core/shell l_pos = 1 always */
      copy_items(buff,line,parse_data->l_pos,SINGLE);
      sscanf(buff,"%s",(data->atoms+i)->label);
/* get data */
      copy_items(buff,line,type+parse_data->x_pos,SINGLE);
      sscanf(buff,"%f",&((data->atoms+i)->x));
      copy_items(buff,line,type+parse_data->y_pos,SINGLE);
      sscanf(buff,"%f",&((data->atoms+i)->y));
      copy_items(buff,line,type+parse_data->z_pos,SINGLE);
      sscanf(buff,"%f",&((data->atoms+i)->z));
/* NEW - occupancy */
      (data->atoms+i)->sof = 1.0;
/* fractional? */
      if (data->fractional)
        {
        (data->atoms+i)->fx = (data->atoms+i)->x;
        (data->atoms+i)->fy = (data->atoms+i)->y;
        (data->atoms+i)->fz = (data->atoms+i)->z;
        (data->atoms+i)->x *= data->pbc[0];
        (data->atoms+i)->y *= data->pbc[1];
/* 2D xtl hack */
        if (data->periodic == 3)
          (data->atoms+i)->z *= data->pbc[2];
        }
/* save additional data (if any) */
      if (parse_data->tail_pos)
        copy_items((data->atoms+i)->tail,line,type+parse_data->tail_pos,ALL);
      else
        strcpy((data->atoms+i)->tail,"\0");
/* NEW - this does setup */
      elem_seek(i, ATOM, data);

#if DEBUG_MORE
printf("%s %f %f %f [%d]\n", (data->atoms+i)->label
                           , (data->atoms+i)->x
                           , (data->atoms+i)->y
                           , (data->atoms+i)->z
                           , (data->atoms+i)->atom_code);
#endif
      i++;
      break;
    case 2:
/* new shell */
      if (j >= data->shell_limit)
        mem_grow(data, SHELL, 10);
/* setup the shell */
      (data->shells+j)->status = NORMAL;
      (data->shells+j)->primary = TRUE;
      (data->shells+j)->orig = TRUE;
      (data->shells+j)->region = region;
      (data->shells+j)->offx = 0;
      (data->shells+j)->offy = 0;
/* read the atom type & corresponding coords */
/* NB: assumes its always [atom] [core/shell] x y z */
      copy_items(buff,line,1,SINGLE);
      sscanf(buff,"%s",(data->shells+j)->label);
      copy_items(buff,line,1,SINGLE);
      sscanf(buff,"%s",(data->shells+j)->element);
      copy_items(buff,line,3,SINGLE);
      sscanf(buff,"%f",&((data->shells+j)->x));
      copy_items(buff,line,4,SINGLE);
      sscanf(buff,"%f",&((data->shells+j)->y));
      copy_items(buff,line,5,SINGLE);
      sscanf(buff,"%f",&((data->shells+j)->z));
/* save additional data */
      if (parse_data->tail_pos)
        copy_items((data->shells+j)->tail,line,type+parse_data->tail_pos,ALL);
      else
        strcpy((data->shells+j)->tail,"\0");
/* do the fractional conversion here */
      if (data->fractional)
        {
        (data->shells+j)->fx = (data->shells+j)->x;
        (data->shells+j)->fy = (data->shells+j)->y;
        (data->shells+j)->fz = (data->shells+j)->z;
        (data->shells+j)->x *= data->pbc[0];
        (data->shells+j)->y *= data->pbc[1];
/* 2D xtl hack */
        if (data->periodic == 3)
          (data->shells+j)->z *= data->pbc[2];
        }

#if DEBUG_MORE
printf("%s %f %f %f\n", (data->shells+j)->element
                      , (data->shells+j)->x
                      , (data->shells+j)->y
                      , (data->shells+j)->z);
#endif 

/* match shell type to database */      
      elem_seek(j, SHELL, data);
/* update counter */
      j++;
      break;
    }
  }

/* got them all */
data->num_asym = data->num_orig = data->num_atoms = i;
data->num_shells = j;

/* MARVIN - region highlighting */
if (data->id == MARVIN)
  {
  j=0;
  for (i=REGION1A ; i<REGION2B ; i++)
    {
    if (!data->region_empty[i] && !j)
      {
      data->region[i] = TRUE;
      j++;
      }
    else
      data->region[i] = FALSE;
    }
  }

#if DEBUG_LOAD
printf("model loaded: found %d atom(s) ",data->num_atoms);
printf("and %d shell(s).\n",data->num_shells);
printf("load_data() done.\n");
#endif

return(status);
}

/********************/
/* BABEL conversion */
/********************/
gint babel_convert(gint model, gchar *inpfile)
{
gint status;
gchar *inp, *out, *outfile;
GString *txt; 

/* TODO - ask user what type of input it is, */
/* rather than an extension based guess */
switch (sysenv.babel_type)
  {
/* FIXME - what about the periodic data??? */
  case CSSR:
    inp = g_strdup("cssr");
    break;
  case FDAT:
    inp = g_strdup("fdat");
    break;
  case GAMOUT:
    inp = g_strdup("gamout");
    break;
  case MOPAC_OUT:
    inp = g_strdup("mopout");
    break;
  case PDB:
    inp = g_strdup("pdb");
    break;
  default:
    printf("babel_convert(): unknown file type.\n");
    return(1);
  }

/* set up new filename */
out = g_strdup("xyz");
txt = g_string_new(NULL);
g_string_assign(txt,inpfile);
g_string_append(txt,".xyz");
outfile = g_strdup(txt->str);
/* convert & load via standard routine */
printf("Converting with babel...\n");
g_string_sprintf(txt,"%s -i%s %s -o%s %s >& /dev/null",sysenv.babel_exe,
                                           inp, inpfile, out, outfile);
system(txt->str);
status = load_model(model, XYZ, outfile);
/* free & exit */
g_string_free(txt, TRUE);
g_free(inp);
g_free(out);
g_free(outfile);
return(status);
}

/*******************************/
/* MARVIN specific file saving */
/*******************************/
gint save_marvin(gchar *filename, struct model_pak *data)
{
gint i, j, m, n;
gint mol, region, flag;
FILE *fp;

/* open the file */
fp = fopen(filename,"w");
if (!fp)
  {
  show_text("Error, bad filename!");
  return(1);
  }

show_text("Saving file in MARVIN restart format!");

/* save via the number of molecules in the */
/* model - since this allows mol numbering */
mol=0;

/* four region types printing */
for (region=REGION1A ; region<REGION2B ; region++)
  {
  if (data->region_empty[region])
    continue;
  switch(region)
    {
    case REGION1A:
      fprintf(fp,"coordinates 1 A\n");
      break;
    case REGION2A:
      fprintf(fp,"coordinates 2 A\n");
      break;
    case REGION1B:
      fprintf(fp,"coordinates 1 B\n");
      break;
    case REGION2B:
      fprintf(fp,"coordinates 2 B\n");
      break;
    }

/* do mols */
  for (n=0 ; n<data->num_mols ; n++)
    {
/* are any atoms written to file? */
    flag=0;
/* m - number of atoms in molecule */
    for (m=0 ; m<(data->mols+n)->num_atoms ; m++)
      {
/* get the index for the mth atom in the nth molecule */
      i = *((data->mols+n)->atom+m);
/* don't print if not in current region */
      if ((data->atoms+i)->region != region)
        continue;

/* NB: y coord no longer inverted */
      if ( (data->atoms+i)->status != DELETED)
        {
        fprintf(fp,"%-7s   core   %14.9f  %14.9f  %14.9f   %d\n",
                 (data->atoms+i)->label,
                 (data->atoms+i)->rx+data->centroid[0],
                 (data->atoms+i)->ry+data->centroid[1],
                 (data->atoms+i)->rz+data->centroid[2], mol);
/* written at least one atom - molecule is non-empty */
        flag++;
/* shell? */
        if ((data->atoms+i)->has_shell)
          {
/* get the shell's index */
          j = (data->atoms+i)->idx_shell;
          fprintf(fp,"%-7s   shel   %14.9f  %14.9f  %14.9f   %d\n",
                 (data->shells+j)->label,
                 (data->shells+j)->rx+data->centroid[0],
                 (data->shells+j)->ry+data->centroid[1],
                 (data->shells+j)->rz+data->centroid[2], mol);
          }
        }
      }
/* completed a non empty molecule */
    if (flag)
      mol++;
    }
/* done region */
  fprintf(fp,"end\n");
  }

/* done all */
fclose(fp);
return(0);
}

/*********************/
/* create a new file */
/*********************/
gint create_file(gchar *filename, gint type, struct model_pak *data)
{
FILE *fp;
gchar label[LABEL_SIZE], prefix[LABEL_SIZE], postfix[LINELEN];
gfloat x[3], s[3];
gint mol, m, n, i, j, flag;
time_t t1;

/* open the file */
fp = fopen(filename,"w");
if (!fp)
  {
  show_text("Error, bad filename!");
  return(1);
  }

/* print header */
switch(type)
  {
  case BIOSYM:
    show_text("Saving file in BIOSYM format!");
    fprintf(fp,"!BIOSYM archive 3\n");
    if (data->periodic)
      fprintf(fp,"PBC=ON\n");
    else
      fprintf(fp,"PBC=OFF\n");
    fprintf(fp,"Created by GDIS\n");
/* get & print the date (streamlined slightly by sxm) */
    t1 = time(NULL);
    fprintf(fp,"!DATE %s",ctime(&t1));
/* convert back to degrees!!! */
    if (data->periodic)
      fprintf(fp,"PBC %9.6f %9.6f %9.6f %4.1f %4.1f %4.1f (P 1)\n"
                  ,data->pbc[0]
                  ,data->pbc[1]
                  ,data->pbc[2]
                  ,180.0*data->pbc[3]/PI
                  ,180.0*data->pbc[4]/PI
                  ,180.0*data->pbc[5]/PI);

/* TODO - if spacenum > 1 - only write asym unit */
/*
                  ,data->sginfo.spacename);
*/
    break;
  case XYZ:
    show_text("Saving file in XYZ format!");
    fprintf(fp,"%d\n",data->num_atoms);
    fprintf(fp,"XYZ\n");
    break;
/* all other types are not supported */
  default:
    show_text("Sorry, only biosym, marvin and xyz save types are supported.");
    return(1);
  }


/* save via the number of molecules in the */
/* model - since this allows mol numbering */
mol=0;
for (n=0 ; n<data->num_mols ; n++)
  {
/* flag - atom written to file? (ie if UNDELETED) */
  flag=0;
/* m - number of atoms in molecule */
  for (m=0 ; m<(data->mols+n)->num_atoms ; m++)
    {
/* get the index for the mth atom in the nth molecule */
    i = *((data->mols+n)->atom+m);

/* setup the strings that go after the coords */
    if (data->id == BIOSYM)
      {
/* we should already have one */
      copy_items(label,(data->atoms+i)->tail, 1, SINGLE);
      copy_items(postfix,(data->atoms+i)->tail, 3, ALL);
      }
    else
      {
/* we need to create one */
      strcpy(label,"CORE");
      strcpy(postfix,"xx      ?   0.000");
      put_item(postfix,(data->atoms+i)->element,2);
      }

/* write atom to file unless deleted */
/* NEW - also if hidden, fudge to get asym part printed in */
/* files that have no provision for symmetry */
    if ((data->atoms+i)->status != DELETED && (data->atoms+i)->status != HIDDEN)
      {
/* use the original label instead of the processed element symbol */
      g_snprintf(prefix,5,"%-4s",(data->atoms+i)->label);

/* get associated shell (if any) */
/* NOTE: no effect at present - how to save shells in BIOSYM format? */
      j=-1;
      if ((data->atoms+i)->has_shell)
        j = (data->atoms+i)->idx_shell;

/* paranoid test */
      if (!data->periodic && data->fractional)
        printf("Serious error in create_file().\n");
/* use orig if these are fractional/periodic coords */
/* as these cannot have any rotational offsets */
/* FIXME - gets the centering wrong on reload, why??? */
      if (data->periodic) 
        {
        x[0] = (data->atoms+i)->x;
        x[1] = (data->atoms+i)->y;
        x[2] = (data->atoms+i)->z;
/* if output type is not fractional (and nothing is yet) then convert */
/* to cart if data to save is fractional */
        if (data->fractional)
          vecmat(data->latmat, x);
/* shell */
        if (j >= 0)
          {
          s[0] = (data->shells+j)->x;
          s[1] = (data->shells+j)->y;
          s[2] = (data->shells+j)->z;
          if (data->fractional)
            vecmat(data->latmat, s);
          }
        }
      else
        {
/* default values to save */
        x[0] = (data->atoms+i)->rx;
        x[1] = -(data->atoms+i)->ry;
        x[2] = (data->atoms+i)->rz;
/* shell */
        if (j >= 0)
          {
          s[0] =  (data->shells+j)->rx;
          s[1] = -(data->shells+j)->ry;
          s[2] =  (data->shells+j)->rz;
          }
        }

/* now write according to respective formats */
      switch(type)
        {
/* unique molecule numbering for BIOSYM files (>0) */
        case BIOSYM:
          fprintf(fp,"%4s %14.9f %14.9f %14.9f %-4s %-6d %s\n",
                      prefix,x[0],x[1],x[2],label,mol+1,postfix);
          break;
        case XYZ:
          fprintf(fp,"%-7s    %14.9f  %14.9f  %14.9f\n",
                      (data->atoms+i)->element,x[0],x[1],x[2]);
          break;
        }
/* indicate we've written at least one atom */
      flag++;
      }
    }     /* for m */

/* if at least one atom in the molecule was written - inc mol numbering */
  if (flag)
    mol++;

/* do anything special at the end of each molecule? */
  switch(type)
    {
/* only write if at least one atom in molecule is undeleted */
    case BIOSYM:
      if (flag)
        fprintf(fp,"end\n");
      break;
    }
  }       /* for n */

/* create tail */
switch(type)
  {
  case BIOSYM:
    fprintf(fp,"end\n");
    break;
  }

fclose(fp);
return(0);
}

/***********/
/* globals */
/***********/
GtkWidget *path_list, *file_list, *curr_path, *file_name;
gint path_width=80, file_width=160;
extern int alphasort();
gint file_filter();
gint path_filter();

/**************************/
/* primary load file call */
/**************************/
void file_load()
{
gint i, model, id, status;
gfloat vec[3], mat[9];
gchar inp_file[FILELEN];
FILE *fp;
struct model_pak *data;

strcpy(inp_file, sysenv.cwd); 
strcat(inp_file, gtk_entry_get_text(GTK_ENTRY(file_name)));

/* if last char is a / -> directory only -> empty filename */
if (inp_file[strlen(inp_file)-1] == '/')
  {
  show_text("Error: bad filename.\n");
  return;
  }

/* get the new model number & type */
model = sysenv.num_models;
if ((id = model_type(inp_file)) < 0)
  return;
status = -1;

if (model < MAX_MODELS)
  {
/* create fp here, so errors in load routines don't have to clean up */
  fp = fopen(inp_file,"r");
  if (!fp)
    {
    show_text("Error: bad or missing filename.");
    return;
    }
/* NEW - cleaner branching to specific loading routines */
/* each routine does it's own alloc, so multiple models can be loaded */
  switch(id)
    {
    case GULP:
      status = load_gulp(fp, model, inp_file);
      fclose(fp);
      break;
    case GULPOUT:
      status = load_gulp_output(fp, model, inp_file);
      fclose(fp);
      break;
    case MVNOUT:
      status = load_mvnout(fp, model, inp_file);
      fclose(fp);
      break;
    case MORPH:
      fclose(fp);
      status = morph_create(inp_file);
      break;
    case BIOSYM:
    case MARVIN:
    case XTL:
    case XYZ:
      fclose(fp);
      status = load_model(model, id, inp_file);
      break;
/* NEW - babel handled filetypes */
    case BABEL:
      fclose(fp);
/* TODO - get id from additional filtermenu */
      status = babel_convert(model, inp_file);
      break;
    default:
      printf("file_load() error, no handling routine for filetype.\n");
      fclose(fp);
    }

/* check for successful file load */
  if (status)
    printf("Load failed, error code: %d\n",status);
  else
    {
/* CURRENT - post-load cartesian -> fractional */
/* NEW - convert 2D periodic coords to fractional */
/* THUS, data->fractional only means that fx,fy,fz are defined */
/* since if periodic, then x,y,z are always the SCALED fractional */
/* coords and only need latmat mult to transform them to cartesian */
    data = model_ptr(model, RECALL);
    if (data->periodic && !data->fractional)
      {
      memcpy(mat, data->latmat, 9*sizeof(gfloat));
      invmat(mat,3);
      for (i=data->num_atoms ; i-- ; )
        {
        vec[0] = (data->atoms+i)->x; 
        vec[1] = (data->atoms+i)->y; 
        vec[2] = (data->atoms+i)->z; 
        vecmat(mat, vec);
        (data->atoms+i)->x = vec[0];
        (data->atoms+i)->y = vec[1];
/* if 2D periodic - keep z as cartesian */
        if (data->periodic == 3)
          (data->atoms+i)->z = vec[2];
        vecmat(data->latmat, vec);
        }
/* recalc centroid */
      init_objs(CENT_COORDS, data);
      calc_bonds(data);
      calc_mols(data);
      }
    show_text("New model loaded.");
/* updates */
    update_geom_info();
    wire_draw(ALL);
    }
  }
else
  show_text("Maximum number of allowed models reached.");

/* finished - only now we destroy the file select widget */
close_dialog_type(FILE_LOAD);
}

/***************************************/
/* update contents of file load widget */
/***************************************/
#define DEBUG_UPDATE_FILE_PANE 0
void update_file_pane()
{
gint i,n,type;
gchar *name[1], full[FILELEN];
#ifdef __sun
struct dirent **files;
#else
struct direct **files;
#endif
struct stat buff;

name[0] = (gchar *) g_malloc(FILELEN*sizeof(gchar));

/* getting from this directory */
gtk_label_set_text(GTK_LABEL(curr_path),sysenv.cwd);

/* clear the CLists */
gtk_clist_clear((GtkCList *) path_list);
gtk_clist_clear((GtkCList *) file_list);

/* get directory listing */
n = scandir(sysenv.cwd, &files, 0, alphasort);

/* special case for a failure (is this always a no permission error?) */
if (n < 0)
  {
  show_text("scandir failed, perhaps you don't have access permission.");
/* stick in the .. so we can always climb out */
  strcpy(name[0], "..");
  gtk_clist_append((GtkCList *) path_list, &name[0]);
  }

for (i=0 ; i<n ; i++)
  {
/* copy to right pointer type */
  strcpy(name[0], files[i]->d_name);
/* stat the file (MUST have the full path) */
  strcpy(full, sysenv.cwd);
  strcat(full, files[i]->d_name);
  stat(full, &buff);

#if DEBUG_UPDATE_FILE_PANE
printf("[%s] : %d\n",full,buff.st_mode);
#endif

/* convert and check if directory */
  if (S_ISDIR(buff.st_mode))
    gtk_clist_append((GtkCList *) path_list, &name[0]);
  else
    {
/* is it a recognized type? */
    if ((type = model_type(files[i]->d_name)) < 0)
      continue;
/* display related types together */
    if (type == MVNOUT)
      type = MARVIN;
    if (type == GULPOUT)
      type = GULP;
/* add it to the list if filter allows */
/* separate babel from everything else */
    if (sysenv.file_type == BABEL || type == BABEL) 
      {
      if (type == sysenv.file_type) 
        gtk_clist_append((GtkCList *) file_list, &name[0]);
      }
    else
      if (type == sysenv.file_type || sysenv.file_type == DATA) 
        gtk_clist_append((GtkCList *) file_list, &name[0]);
    }
  }

/* done */
g_free(name[0]);
}

/*****************************/
/* CList handler - directory */
/*****************************/
#define DEBUG_PATH_HANDLER 0
void path_handler(GtkWidget *clist, gint row, gint column,
                  GdkEventButton *event, gpointer data)
{
gint i;
gchar *text;

gtk_clist_get_text(GTK_CLIST(path_list), row, column, &text);

/* remove any \n */
i=strlen(text);
while(--i > 0)
  if (*(text+i) == '\n')
    *(text+i) = '\0';

/* do we go up a subdir? */
if (g_strncasecmp("..",text,2) == 0)
  {
  i=strlen(sysenv.cwd)-1;
  while(--i > 0)
    if (*(sysenv.cwd+i) == '/')
      {
      *(sysenv.cwd+i) = '\0';
      break; 
      }
  }
else
  strcat(sysenv.cwd, text);

/* slap this on the end */
strcat(sysenv.cwd,"/");

#if DEBUG_PATH_HANDLER
printf("Path changed to: [%s]\n",sysenv.cwd);
#endif

/* get contents of new directory */
update_file_pane();
}

/*****************************/
/* CList handler - file name */
/*****************************/
void file_handler(GtkWidget *clist, gint row, gint column,
                  GdkEventButton *event, gpointer data)
{
gchar *text;

gtk_clist_get_text(GTK_CLIST(file_list), row, column, &text);
gtk_entry_set_text(GTK_ENTRY (file_name), text);
}

/*******************/
/* filter handlers */
/*******************/
void change_filter(GtkWidget *w, gint type)
{
sysenv.file_type = type;
update_file_pane();
}
void change_babel_filter(GtkWidget *w, gint type)
{
sysenv.babel_type = type;
update_file_pane();
}

/****************************/
/* new filter item routines */
/****************************/
GtkWidget *make_filter_item(gchar *name, gpointer data)
{
GtkWidget *item;
 
item = gtk_menu_item_new_with_label(name);
gtk_signal_connect(GTK_OBJECT(item), "activate", change_filter, data);
gtk_widget_show(item);

return(item);
}

GtkWidget *make_babel_item(gchar *name, gpointer data)
{
GtkWidget *item;
 
item = gtk_menu_item_new_with_label(name);
gtk_signal_connect(GTK_OBJECT(item), "activate", change_babel_filter, data);
gtk_widget_show(item);

return(item);
}

/*************************************************/
/* customized load widget (with filetype filter) */
/*************************************************/
void load_widget()
{
gint id;
GtkWidget *swin, *hbox;
GtkWidget *filter, *filter_menu, *item;
GtkWidget *babel, *babel_menu;
GtkWidget *label, *button;
struct dialog_pak *fl_data;

/* get a dialog if possible */
if ((id = request_dialog(-1,FILE_LOAD)) < 0)
  return;
fl_data = &sysenv.dialog[id];

/* make and set up the dialog window */
fl_data->win = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW(fl_data->win), "Load file");
gtk_window_set_default_size(GTK_WINDOW(fl_data->win), 400, 350);
gtk_window_set_position(GTK_WINDOW(fl_data->win), GTK_WIN_POS_CENTER);
gtk_container_set_border_width(
    GTK_CONTAINER(GTK_BOX(GTK_DIALOG(fl_data->win)->vbox)),10);
gtk_signal_connect(GTK_OBJECT(fl_data->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

/* TOP ROW - cwd printed */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fl_data->win)->vbox),hbox,FALSE,FALSE,0);
label = gtk_label_new("Current path: ");
gtk_box_pack_start(GTK_BOX(hbox), label, FALSE, FALSE, 0);

curr_path = gtk_label_new("dummy");
gtk_box_pack_start(GTK_BOX(hbox), curr_path, FALSE, FALSE, 0);

/* SECOND ROW - sub directory & file listings */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fl_data->win)->vbox), hbox, TRUE, TRUE, 0);

/* PATH - scrolled model pane */
swin = gtk_scrolled_window_new(NULL, NULL);
gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
gtk_box_pack_start(GTK_BOX(hbox), swin, TRUE, TRUE, 0);
/* Create the CList */
path_list = gtk_clist_new(1);
/* event */
gtk_signal_connect(GTK_OBJECT(path_list), "select_row",
                   GTK_SIGNAL_FUNC(path_handler), NULL);

/* set some properties of the CList */
gtk_clist_set_column_width(GTK_CLIST(path_list), 0, path_width);
gtk_clist_set_column_title(GTK_CLIST(path_list), 0, "Tree");
gtk_clist_column_titles_show(GTK_CLIST(path_list));
gtk_clist_column_titles_passive(GTK_CLIST(path_list));
gtk_container_add(GTK_CONTAINER(swin), path_list);
gtk_container_set_border_width (GTK_CONTAINER (swin), 10);

/* FILE - scrolled model pane */
swin = gtk_scrolled_window_new(NULL, NULL);
gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW (swin),
                               GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
gtk_box_pack_start(GTK_BOX(hbox), swin, TRUE, TRUE, 0);
/* Create the CList */
file_list = gtk_clist_new(1);
/* event */
gtk_signal_connect(GTK_OBJECT(file_list), "select_row",
                   GTK_SIGNAL_FUNC(file_handler), NULL);

/* set some properties of the CList */
gtk_clist_set_column_width(GTK_CLIST(file_list), 0, file_width);
gtk_clist_set_column_title(GTK_CLIST(file_list), 0, "Files");
gtk_clist_column_titles_show(GTK_CLIST(file_list));
gtk_clist_column_titles_passive(GTK_CLIST(file_list));
gtk_container_add(GTK_CONTAINER(swin), file_list);
gtk_container_set_border_width (GTK_CONTAINER (swin), 10);

/* THIRD ROW - filename currently selected & file extension filter */
hbox = gtk_hbox_new(FALSE,5);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fl_data->win)->vbox), hbox, FALSE, FALSE, 0);

/* filename */
file_name = gtk_entry_new();
gtk_box_pack_start(GTK_BOX (hbox), file_name, TRUE, TRUE, 0);
gtk_entry_set_editable(GTK_ENTRY (file_name), TRUE);
/* hook a <CR> event to the load action */
gtk_signal_connect(GTK_OBJECT (file_name), "activate", 
                   GTK_SIGNAL_FUNC (file_load), NULL);

/* file extension filter */
filter= gtk_option_menu_new();
gtk_widget_ref(filter);
gtk_box_pack_start(GTK_BOX (hbox), filter, FALSE, FALSE, 0);
filter_menu = gtk_menu_new();

/* NB: these should be sequential in the gdis.h enum{} */
item = make_filter_item ("all known", (gpointer *) DATA);
gtk_menu_append (GTK_MENU(filter_menu), item);

item = make_filter_item ("filterable", (gpointer *) BABEL);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("biosym", (gpointer *) BIOSYM);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("gulp", (gpointer *) GULP);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("marvin", (gpointer *) MARVIN);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("morphology", (gpointer *) MORPH);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("xtl", (gpointer *) XTL);
gtk_menu_append (GTK_MENU (filter_menu), item);

item = make_filter_item ("xyz", (gpointer *) XYZ);
gtk_menu_append (GTK_MENU (filter_menu), item);

gtk_option_menu_set_menu(GTK_OPTION_MENU (filter), filter_menu);

/* NB: if the above are not sequential, this will select the wrong item */
gtk_option_menu_set_history(GTK_OPTION_MENU(filter), sysenv.file_type - DATA);
gtk_menu_set_active(GTK_MENU(filter_menu), sysenv.file_type - DATA);

/* babel filter */
babel = gtk_option_menu_new();
gtk_widget_ref(babel);
gtk_box_pack_start(GTK_BOX (hbox), babel, FALSE, FALSE, 0);
babel_menu = gtk_menu_new();

item = make_babel_item("no filter", (gpointer *) NONBABEL);
gtk_menu_append(GTK_MENU(babel_menu), item);

item = make_babel_item("CSD cssr", (gpointer *) CSSR);
gtk_menu_append(GTK_MENU(babel_menu), item);

item = make_babel_item("CSD fdat", (gpointer *) FDAT);
gtk_menu_append(GTK_MENU(babel_menu), item);

item = make_babel_item("Gamess output", (gpointer *) GAMOUT);
gtk_menu_append(GTK_MENU(babel_menu), item);

item = make_babel_item("Mopac output", (gpointer *) MOPAC_OUT);
gtk_menu_append(GTK_MENU(babel_menu), item);

item = make_babel_item("PDB", (gpointer *) PDB);
gtk_menu_append(GTK_MENU(babel_menu), item);

gtk_option_menu_set_menu(GTK_OPTION_MENU (babel), babel_menu);
gtk_option_menu_set_history(GTK_OPTION_MENU(babel), sysenv.babel_type - NONBABEL);
gtk_menu_set_active(GTK_MENU(babel_menu), sysenv.babel_type - NONBABEL);

/* LAST ROW - terminating buttons */
hbox = gtk_hbox_new(FALSE,10);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(fl_data->win)->action_area), hbox,
                                                     FALSE, TRUE, 0);

button = gtk_button_new_with_label ("     Ok    ");
gtk_box_pack_start (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (file_load), NULL);

button = gtk_button_new_with_label ("  Cancel  ");
gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT(button), "clicked",
                           GTK_SIGNAL_FUNC(close_dialog),
                          (gpointer) id);
/* all done */
gtk_widget_show_all(fl_data->win);
update_file_pane();
}

/********************/
/* save file action */
/********************/
/* no longer used */
void file_save()
{
struct model_pak *data;

if (sysenv.num_models)
  {
  data = model_ptr(sysenv.active, RECALL);
  if (!data)
    {
    printf("file_save(): failed to get model pointer!\n");
    return;
    }
  file_action(data->filename, FILE_SAVE, sysenv.active);
  }
}

/***********************/
/* save as file action */
/***********************/
void file_save_as(GtkWidget *w, GtkWidget *fs)
{
gchar *inp_file;

/* get selected name and check */
inp_file = (gchar *) gtk_file_selection_get_filename (GTK_FILE_SELECTION (fs));
if (!strlen(inp_file))
  return;
/* paranoid check */
if (sysenv.num_models)
  file_action(inp_file, FILE_SAVE_AS, sysenv.active);
 
/* finished saving data - only now we destroy the file select widget */
gtk_widget_destroy(fs);
}

/******************/
/* file - save as */
/******************/
void save_as_widget()
{
GtkWidget *fs_widget;
struct model_pak *data;

/* return if nothing is loaded */
if (!sysenv.num_models)
  return;

fs_widget = gtk_file_selection_new ("File save as");
gtk_window_set_position(GTK_WINDOW(fs_widget), GTK_WIN_POS_CENTER);
gtk_file_selection_hide_fileop_buttons(GTK_FILE_SELECTION(fs_widget));
 
/* setup events for the file selection widget */
gtk_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (fs_widget)->ok_button),
                    "clicked", (GtkSignalFunc) file_save_as, fs_widget);

gtk_signal_connect_object (GTK_OBJECT (GTK_FILE_SELECTION 
                          (fs_widget)->cancel_button), "clicked",
                          (GtkSignalFunc) gtk_widget_destroy,
                           GTK_OBJECT (fs_widget));

/* set default filename to save as */
data = model_ptr(sysenv.active, RECALL);
gtk_file_selection_set_filename(GTK_FILE_SELECTION(fs_widget), data->filename);
gtk_widget_show(fs_widget);
}

