/* 
   Copyright (C) 1999 Kyle R. Burton, All Rights Reserved
   mortis@voicenet.com
   http://www.bgw.org
   http://www.voicenet.com/~mortis

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
*/

/*
Modified by Sean Fleming for shoehorning into GDIS, Feb 2001
*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <string.h>
#include <unistd.h>
#include <assert.h>
#include <errno.h>
#include <gtk/gtk.h>

#include "gperiodic.h"
#include "table_data.h"

#include "gdis.h"

extern struct sysenv_pak sysenv;
extern struct elem_pak elements[];

extern GtkWidget *window;

gchar *optstr            = "hl:sd:c";    /* cmdline option string... */
gchar *bin;                              /* get name from argv... */
gint  enable_colorscale  = 0;

/* convert element symbol to gdis atom code */
#define GET_CODE(i) (elem_type(table[i].info[SYMBOL]))

void dialog_destroy(GtkWidget *w)
{
gtk_grab_remove(GTK_WIDGET(w));
}

/*****************/
/* spinner hooks */
/*****************/
void change_cova(GtkWidget *w, gpointer *spin)
{
gint code;
gfloat r;
GtkWidget *parent;

/* retrieve title from top level widget, which */
/* should be the element to apply the change to */
parent = gtk_widget_get_toplevel(GTK_WIDGET(spin));
code = elem_type(GTK_WINDOW(parent)->title);
r = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));

elements[code].cova = r;
}

void change_vdw(GtkWidget *w, gpointer *spin)
{
gint code;
gfloat r;
GtkWidget *parent;

parent = gtk_widget_get_toplevel(GTK_WIDGET(spin));
code = elem_type(GTK_WINDOW(parent)->title);
r = gtk_spin_button_get_value_as_float(GTK_SPIN_BUTTON(spin));

elements[code].vdw = r;
}

/***********************************/
/* on the fly table colour refresh */
/***********************************/
void refresh_table()
{
gint i, j;
GtkStyle *style;

for(i=0 ; i<sizeof(table) ; i++) 
  {
/* stop if no data */
  if (!table[i].info[SYMBOL]) 
    break;
/* jump the gaps */
  if (GTK_IS_WIDGET(table[i].button)) 
    {
    j = GET_CODE(i);
    style = gtk_style_copy(gtk_widget_get_style(window));
    style->bg[GTK_STATE_NORMAL].red = elements[j].colour[0];
    style->bg[GTK_STATE_NORMAL].green = elements[j].colour[1];
    style->bg[GTK_STATE_NORMAL].blue = elements[j].colour[2];
    gtk_widget_set_style(table[i].button, style);
    }
  }
}

/*****************************************/
/* make a change to an elements's colour */ 
/*****************************************/
void elem_colour_change(GtkWidget *w, gpointer *obj)
{
gdouble colour[3];
gpointer *widget;
GtkStyle *style;
struct model_pak *data;
struct elem_pak *elem_data;

/* checks */
g_return_if_fail(obj != NULL);

/* get colour from csd */
gtk_color_selection_get_color((GtkColorSelection *) obj, colour);
VEC3MUL(colour, 65535);

/* retreive the element & update */
elem_data = gtk_object_get_data(GTK_OBJECT(obj), "ptr");
ARR3SET(elem_data->colour, colour);

/* get style copy & update single element colour sample */
widget = gtk_object_get_data(GTK_OBJECT(obj), "wid");
/* check, since the user might have since deleted the single element dialog */
if (GTK_IS_WIDGET(widget))
  {
  style = gtk_style_copy(gtk_widget_get_style(w));
  style->bg[0].red   = colour[0];
  style->bg[0].green = colour[1]; 
  style->bg[0].blue  = colour[2]; 
  gtk_widget_set_style(GTK_WIDGET(widget), style);
  }
/* update entire table */
refresh_table();

/* refresh atom colours */
data = model_ptr(sysenv.active, RECALL);
if (data)
  {
  init_objs(REFRESH_COLOUR, data);
/* redo region lighting (if any) */
  if (sysenv.mode[sysenv.active] == REGION_LITE)
    init_objs(REFRESH_LITE, data);
  }

/* redraw */
wire_draw(SINGLE);
}

/********************************/
/* reset all values to defaults */
/********************************/
void reset_elem_colours()
{
gint i;
struct model_pak *data;

/* reset element data */
elem_init();
refresh_table();

/* refresh all models */
for (i=0 ; i<sysenv.num_models ; i++)
  {
  data = model_ptr(i, RECALL);
/* redo atom colours */
  init_objs(REFRESH_COLOUR, data);
/* redo region lighting (if needed) */
  if (sysenv.mode[i] == REGION_LITE)
    init_objs(REFRESH_LITE, data);
  }

/* redraw */
wire_draw(ALL);
}

/***************************/
/* Single element dialogue */
/***************************/
void display_element_dialog(GtkWidget *w, gint i)
{
struct table_entry *entry;
GtkWidget *dialog, *vbox, *hbox, *tbl;
gint code, col;
GtkWidget *label, *button, *cr_spin, *vr_spin;
GtkStyle *style, *newstyle;
GtkAdjustment *adj;

entry = (struct table_entry *) &table[i];

/* get element's GDIS code */
code = GET_CODE(i);

/* display this in a dialog box... */
dialog = gtk_dialog_new();
gtk_signal_connect(GTK_OBJECT(dialog),"destroy",
                   GTK_SIGNAL_FUNC(dialog_destroy), dialog);
gtk_window_set_title(GTK_WINDOW(dialog), elements[code].element);
gtk_container_set_border_width(GTK_CONTAINER(dialog), 5 );

vbox = gtk_vbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->vbox),
         vbox, TRUE, TRUE, 0 );

/* add 3 more rows - vdW,cov radii & colour */
tbl = gtk_table_new(MAX_INFO_NR+3, 2,FALSE);
gtk_box_pack_start(GTK_BOX(vbox), tbl, FALSE, FALSE, 0 );

/* add the labels to the dialog box verticaly... */
for (col = 0; col < 2; col++) 
  {
  int row;
  for (row = 0; row < MAX_INFO_NR; row++) 
    {
    GtkWidget *label;

    if (col == 0)
      label = gtk_label_new(_(header.info[row]));
    else if (row == 0)
      label = gtk_label_new(_(entry->info[row]));
    else
      label = gtk_label_new(entry->info[row]);

    gtk_label_set_justify(GTK_LABEL(label), GTK_JUSTIFY_LEFT);
    gtk_table_attach(GTK_TABLE(tbl), label,
           col, col + 1, row, row + 1,
           GTK_FILL, GTK_FILL, 0, 0);
    }
  }

/* add the extra GDIS related info */
gtk_widget_realize(dialog);
style = gtk_widget_get_style(dialog);

/* cova radius */
label = gtk_label_new ("Covalent radius:");
gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, MAX_INFO_NR, MAX_INFO_NR+1,
                 GTK_FILL, GTK_FILL, 0, 0);
adj = (GtkAdjustment *) gtk_adjustment_new
                        (elements[code].cova, 0.1, 3.0, 0.1, 0.1, 0);
/* data to pass to callback */
cr_spin = gtk_spin_button_new (adj, 0.1, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(cr_spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(cr_spin), GTK_SHADOW_OUT);
gtk_table_attach(GTK_TABLE(tbl), cr_spin, 1, 2, MAX_INFO_NR, MAX_INFO_NR+1,
                 GTK_FILL, GTK_FILL, 0, 0);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (change_cova), (gpointer) cr_spin);


/* vdW radius */
label = gtk_label_new ("vdW radius:");
gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, MAX_INFO_NR+1, MAX_INFO_NR+2,
                 GTK_FILL, GTK_FILL, 0, 0);
adj = (GtkAdjustment *) gtk_adjustment_new
                        (elements[code].vdw, 0.1, 4.0, 0.1, 0.1, 0);
/* data to pass to callback */
vr_spin = gtk_spin_button_new(adj, 0.1, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(vr_spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(vr_spin), GTK_SHADOW_OUT);
gtk_table_attach(GTK_TABLE(tbl), vr_spin, 1, 2, MAX_INFO_NR+1, MAX_INFO_NR+2,
                 GTK_FILL, GTK_FILL, 0, 0);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (change_vdw), (gpointer) vr_spin);


/* Colour */
label = gtk_label_new ("Colour: ");
gtk_table_attach(GTK_TABLE(tbl), label, 0, 1, MAX_INFO_NR+2, MAX_INFO_NR+3,
                 GTK_FILL, GTK_FILL, 0, 0);

/* button with element's colour, click to edit */
button = gtk_button_new_with_label("    edit    ");
newstyle = gtk_style_copy(style);

newstyle->bg[0].red   = elements[code].colour[0];
newstyle->bg[0].green = elements[code].colour[1];
newstyle->bg[0].blue  = elements[code].colour[2];
/* update!!! */
gtk_widget_set_style(button, newstyle);

gtk_table_attach(GTK_TABLE(tbl), button, 1, 2, MAX_INFO_NR+2, MAX_INFO_NR+3,
                 0, 0, 0, 0);

gtk_signal_connect(GTK_OBJECT (button), "clicked",
                   GTK_SIGNAL_FUNC(colsel_dialog), (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "id", (gpointer) ELEMENT_COLOUR);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) &elements[code]);


hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dialog)->action_area), hbox,
                                                     FALSE, TRUE, 0);
button = gtk_button_new_with_label ("  Close  ");
gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, FALSE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(gtk_widget_destroy),
                          GTK_OBJECT(dialog));

gtk_widget_show_all(dialog);
}

gint criterion2scale(double r, int maxcolor)
{
gint result;

if(r < 0)
  return 0;
result = 1 + (maxcolor-1) * r;
if (result >= maxcolor)
  result = maxcolor - 1;

return result;
}

/******************************/
/* Construct a periodic table */
/******************************/
void gperiodic_win(void)
{
gint i, j, id;
gchar *buff=NULL;
GtkWidget *label, *vbox, *periodic_table, *hbox, *button;
GdkColor tooltip_color_bg, tooltip_color_fg;
GtkStyle *style;
struct dialog_pak *gperiodic;

/* retrieve a suitable dialog */
if ((id = request_dialog(-1, GPERIODIC)) < 0)
  return;
gperiodic = &sysenv.dialog[id];

/* create the main window */
gperiodic->win = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW(gperiodic->win),"GPeriodic");
gtk_signal_connect(GTK_OBJECT(gperiodic->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

/* background colour for GPeriodic - LEMON */
style = gtk_widget_get_style(gperiodic->win);
style->bg[0].red = 64890;
style->bg[0].green = 65535;
style->bg[0].blue = 53739;
gtk_widget_set_style(gperiodic->win, style);

/* use a vbox for the menubar and the table of elements... */
vbox = gtk_vbox_new(FALSE,0);
gtk_container_border_width(GTK_CONTAINER(vbox),2);
gtk_container_add (GTK_CONTAINER(GTK_BOX(GTK_DIALOG(gperiodic->win)->vbox)),vbox);

/* nice heading */
label = gtk_label_new("Periodic table of the Elements");
gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0 );

/* create the table widget to hold the periodic table */
periodic_table = gtk_table_new(1,1,TRUE);
gtk_box_pack_start( GTK_BOX(vbox), periodic_table, FALSE, FALSE, 0 );

/* now for each element in the table of elements, create a display */
/* item for it, and add it to the table... */
for(i=0 ; i<sizeof(table) ; i++ ) 
  {
/* don't display over 108 (actual 109) */
  if (i > 108)
    continue;
/* stop if no data... */
  if( !table[i].info[SYMBOL] ) 
    break;
/* create the button */
  table[i].button = gtk_button_new_with_label(_(table[i].info[SYMBOL]));
  if( !table[i].button ) 
    {
    g_warning("%s(%d) %s", __FILE__, __LINE__, _("Error button was null?"));
    break;
    }

/******************************************************************************/
  {
  GtkStyle *button_style = gtk_widget_get_style(gperiodic->win);

  table[i].style = gtk_style_copy(button_style);

/* set for 0, 1, and 2 */
/* NEW - assign the GDIS colour */
  j = GET_CODE(i);
/* unlit */
  table[i].style->bg[0].red   = elements[j].colour[0];
  table[i].style->bg[0].green = elements[j].colour[1];
  table[i].style->bg[0].blue  = elements[j].colour[2];
/* lit */

/*
  table[i].style->bg[1].red   = table[i].color.red;
  table[i].style->bg[1].green = table[i].color.green;
  table[i].style->bg[1].blue  = table[i].color.blue;

  table[i].style->bg[2].red   = table[i].color.red;
  table[i].style->bg[2].green = table[i].color.green;
  table[i].style->bg[2].blue  = table[i].color.blue;
*/

  gtk_widget_set_style(table[i].button, table[i].style );
}
/******************************************************************************/

/* connect the destroy method to it */
  gtk_signal_connect(GTK_OBJECT(table[i].button), "clicked",
                     GTK_SIGNAL_FUNC(display_element_dialog), (gpointer) i);

/* set up a string for the tooltips */
  buff = g_strdup_printf(_("%s  n:%s w:%s"), _(table[i].info[NAME]),
                         table[i].info[NUMBER], table[i].info[WEIGHT]);

/* create a new tooltips object... */
  table[i].tooltip = gtk_tooltips_new();
  gtk_tooltips_set_delay(table[i].tooltip,100);
  gtk_tooltips_set_tip(table[i].tooltip,table[i].button,buff,NULL);
  gtk_tooltips_set_colors(table[i].tooltip,&tooltip_color_bg,&tooltip_color_fg);

/* attach the button to the table */
  gtk_table_attach(GTK_TABLE(periodic_table), table[i].button,
                   table[i].x - 1, table[i].x,
                   table[i].y - 1, table[i].y,
                   GTK_FILL, GTK_FILL,              /* opts */
                   2, 2);                           /* ypad */
  }

/* terminating buttons */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX(GTK_DIALOG(gperiodic->win)->action_area),
                    hbox, TRUE, FALSE, 0);

button = gtk_button_new_with_label("    Reset    ");
gtk_widget_set_style(button, style);
gtk_signal_connect(GTK_OBJECT(button), "clicked",
                   GTK_SIGNAL_FUNC(reset_elem_colours), NULL);
gtk_box_pack_start (GTK_BOX(hbox), button, FALSE, FALSE, 0);

button = gtk_button_new_with_label("    Close    ");
gtk_widget_set_style(button, style);
gtk_box_pack_start (GTK_BOX(hbox), button, FALSE, FALSE, 0);
gtk_signal_connect(GTK_OBJECT(button), "clicked",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

/* show all the widgets */
gtk_widget_show_all(gperiodic->win);
free(buff);
}
