/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include "gdis.h"

#define DEBUG_INVERT 0

/*************************/
/* multiply two matrices */
/*************************/
void matmat(gfloat *mat1, gfloat *mat2)
/* mat2 -> mat1*mat2 */
{
gint i;
gfloat tmp[9], *res, *ptr1, *ptr2;

/* init */
transpose(mat2);
res = tmp;
ptr1 = mat1;

/* mult - loop over rows of mat 1*/
for (i=0 ; i<3 ; i++)
  {
  ptr2 = mat2;

  *res = *(ptr1++) * (*(ptr2++));
  *res += *(ptr1++) * (*(ptr2++));
  *(res++) += *(ptr1--) * (*(ptr2++));
  ptr1--;

  *res = *(ptr1++) * (*(ptr2++));
  *res += *(ptr1++) * (*(ptr2++));
  *(res++) += *(ptr1--) * (*(ptr2++));
  ptr1--;

  *res = *(ptr1++) * (*(ptr2++));
  *res += *(ptr1++) * (*(ptr2++));
  *(res++) += *(ptr1++) * (*ptr2);
  }

/* copy result to mat2 */
memcpy(mat2,tmp,9*sizeof(gfloat));

return;
}

/****************************************/
/* transform a 3D vector by matrix mult */
/****************************************/
void vecmat(gfloat *mat, gfloat *vec)
{
gfloat x, y, z;
gfloat *mptr=mat, *vptr=vec;

/* init */
x = *(vptr++);
y = *(vptr++);
z = *(vptr--);
/* mult */
*(--vptr) = *(mptr++) * x;
*vptr += *(mptr++) * y;
*(vptr++) += *(mptr++) * z;
*vptr = *(mptr++) * x;
*vptr += *(mptr++) * y;
*(vptr++) += *(mptr++) * z;
*vptr = *(mptr++) * x;
*vptr += *(mptr++) * y;
*vptr += *(mptr++) * z;
}

/**********************/
/* transpose a matrix */
/**********************/
void transpose(gfloat *mat)
{
gfloat tmp[9], *ptr=mat;

memcpy(tmp, mat, 9*sizeof(gfloat));
ptr++; 
*(ptr++) = tmp[3]; 
*(ptr++) = tmp[6]; 
*(ptr++) = tmp[1]; 
ptr++;
*(ptr++) = tmp[7]; 
*(ptr++) = tmp[2]; 
*ptr     = tmp[5]; 
}

/*******************/
/* Invert a matrix */
/*******************/
/* Gauss-Jordan */
gint invmat(gfloat *mat, gint dim)
{
gint i,j,k,ix,ref,refrow,x,y;
gfloat mult, *new;

new = (gfloat *) g_malloc(dim*dim*sizeof(gfloat));

/* TODO - checks? */

/* setup the unit matrix */
*(new+0) = 1.0;
*(new+1) = 0.0;
*(new+2) = 0.0;
*(new+3) = 0.0;
*(new+4) = 1.0;
*(new+5) = 0.0;
*(new+6) = 0.0;
*(new+7) = 0.0;
*(new+8) = 1.0;

#if DEBUG_INVERT
printf("*** Start ***\n");
printf("| %5.2f %5.2f %5.2f |",*mat,*(mat+1),*(mat+2));
printf("| %5.2f %5.2f %5.2f |\n",*new,*(new+1),*(new+2));
printf("| %5.2f %5.2f %5.2f |",*(mat+3),*(mat+4),*(mat+5));
printf("| %5.2f %5.2f %5.2f |\n",*(new+3),*(new+4),*(new+5));
printf("| %5.2f %5.2f %5.2f |",*(mat+6),*(mat+7),*(mat+8));
printf("| %5.2f %5.2f %5.2f |\n",*(new+6),*(new+7),*(new+8));
#endif

/*******************/
/* pass one - L->0 */
/*******************/

/* loop over the rows (down) */
for(i=1 ; i<dim ; i++)
  {
/* diagonal element to compare against to get the multiplier */
  ref = 0;
  refrow = 0;
/* loop along the row */
  for (j=0 ; j<i ; j++)
    {
    ix = i*dim+j;
/* calc multiplier to make ix 0 */
    if (*(mat+ref))
      mult = (gfloat) (*(mat+ix))/(gfloat) (*(mat+ref));
    else
      mult = 0.0; 
/* change the row */
    for (k=0 ; k<dim ; k++)
      {
      x = i*dim+k;
      y = refrow*dim+k;
      *(mat+x) -= mult*(*(mat+y));
      *(new+x) -= mult*(*(new+y));
      }
    
/* change ref  - slide down the diagonal */
    ref += dim+1;
    refrow++;
    }
  }

#if DEBUG_INVERT
printf("*** Pass 1 ***\n");
printf("| %5.2f %5.2f %5.2f |",*mat,*(mat+1),*(mat+2));
printf("| %5.2f %5.2f %5.2f |\n",*new,*(new+1),*(new+2));
printf("| %5.2f %5.2f %5.2f |",*(mat+3),*(mat+4),*(mat+5));
printf("| %5.2f %5.2f %5.2f |\n",*(new+3),*(new+4),*(new+5));
printf("| %5.2f %5.2f %5.2f |",*(mat+6),*(mat+7),*(mat+8));
printf("| %5.2f %5.2f %5.2f |\n",*(new+6),*(new+7),*(new+8));
#endif

/*******************/
/* pass two - U->0 */
/*******************/

/* loop over the rows (up) */
for(i=dim-2 ; i>=0 ; i--)
  {
/* diagonal element to compare against to get the multiplier */
  ref = dim*dim - 1;
  refrow = dim-1;
/* loop along the row */
  for (j=dim-1 ; j>i ; j--)
    {
    ix = i*dim+j;
/* calc multiplier to make ix 0 */
    if (*(mat+ref))
      mult = (gfloat) (*(mat+ix))/(gfloat) (*(mat+ref));
    else
      mult = 0.0;
/* change the row */
    for (k=0 ; k<dim ; k++)
      {
      x = i*dim+k;
      y = refrow*dim+k;
      *(mat+x) -= mult*(*(mat+y));
      *(new+x) -= mult*(*(new+y));
      }
    
/* change ref  - slide up the diagonal */
    ref -= dim+1;
    refrow--;
    }
  }

#if DEBUG_INVERT
printf("*** Pass 2 ***\n");
printf("| %5.2f %5.2f %5.2f |",*mat,*(mat+1),*(mat+2));
printf("| %5.2f %5.2f %5.2f |\n",*new,*(new+1),*(new+2));
printf("| %5.2f %5.2f %5.2f |",*(mat+3),*(mat+4),*(mat+5));
printf("| %5.2f %5.2f %5.2f |\n",*(new+3),*(new+4),*(new+5));
printf("| %5.2f %5.2f %5.2f |",*(mat+6),*(mat+7),*(mat+8));
printf("| %5.2f %5.2f %5.2f |\n",*(new+6),*(new+7),*(new+8));
#endif

/* set the diagonals to unity */
ref=0;
for(i=0 ; i<dim ; i++)
  {
  if (*(mat+ref))
    mult = 1.0/(*(mat+ref)); 
  else
    mult = 1.0;
/* start of the row */
  ix = i*dim;
  *(mat+ref) *= mult;
  for (j=0 ; j<dim ; j++)
    *(new+ix+j) *= mult;
/* slide down the diagonal */
  ref += dim+1;
  }

#if DEBUG_INVERT
printf("*** Completed ***\n");
printf("| %5.2f %5.2f %5.2f |",*mat,*(mat+1),*(mat+2));
printf("| %5.2f %5.2f %5.2f |\n",*new,*(new+1),*(new+2));
printf("| %5.2f %5.2f %5.2f |",*(mat+3),*(mat+4),*(mat+5));
printf("| %5.2f %5.2f %5.2f |\n",*(new+3),*(new+4),*(new+5));
printf("| %5.2f %5.2f %5.2f |",*(mat+6),*(mat+7),*(mat+8));
printf("| %5.2f %5.2f %5.2f |\n",*(new+6),*(new+7),*(new+8));
#endif

/* TODO - checks??? */

/* assign the inverse to the input matrix */
memcpy(mat,new,dim*dim*sizeof(gfloat));

g_free(new);
return(0);
}

/* TODO - replace the two below with 2D & 3D macros? */
/****************************/
/* general vector magnitude */
/****************************/
gfloat magnitude(gfloat *vec, gint dim)
{
gint i;
gfloat sum=0.0, *ptr=vec;

for (i=0 ; i<dim ; i++)
  sum += (*ptr) * (*ptr++);

return(sqrt(sum));
}

/********************************/
/* general vector normalization */
/********************************/
gint normalize(gfloat *vec, gint dim)
{
gint i;
gfloat len, *ptr=vec;

len = magnitude(vec, dim);
if (len < FRACTION_TOLERANCE)
  return(1);

for (i=0 ; i<dim ; i++)
  *ptr++ /= len;

return(0);
}

/*****************************/
/* vector intersection angle */
/*****************************/
/* TODO - eliminate the slow acos by using the cos of the angle */
/*        instead; may not be possible for all calling routines... */
gfloat via(gfloat *vec1, gfloat *vec2, gint dim)
{
gint i;
gfloat lenprod, dot=0.0;
gfloat *ptr1=vec1, *ptr2=vec2;

for (i=0 ; i<dim ; i++)
  dot += (*ptr1++) * (*ptr2++);

lenprod = magnitude(vec1, dim) * magnitude(vec2, dim);

if (lenprod > FRACTION_TOLERANCE)
  return(acos(dot/lenprod));

return(0.0);
}

/*****************/
/* cross product */
/*****************/
void crossprod(gfloat *res, gfloat *vec1, gfloat *vec2)
{
*(res+0) = *(vec1+1) * *(vec2+2) - *(vec1+2) * *(vec2+1);
*(res+1) = *(vec1+2) * *(vec2+0) - *(vec1+0) * *(vec2+2);
*(res+2) = *(vec1+0) * *(vec2+1) - *(vec1+1) * *(vec2+0);
/*
printf("%5.1f,%5.1f,%5.1f X %5.1f,%5.1f,%5.1f = %5.1f,%5.1f,%5.1f\n"
                                         ,*(vec1+0),*(vec1+1),*(vec1+2)
                                         ,*(vec2+0),*(vec2+1),*(vec2+2)
                                         ,*(res+0),*(res+1),*(res+2));
*/
}

/********************************************/
/* construct matrix for rotation about axis */
/********************************************/
/* NB: vec is assumed to point from the origin */
/* and is the normal for a mirror, and the axis for a rotation */
#define DEBUG_GET_MATRIX 0
void get_matrix(gfloat *mat, gint type, gdouble *vec, gint order) 
{
gfloat proj, len, angle;
gfloat mop[9];

#if DEBUG_GET_MATRIX
P3VEC("direction vector: ",vec);
#endif

switch(type)
  {
/* proper rotation axis */
  case PAXIS:
#if DEBUG_GET_MATRIX
printf("constructing for order %d axis...\n", order);
#endif
/* constants */
    len = VEC3MAG(vec);
    proj = sqrt(vec[1]*vec[1] + vec[2]*vec[2]);
    angle = 2.0*PI/order;

/* init the return matrix */
    VEC3SET(&mat[0], 1.0, 0.0, 0.0);
    VEC3SET(&mat[3], 0.0, 1.0, 0.0);
    VEC3SET(&mat[6], 0.0, 0.0, 1.0);

/* yz plane coincidence */
    VEC3SET(&mop[0], 1.0, 0.0, 0.0);
    VEC3SET(&mop[3], 0.0, vec[2]/proj, vec[1]/proj);
    VEC3SET(&mop[6], 0.0, -1.0*vec[1]/proj, vec[2]/proj);
    matmat(mop,mat);

/* z axis coincidence */
    VEC3SET(&mop[0], proj/len, 0.0, vec[0]/len);
    VEC3SET(&mop[3], 0.0, 1.0, 0.0);
    VEC3SET(&mop[6], -1.0*vec[0]/len, 0.0, proj/len);
    matmat(mop,mat);

/* rotation operation (about z) */
    VEC3SET(&mop[0], cos(angle), sin(angle), 0.0);
    VEC3SET(&mop[3], -1.0*sin(angle), cos(angle), 0.0);
    VEC3SET(&mop[6], 0.0, 0.0, 1.0);
    matmat(mop,mat);

/* z axis inverse */
    VEC3SET(&mop[0], proj/len, 0.0, -1.0*vec[0]/len);
    VEC3SET(&mop[3], 0.0, 1.0, 0.0);
    VEC3SET(&mop[6], vec[0]/len, 0.0, proj/len);
    matmat(mop,mat);

/* yz plane inverse */
    VEC3SET(&mop[0], 1.0, 0.0, 0.0);
    VEC3SET(&mop[3], 0.0, vec[2]/proj, -1.0*vec[1]/proj);
    VEC3SET(&mop[6], 0.0, vec[1]/proj, vec[2]/proj);
    matmat(mop,mat);
    break;

/* mirror plane */
  case PLANE:
#if DEBUG_GET_MATRIX
printf("NIY - constructing for mirror plane\n");
#endif
    break;

  default:
  }

}

