/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "gdis.h"

extern struct sysenv_pak sysenv;
extern struct elem_pak elements[];

/* globals for optimize - put in struct? */
/* TODO - also make arrays (ie one per file/dialog) */
GtkWidget *gin_entry, *lib_entry;
GtkWidget *name_entry, *energy_entry;
gchar *ginfile;

/*******************/
/* run a gulp file */
/*******************/
gint exec_gulp(gchar *input, gchar *output)
{
GString *cmd;

/* stop gulp's renaming (if it exists already) */
unlink(output);

/* TODO - scan output to print updates */
/* build & execute command line */
cmd = g_string_new(NULL);
g_string_sprintf(cmd,"%s < %s > %s",sysenv.gulp_exe,input,output);
system(cmd->str);

/* done */
g_string_free(cmd, TRUE);
return(0);
}

/*****************************/
/* generic gulp output parse */
/*****************************/
#define DEBUG_READ_GULP_OUTPUT 0
gint read_gulp_output(FILE *fp, struct model_pak *data)
{
gint i, m, n, flag, sflag=0;
gchar line[LINELEN], **buff;

/* TODO - keyword & name parse */
/* TODO - how to sensibly handle multiple structures */

/* look for interesting output */
flag=0;
while(!fgetline(fp,line))
  {
/* space group search */
  if (g_strncasecmp("  Space group ",line,14) == 0)
    {
    g_free(data->sginfo.spacename);
    data->sginfo.spacename = g_strstrip(g_strdup(get_token_pos(line,4)));
    sflag++;
    }

/* minimized energy search */
  if (g_strncasecmp("  Final energy =",line,16) == 0)
    {
    buff = get_tokens(line,4);
    sscanf(*(buff+3),"%f",&data->gulp.energy);
    flag++;
    g_strfreev(buff);
    }
/* search for gnorm */
  if (g_strncasecmp("  Final Gnorm  =",line,16) == 0)
    {
    buff = get_tokens(line,4);
    sscanf(*(buff+3),"%f",&data->gulp.gnorm);
    flag++;
    g_strfreev(buff);
    }

/* single point energy search */
  if (g_strncasecmp("  Total lattice energy",line,22) == 0)
    {
    buff = get_tokens(line,6);
/* if nothing on the rest of this line, then should be on the next */
    if (!strlen(*(buff+5)))
      {
      while(!fgetline(fp,line))
        if (g_strncasecmp("    Primitive unit cell",line,20) == 0)
          break;
      g_strfreev(buff);
      buff = get_tokens(line, 6);
      }
    if (g_strncasecmp("eV",*(buff+5),2) == 0)
      sscanf(*(buff+4),"%f",&data->gulp.energy);
    g_strfreev(buff);
    }

/* FIXME - unminimized coords? */

/* periodic coords search */
/* FIXME - what if cartesian (ie not fractional) ??? */
  if (g_strncasecmp("  Final asymmetric unit coordinates",line,35) == 0 ||
      g_strncasecmp("  Final fractional coordinates",line,30) == 0 )
    {
    data->periodic = 3;
    data->fractional = TRUE;
/* skip rubbish */
    for (i=0 ; i<5 ; i++)
      if (fgetline(fp,line))
        return(2);
/* loop until we run out of atomic data lines */
    i=n=0;
    flag=0;
/* read as many atoms as we can */
    while(!fgetline(fp,line))
      {
      buff = get_tokens(line,6);
/* exit if atom number is not consecutive */ 
      sscanf(*buff,"%d",&m);
      if (m != ++n)
        break;
      else
        {
/* read the data in - assume the same order */
/* core only */
        if (g_strncasecmp("c", *(buff+2), 1) == 0)
          {
          if (i >= data->atom_limit)
            mem_grow(data, ATOM, 10);

          (data->atoms+i)->status = NORMAL;
          (data->atoms+i)->primary = TRUE;
          (data->atoms+i)->orig = TRUE;
          (data->atoms+i)->region = REGION1A;
          (data->atoms+i)->offx = 0;
          (data->atoms+i)->offy = 0;

          sscanf(*(buff+1),"%s",(data->atoms+i)->element);
          sscanf(*(buff+1),"%s",(data->atoms+i)->label);
          sscanf(*(buff+3),"%f",&(data->atoms+i)->x);
          sscanf(*(buff+4),"%f",&(data->atoms+i)->y);
          sscanf(*(buff+5),"%f",&(data->atoms+i)->z);

          strcpy((data->atoms+i)->tail, "");

#if DEBUG_READ_GULP_OUTPUT
printf("coords: %f %f %f\n", 
      (data->atoms+i)->x, (data->atoms+i)->y, (data->atoms+i)->z);
#endif

/* elem setup (colour, code etc.) */
          elem_seek(i, ATOM, data);
          i++;
          }
/* TODO - SHELLS */
        }
      g_strfreev(buff);
      }
    data->num_asym = data->num_orig = data->num_atoms = i;
#if DEBUG_READ_GULP_OUTPUT
printf("Retrieved %d atoms (periodic).\n",i);
#endif
    }

/* cell paramaters - search 1 */
  if (g_strncasecmp("  Final cell parameters",line,22) == 0)
    {
/* skip to data */
    fgetline(fp,line);
    fgetline(fp,line);
/* get cell lengths */
    for (i=0 ; i<6 ; i++)
      {
      if (fgetline(fp,line))
        break;
      buff = get_tokens(line,3);
      sscanf(*(buff+1),"%f",&data->pbc[i]);
      g_strfreev(buff);
      }
/* convert to radians */
    data->pbc[3] *= PI/180.0;
    data->pbc[4] *= PI/180.0;
    data->pbc[5] *= PI/180.0;
    }

/* cell paramaters - search 2 */
  if (g_strncasecmp("  Non-primitive lattice parameters",line,34) == 0)
    {
/* skip to data */
    if (fgetline(fp,line))
      break;
    if (fgetline(fp,line))
      break;
/* get cell lengths */
    buff = get_tokens(line,9);
    sscanf(*(buff+2),"%f",&data->pbc[0]);
    sscanf(*(buff+5),"%f",&data->pbc[1]);
    sscanf(*(buff+8),"%f",&data->pbc[2]);
    if (fgetline(fp,line))
      break;
/* get cell angles */
    g_strfreev(buff);
    buff = get_tokens(line,6);
    sscanf(*(buff+1),"%f",&data->pbc[3]);
    sscanf(*(buff+3),"%f",&data->pbc[4]);
    sscanf(*(buff+5),"%f",&data->pbc[5]);
/* convert to radians */
    data->pbc[3] *= PI/180.0;
    data->pbc[4] *= PI/180.0;
    data->pbc[5] *= PI/180.0;
    g_strfreev(buff);
    }

/* isolated coords search */
  if (g_strncasecmp("  Final cartesian coordinates",line,29) == 0)
    {
    data->periodic = 0;
    data->fractional = 0;

    for (i=0 ; i<5 ; i++)
      fgetline(fp,line);
/* loop until we run out of atomic data lines */
    i=n=0;
    flag=0;
/* don't read any more than num_atoms -> memory allocation problems! */
printf("%s\n",line);
    while(!fgetline(fp,line))
      {
      buff = get_tokens(line,6);
/* exit if atom number is not consecutive */ 
      sscanf(*buff,"%d",&m);
      if (m != ++n)
        flag=1;
      else
        {
/* read the data in - assume the same order */
/* core only */
        if (g_strncasecmp("c",*(buff+2),1) == 0)
          {
          if (i >= data->atom_limit)
            mem_grow(data, ATOM, 10);

          (data->atoms+i)->status = NORMAL;
          (data->atoms+i)->primary = TRUE;
          (data->atoms+i)->orig = TRUE;
          (data->atoms+i)->region = REGION1A;
          (data->atoms+i)->offx = 0;
          (data->atoms+i)->offy = 0;

          sscanf(*(buff+1),"%s",(data->atoms+i)->element);
          sscanf(*(buff+1),"%s",(data->atoms+i)->label);
          sscanf(*(buff+3),"%f",&(data->atoms+i)->x);
          sscanf(*(buff+4),"%f",&(data->atoms+i)->y);
          sscanf(*(buff+5),"%f",&(data->atoms+i)->z);

          strcpy((data->atoms+i)->tail, "");

#if DEBUG_READ_GULP_OUTPUT
printf("coords: %f %f %f\n", 
      (data->atoms+i)->x, (data->atoms+i)->y, (data->atoms+i)->z);
#endif
          elem_seek(i, ATOM, data);
          i++;
          }
/* TODO - SHELLS */
        }
      g_strfreev(buff);
      }
    data->num_asym = data->num_orig = data->num_atoms = i;
#if DEBUG_READ_GULP_OUTPUT
printf("Retrieved %d atoms (cluster).\n",i);
#endif
    }
  }      /* while data in file */

#if DEBUG_READ_GULP_OUTPUT
P3VEC("cell lengths ",&data->pbc[0]);
P3VEC("cell angles ",&data->pbc[3]);
#endif

/* fractional? */
if (data->fractional)
  {
  for (i=0 ; i<data->num_atoms ; i++)
    {
/* store in case needed in the future */
    (data->atoms+i)->fx = (data->atoms+i)->x;
    (data->atoms+i)->fy = (data->atoms+i)->y;
    (data->atoms+i)->fz = (data->atoms+i)->z;
/* these are the primary coords (for the purpose of display) */
    (data->atoms+i)->x *= data->pbc[0];
    (data->atoms+i)->y *= data->pbc[1];
    (data->atoms+i)->z *= data->pbc[2];
    }
  }

/* init for display */
init_objs(AXES_ON, data);
if (data->periodic)
  {
  init_objs(CELL_ON, data);
  data->axes_type = OTHER;
/* space group init here */
  update_shells(data);
  if (!sflag)
    {
#if DEBUG_READ_GULP_OUTPUT
printf("No Space Group found: assuming P1.\n");
#endif
    data->sginfo.spacenum=1;
    }
  if (genpos(data))
    printf("Error in Space Group lookup.\n");
  }
init_objs(INIT_COORDS, data);
calc_bonds(data);
calc_mols(data);
pick_model(data->number);

new_tree_item(data->number, APPEND);

return(0);
}

/************************/
/* load a new gulp file */
/************************/
gint load_gulp_output(FILE *fp, gint model, gchar *inp_file)
{
struct model_pak *data;

/* alloc, setup & read */
data = model_ptr(model, ASSIGN);
if (data == NULL)
  return(1);

data->id = GULPOUT;
g_free(data->basename);
data->basename = strdup_no_extension(g_basename(inp_file));
strcpy(data->filename, g_basename(inp_file));
data->region_empty[REGION1A] = FALSE;

return(read_gulp_output(fp, data));
}

/***************************************************/
/* get single point energy from a GULP output file */
/***************************************************/
#define DEBUG_READ_GULP 0
gint read_single_gulp(gchar *file, struct model_pak *data)
{
gchar line[LINELEN], **buff;
FILE *fp;

/* open & scan gulp output file */
fp = fopen(file,"rt");
if (!fp)
  {
  printf("No output file found!\n");
  return(1);
  }

/* look for last (eV) energy output */
while(!fgetline(fp,line))
  { 
  buff = get_tokens(line, 6);
/* search for energy */
  if (g_strncasecmp("  Total lattice energy",line,22) == 0)
    {
/* if nothing on the rest of this line, then should be on the next */
    if (!strlen(*(buff+5)))
      {
      while(!fgetline(fp,line))
        if (g_strncasecmp("    Primitive unit cell",line,20) == 0)
          break;
      g_strfreev(buff);
      buff = get_tokens(line, 6);
      }

    if (g_strncasecmp("eV",*(buff+5),2) == 0)
      sscanf(*(buff+4),"%f",&data->gulp.energy);
    }
  g_strfreev(buff);
  }
/* done */
g_snprintf(line,35,"Single point Energy: %10.4f eV\n", data->gulp.energy);
show_text(line);
return(0);
}

/**********************************************/
/* get updated coords from a GULP output file */
/**********************************************/
/* TODO - merge with a general routine to read gulp output files */
#define DEBUG_READ_GULP 0
gint read_minimize_gulp(gchar *file, struct model_pak *data)
{
gint i,n,m,flag;
gfloat gnorm;
gchar line[LINELEN], **buff;
FILE *fp;

/* open & scan gulp output file */
fp = fopen(file,"rt");
if (!fp)
  {
  printf("No output file found!\n");
  return(1);
  }

/* look for interesting output */
flag=0;
while(!fgetline(fp,line) && flag<2)
  {
  buff = get_tokens(line,5);
/* search for energy */
  if (g_strncasecmp("  Final energy =",line,16) == 0)
    {
    sscanf(*(buff+3),"%f",&data->gulp.energy);
    flag++;
    }
/* search for gnorm */
  if (g_strncasecmp("  Final Gnorm  =",line,16) == 0)
    {
    sscanf(*(buff+3),"%f",&gnorm);
    flag++;
    }
  g_strfreev(buff);
  }

#if DEBUG_READ_GULP
printf("parse flag = %d\n",flag);
#endif

/* check result - setup for next search */
if (flag > 1)
  {
  if (data->periodic == 3)
    {
    g_snprintf(line,63,"Final energy: %10.4f eV, gnorm: %7.4f (primitive cell)",
                        data->gulp.energy, gnorm);
    }
  else
    {
    g_snprintf(line,45,"Final energy: %10.4f eV, gnorm: %7.4f",
                        data->gulp.energy, gnorm);
    }
  show_text(line);
  flag=0;
  }
else
  {
  g_snprintf(line,27,"GULP run was unsuccessful.");
  show_text(line);
  flag=1;
  }

/* find the data */
while(!fgetline(fp,line) && !flag)
  {
  if (g_strncasecmp("  Final cartesian coordinates",line,29) == 0)
    flag = 1;
  if (g_strncasecmp("  Final asymmetric unit coordinates",line,35) == 0)
    flag = 2;
  }

/* skip some rubbish */
if (flag == 1)
  {
  for (i=0 ; i<4 ; i++)
    fgetline(fp,line);

/* loop until we run out of atomic data lines */
  i=n=0;
  flag=0;
/* don't read any more than num_atoms -> memory allocation problems! */
  while(!flag && i<data->num_atoms)
    {
/* EOF check */
    if (fgetline(fp,line))
      break;
    buff = get_tokens(line,6);

/* exit if atom number is not consecutive */ 
    sscanf(*buff,"%d",&m);
    if (m != ++n)
      flag=1;
    else
      {
/* read the data in - assume the same order */
/* core only */
      if (g_strncasecmp("c",*(buff+2),1) == 0)
        {
        sscanf(*(buff+3),"%f",&(data->atoms+i)->x);
        sscanf(*(buff+4),"%f",&(data->atoms+i)->y);
        sscanf(*(buff+5),"%f",&(data->atoms+i)->z);
        i++;
        }
/* TODO - SHELLS */
      }
    g_strfreev(buff);
    }
#if DEBUG_READ_GULP
printf("Retrieved (%d of %d) atoms.\n",i,data->num_atoms);
#endif
  }
else if (flag == 2)
  {
/* skip rubbish */
  for (i=0 ; i<4 ; i++)
    fgetline(fp,line);
/* loop until we run out of atomic data lines */
  i=n=0;
  flag=0;
/* don't read any more than num_atoms -> memory allocation problems! */
  while(!flag && i<data->num_atoms)
    {
/* EOF check */
    if (fgetline(fp,line))
      break;

    buff = get_tokens(line,6);
/* exit if atom number is not consecutive */ 
    sscanf(*buff,"%d",&m);
    if (m != ++n)
      flag=1;
    else
      {
/* read the data in - assume the same order */
/* core only */
      if (g_strncasecmp("c", *(buff+2), 1) == 0)
        {
        sscanf(*(buff+3),"%f",&(data->atoms+i)->fx);
        sscanf(*(buff+4),"%f",&(data->atoms+i)->fy);
        sscanf(*(buff+5),"%f",&(data->atoms+i)->fz);
        i++;
        }
/* TODO - SHELLS */
      }
    g_strfreev(buff);
    }

/* update lattice params too */
  while(!fgetline(fp,line))
    {
    if (g_strncasecmp("  Non-primitive lattice parameters",line,34) == 0)
      {
      fgetline(fp,line);
      if (fgetline(fp,line))
        break;

      buff = get_tokens(line,9);
      sscanf(*(buff+2),"%f",&data->pbc[0]);
      sscanf(*(buff+5),"%f",&data->pbc[1]);
      sscanf(*(buff+8),"%f",&data->pbc[2]);
      if (fgetline(fp,line))
        break;

      g_strfreev(buff);
      buff = get_tokens(line,9);
      sscanf(*(buff+1),"%f",&data->pbc[3]);
      sscanf(*(buff+3),"%f",&data->pbc[4]);
      sscanf(*(buff+5),"%f",&data->pbc[5]);
/* convert to radians */
      data->pbc[3] *= PI/180.0;
      data->pbc[4] *= PI/180.0;
      data->pbc[5] *= PI/180.0;

      g_strfreev(buff);
      }
    }

/* rebuild full cell */
  if (data->num_asym == i)
    {
    data->num_atoms = i;
    for (i=0 ; i<data->num_atoms ; i++)
      {
      (data->atoms+i)->x = (data->atoms+i)->fx * data->pbc[0];
      (data->atoms+i)->y = (data->atoms+i)->fy * data->pbc[1];
      (data->atoms+i)->z = (data->atoms+i)->fz * data->pbc[2];
      }
    
    if (genpos(data))
      printf("Error in space group lookup.\n");
    }
/* FIXME - deleted atoms??? */
  else
    printf("Warning: fractional coordinate mismatch.\n");
  }
else
  {
  g_snprintf(line,30,"Failed to find output coords.\n");
  show_text(line);
  }

/* done */
fclose(fp);
return(0);
}

/***********************************************/
/* controls the use of GULP to optimise coords */
/***********************************************/
void run_gulp(gint id)
{
gint model;
gchar *gotfile, *libfile;
GString *line;
struct model_pak *data;

/* FIXME - get this from the model->gulp data */
/* FIXME - reflect dialog changes in model->gulp data */
/* get temp input file to use */
ginfile = (gchar *) gtk_entry_get_text (GTK_ENTRY (gin_entry));
/* make output file from input file */
gotfile = (gchar *) g_malloc((5+strlen(ginfile))*sizeof(gchar));
strcpy(gotfile,ginfile);
strcat(gotfile,".got");

/* get model attached to dialog id */
model = sysenv.dialog[id].model;
data = model_ptr(model, RECALL);

/* make an input file */
libfile = (gchar *) gtk_entry_get_text(GTK_ENTRY(lib_entry));
g_free(data->gulp.libfile);
data->gulp.libfile = g_strdup(libfile);
write_gulp(ginfile, data);
/* run & don't return until done */
exec_gulp(ginfile, gotfile);

/* get new coords */
if (data->gulp.run == MINIMIZE)
  read_minimize_gulp(gotfile, data);
else
  read_single_gulp(gotfile, data);

/* update energy (TODO - only if successful) */
line = g_string_new(NULL);
g_string_sprintf(line,"%f",data->gulp.energy);
gtk_entry_set_text(GTK_ENTRY(energy_entry), line->str);
g_string_free(line, TRUE);

/* update coords etc - INIT's */
init_objs(REDO_COORDS, data);  /* INIT coords? */
calc_bonds(data);
calc_mols(data);
wire_draw(SINGLE);

/* done */
g_free(gotfile);
}

/***********************/
/* GULP keyword toggle */
/***********************/
gint gulp_keyword(GtkWidget *w, gpointer *obj)
{
gint keyword;
struct model_pak *data;

keyword = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(obj), "key"));

data = (struct model_pak *) gtk_object_get_data(GTK_OBJECT(obj), "ptr");
g_return_val_if_fail(data != NULL, FALSE);

switch(keyword)
  {
  case ENERGY:
    data->gulp.run = ENERGY;
    break;
  case MINIMIZE:
    data->gulp.run = MINIMIZE;
    break;
  case PRESSURE:
    data->gulp.constant = PRESSURE;
    break;
  case VOLUME:
    data->gulp.constant = VOLUME;
    break;
  case MOLE:
    data->gulp.coulomb = MOLE;
    break;
  case MOLMEC:
    data->gulp.coulomb = MOLMEC;
    break;
  case MOLQ:
    data->gulp.coulomb = MOLQ;
    break;
  case MAXCYC:
    data->gulp.maxcyc = (gint) GTK_ADJUSTMENT(obj)->value;
    break;
  }

return(FALSE);
}

/*************************/
/* The GULP setup widget */
/*************************/
void gulp_widget(GtkWidget *w, struct model_pak *data)
{
gint id, model;
struct dialog_pak *gulp_dialog;
GtkWidget *main_hbox, *hbox, *vbox, *vbox1, *vbox2;
GtkWidget *frame, *button, *label;
GtkAdjustment *adj;
GSList *keyword[5];
GString *line;

/* do we have to do anything? */
g_return_if_fail(data != NULL);
if (!sysenv.have_gulp)
  {
  show_text("Sorry, gulp executable was not found.");
  return;
  }

/* NEW - default run setup is done by the template routine */
model = data->number;
pick_model(model);
if ((id = request_dialog(model, GULP)) < 0)
  return;
gulp_dialog = &sysenv.dialog[id];

line = g_string_new(NULL);

/* TODO - show model's name */
/* create a new dialogue */
gulp_dialog->win = gtk_dialog_new();
gtk_window_set_title (GTK_WINDOW(gulp_dialog->win), "GULP interface");
gtk_signal_connect(GTK_OBJECT(gulp_dialog->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

main_hbox = gtk_hbox_new(TRUE,3);
gtk_container_add(GTK_CONTAINER(GTK_DIALOG(gulp_dialog->win)->vbox),main_hbox);

/* left vbox */
vbox1 = gtk_vbox_new(FALSE, 0);
gtk_box_pack_start (GTK_BOX(main_hbox), vbox1, TRUE, TRUE, 0);

/* frame 1 */
frame = gtk_frame_new("run type");
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox1)),frame);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
vbox = gtk_vbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame),vbox);

/* do the first (DEFAULT MODE) radio button */
button = gtk_radio_button_new_with_label(NULL, "single point");
gtk_box_pack_start(GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) ENERGY);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.run == ENERGY)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* make a radio group */
keyword[0] = gtk_radio_button_group(GTK_RADIO_BUTTON(button));
/* do the next button */
button = gtk_radio_button_new_with_label (keyword[0], "minimize");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) MINIMIZE);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.run == MINIMIZE)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* frame 2 */
frame = gtk_frame_new("constant");
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox1)),frame);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
vbox = gtk_vbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame),vbox);
/* do the first (DEFAULT MODE) radio button */
button = gtk_radio_button_new_with_label (NULL, "pressure");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) PRESSURE);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.constant == PRESSURE)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* make a radio group */
keyword[1] = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
/* do the next button */
button = gtk_radio_button_new_with_label (keyword[1], "volume");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) VOLUME);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.constant == VOLUME)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* right vbox */
vbox2 = gtk_vbox_new(FALSE, 0);
gtk_box_pack_start (GTK_BOX(main_hbox), vbox2, TRUE, TRUE, 0);

/* frame 3 */
frame = gtk_frame_new("coulomb subtraction");
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox2)),frame);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
vbox = gtk_vbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame),vbox);
/* do the first radio button */
button = gtk_radio_button_new_with_label (NULL, "1-2");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) MOLE);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.coulomb == MOLE)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* make a radio group */
keyword[2] = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
/* do the next button */
button = gtk_radio_button_new_with_label (keyword[2], "1-2 and 1-3");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) MOLMEC);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.coulomb == MOLMEC)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* subsequent button... */
button = gtk_radio_button_new_with_label(
         gtk_radio_button_group (GTK_RADIO_BUTTON (button)), "none");
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "key", (gpointer) MOLQ);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* NEW */
if (data->gulp.coulomb == MOLQ)
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(button), TRUE);

/* frame 4 */
frame = gtk_frame_new(NULL);
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox2)),frame);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
hbox = gtk_hbox_new(FALSE, 0);
gtk_container_add (GTK_CONTAINER(frame),hbox);
gtk_container_set_border_width(GTK_CONTAINER(hbox),3);

/* TODO - ability to edit (via another dialog) */
label = gtk_label_new(" Maximum cycles ");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, FALSE, 0);

adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->gulp.maxcyc, 10, 500, 10, 10, 0);
button = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(button), FALSE);
gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object(GTK_OBJECT (adj), "value_changed",
                          GTK_SIGNAL_FUNC (gulp_keyword),
                         (gpointer) adj);
gtk_object_set_data(GTK_OBJECT(adj), "key", (gpointer) MAXCYC);
gtk_object_set_data(GTK_OBJECT(adj), "ptr", (gpointer) data);

/* setup the filename entry part */
frame = gtk_frame_new("Supplied data");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gulp_dialog->win)->vbox),frame,FALSE,FALSE,0);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
hbox = gtk_hbox_new(FALSE, 0);
gtk_container_add (GTK_CONTAINER(frame),hbox);
gtk_container_set_border_width(GTK_CONTAINER(hbox),3);

/* left vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

/* GULP template data */
/* TODO - ability to edit (via another dialog) */
label = gtk_label_new("Potentials");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);
gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
label = gtk_label_new("Species data");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);
gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);
label = gtk_label_new("Element data");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, FALSE, 0);
gtk_misc_set_alignment(GTK_MISC(label), 0.0f, 0.0f);

/* right vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, FALSE, FALSE, 0);

g_string_sprintf(line,"%d",data->gulp.num_potentials);
label = gtk_label_new(line->str);
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
g_string_sprintf(line,"%d",data->gulp.num_species);
label = gtk_label_new(line->str);
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
g_string_sprintf(line,"%d",data->gulp.num_elem);
label = gtk_label_new(line->str);
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);


/* setup the filename entry */
frame = gtk_frame_new("Files");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gulp_dialog->win)->vbox),frame,FALSE,FALSE,0);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
hbox = gtk_hbox_new(FALSE, 0);
gtk_container_add (GTK_CONTAINER(frame),hbox);
gtk_container_set_border_width(GTK_CONTAINER(hbox),3);

/* left vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

/* temporary GULP input filename */
label = gtk_label_new(" Temporary input ");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
label = gtk_label_new(" Potential library ");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);

/* right vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

gin_entry = gtk_entry_new_with_max_length(LINELEN);
gtk_entry_set_text(GTK_ENTRY(gin_entry),"temp.gin");
gtk_box_pack_start (GTK_BOX (vbox), gin_entry, TRUE, TRUE, 0);
lib_entry = gtk_entry_new_with_max_length(LINELEN);
gtk_entry_set_text(GTK_ENTRY(lib_entry),"gdis.lib");
gtk_box_pack_start (GTK_BOX (vbox), lib_entry, TRUE, TRUE, 0);

/* setup the filename entry */
frame = gtk_frame_new("Details");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(gulp_dialog->win)->vbox),frame,FALSE,FALSE,0);
gtk_container_set_border_width(GTK_CONTAINER(frame),3);
hbox = gtk_hbox_new(FALSE, 0);
gtk_container_add (GTK_CONTAINER(frame),hbox);

/* left vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

/* temporary GULP input filename */
label = gtk_label_new("Structure name");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);
label = gtk_label_new("Energy (eV)");
gtk_box_pack_start (GTK_BOX (vbox), label, TRUE, TRUE, 0);

/* right vbox */
vbox = gtk_vbox_new(TRUE, 2);
gtk_box_pack_start (GTK_BOX (hbox), vbox, TRUE, TRUE, 0);

name_entry = gtk_entry_new_with_max_length(LINELEN);
gtk_entry_set_text(GTK_ENTRY(name_entry),data->basename);
gtk_box_pack_start (GTK_BOX (vbox), name_entry, TRUE, TRUE, 0);

energy_entry = gtk_entry_new_with_max_length(LINELEN);
g_string_sprintf(line,"%f",data->gulp.energy);
gtk_entry_set_text(GTK_ENTRY(energy_entry),line->str);
gtk_box_pack_start (GTK_BOX (vbox), energy_entry, TRUE, TRUE, 0);
gtk_entry_set_editable(GTK_ENTRY(energy_entry), FALSE);


/* TODO - choose foreground or background run ?*/
/* terminating buttons */
button = gtk_button_new_with_label ("Run");
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gulp_dialog->win)->action_area),
                                         button, FALSE, TRUE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (run_gulp), (gpointer) id);

button = gtk_button_new_with_label ("Close");
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (gulp_dialog->win)->action_area),
                                         button, FALSE, TRUE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (close_dialog),
                          (gpointer) id);
/* done */
gtk_widget_show_all(gulp_dialog->win);

g_string_free(line, TRUE);
}

