/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "gdis.h"

/* the main pak structures */
extern struct sysenv_pak sysenv;

/*********************************************/
/* assign indices to qhull calculated planes */
/*********************************************/
/* NEW - with cdd this is just a true (transformed) normal calc */
#define DEBUG_LABEL 0
void facet_label(struct model_pak *data)
{
gint i,p,q;
gfloat v1[3], v2[3], fn[3];

/* loop over all calculated facets */
for (i=0 ; i<data->num_facets ; i++)
  {
/* true (transformed normal) */
  p = *((data->facets+i)->points);
  q = *((data->facets+i)->points+1);
  v1[0] = (data->vertices+q)->rx - (data->vertices+p)->rx;
  v1[1] = (data->vertices+q)->ry - (data->vertices+p)->ry;
  v1[2] = (data->vertices+q)->rz - (data->vertices+p)->rz;
  q = *((data->facets+i)->points+2);
  v2[0] = (data->vertices+q)->rx - (data->vertices+p)->rx;
  v2[1] = (data->vertices+q)->ry - (data->vertices+p)->ry;
  v2[2] = (data->vertices+q)->rz - (data->vertices+p)->rz;
  crossprod(fn,v1,v2);
  (data->facets+i)->norm[0] = fn[0];
  (data->facets+i)->norm[1] = fn[1];
  (data->facets+i)->norm[2] = fn[2];
/* untransformed normal calc */
/* cross product of 3 points on the facet */
/* use rx,ry,rz - is latmat (lattice->cartesian) transformed coords */
  p = *((data->facets+i)->points);
  q = *((data->facets+i)->points+1);
  v1[0] = (data->vertices+q)->x - (data->vertices+p)->x;
  v1[1] = (data->vertices+q)->y - (data->vertices+p)->y;
  v1[2] = (data->vertices+q)->z - (data->vertices+p)->z;
  q = *((data->facets+i)->points+2);
  v2[0] = (data->vertices+q)->x - (data->vertices+p)->x;
  v2[1] = (data->vertices+q)->y - (data->vertices+p)->y;
  v2[2] = (data->vertices+q)->z - (data->vertices+p)->z;
  crossprod(fn,v1,v2);
/* vector from centroid to one facet point */
  v1[0] = (data->vertices+p)->x - data->centroid[0];
  v1[1] = (data->vertices+p)->y - data->centroid[1];
  v1[2] = (data->vertices+p)->z - data->centroid[2];
/* if not in the same half space, invert normal */
  if (via(v1,fn,3) > PI/2.0)
    {
    fn[0] *= -1.0;
    fn[1] *= -1.0;
    fn[2] *= -1.0;
    (data->facets+i)->norm[0] *= -1.0;
    (data->facets+i)->norm[1] *= -1.0;
    (data->facets+i)->norm[2] *= -1.0;
    }

#if DEBUG_LABEL
printf("[*] facet %d, normal: %f,%f,%f\n",i,fn[0],fn[1],fn[2]);
#endif

  }
}

/***********************************/
/* determine if a plane is visible */
/***********************************/
#define DEBUG_FACET 0
gint facet_visible(struct model_pak *data, gint f)
{
gfloat a, norm[3], vec[3];

/* get normal and rotate */
norm[0] = (data->facets+f)->norm[0];
norm[1] = (data->facets+f)->norm[1];
norm[2] = (data->facets+f)->norm[2];
vecmat(data->rotmat, norm);

#if DEBUG_FACET
printf("facet (%2d%2d%2d), norm: %5.2f,%5.2f,%5.2f    " 
,(data->facets+f)->index[0]
,(data->facets+f)->index[1]
,(data->facets+f)->index[2]
,norm[0],norm[1],norm[2]);
#endif
/* absolute viewing vector, determining if visible or not */
vec[0] = 0.0;
vec[1] = 0.0;
vec[2] = -1.0;

/* got correct facet normal - is it visible? */
a = via(norm,vec,3);
if (a < PI/2.0)
  {
#if DEBUG_FACET
printf("view: %5.2f,%5.2f,%5.2f (%5.1f) YES\n",vec[0],vec[1],vec[2],180.0*a/PI);
#endif
  return(1);
  }
/* else... */
#if DEBUG_FACET
printf("view: %5.2f,%5.2f,%5.2f (%5.1f) NO\n",vec[0],vec[1],vec[2],180.0*a/PI);
#endif
return(0);
}

/***********************************/
/* determine if a ridge is visible */
/***********************************/
gint ridge_visible(struct model_pak *data, gint i, gint j)
{
gint f, p, id, flag;

/* loop through all facets */
for (f=0 ; f<data->num_facets ; f++)
  {
  if (!(data->facets+f)->visible)
    continue;
  flag=0;
/* if facet is visible, check if bounded by i or j  */
  for (p=0 ; p<(data->facets+f)->num_points ; p++)
    {
    id = *((data->facets+f)->points+p);
    if (id == i || id == j)
      flag++;
    }
/* found a visible face containing BOTH pts -> ridge is visible */
  if (flag == 2)
    return(1);
  }
return(0);
}

/******************************************/
/* load miller & normal lengths from file */
/******************************************/
#define DEBUG_LOAD_PLANES 0
gint load_planes(struct model_pak *data)
{
gint i, flag;
gfloat norm;
gchar line[LINELEN], **key;
FILE *fp=NULL;

fp = fopen(data->filename, "rt");
if (fp==NULL)
  return(1);

flag=0;
i=0;
while(!fgetline(fp,line))
  {
  key = get_tokens(line, 8);
  if (g_strncasecmp(*key,"cell",4) == 0)
    {
/* get PBC */
    sscanf(*(key+1),"%f",&(data->pbc[0]));
    sscanf(*(key+2),"%f",&(data->pbc[1]));
    sscanf(*(key+3),"%f",&(data->pbc[2]));
    sscanf(*(key+4),"%f",&(data->pbc[3]));
    sscanf(*(key+5),"%f",&(data->pbc[4]));
    sscanf(*(key+6),"%f",&(data->pbc[5]));
/* degrees -> radians */
    data->pbc[3] *= PI/180.0;
    data->pbc[4] *= PI/180.0;
    data->pbc[5] *= PI/180.0;
/* next line is number of planes */
    if (fgetline(fp,line))
      return(1);

    g_strfreev(key);
    key = get_tokens(line, 1);
    sscanf(*key,"%d",&data->num_planes);

/* ALLOC */
#if DEBUG_LOAD_PLANES
printf("Alloc for %d planes.\n",data->num_planes);
#endif
    data->planes = (struct plane_pak *) 
                    g_malloc(data->num_planes*sizeof(struct plane_pak));

    flag=1;
    if (fgetline(fp,line))
      return(1);
    } 

/* data read */
  if (flag)
    {
    sscanf(line,"%f %f %f %f",&(data->planes+i)->m[0]
                             ,&(data->planes+i)->m[1]
                             ,&(data->planes+i)->m[2]
                             ,&(data->planes+i)->len);
/* normalize the miller indices for halfspace intersection calcs */
   norm = sqrt((data->planes+i)->m[0]*(data->planes+i)->m[0]
              +(data->planes+i)->m[1]*(data->planes+i)->m[1]
              +(data->planes+i)->m[2]*(data->planes+i)->m[2]);
   if (norm)
     {
/* NB: cdd ineq. format needs the *negative* of the miller index */
     (data->planes+i)->x[0] = -1.0*(data->planes+i)->m[0]/norm;
     (data->planes+i)->x[1] = -1.0*(data->planes+i)->m[1]/norm;
     (data->planes+i)->x[2] = -1.0*(data->planes+i)->m[2]/norm;
     }
   else
     return(1);
/* FIXME - better treatment of sign? */
    if ((data->planes+i)->len < 0.0)
      (data->planes+i)->len *= -1.0;      
    i++;
    if (i >= data->num_planes)
      break;
    }
  g_strfreev(key);
  }
return(0);
}

/*********************************/
/* Morphology creation via qhull */
/*********************************/
#define DEBUG_MORPH_CREATE 0
gint morph_create(gchar *filename)
{
gint i, j;
gfloat sum, len, tmp;
struct model_pak *data=NULL;

/* make a new (MORPH) model */
data = model_ptr(sysenv.num_models, ASSIGN);
if (data == NULL)
  return(1);
sysenv.active = sysenv.num_models-1;
sysenv.mode[sysenv.active] = FREE;
/* setup model parameters */
template_model(data);
data->id = MORPH;
data->axes_type = OTHER;
data->hpr = TRUE;
data->morph_label = TRUE;
strcpy(data->filename, filename);
g_free(data->basename);
data->basename = strdup_no_extension(g_basename(data->filename));

/* read from a planes file */
if (load_planes(data))
  {
  printf("File load failed, check %s.\n",data->filename);
  delete_model(sysenv.active);
  return(1);
  }

/* normalize the vectors */
for (i=0 ; i<data->num_planes ; i++)
  {
  sum = 0.0;
  for (j=0 ; j<3 ; j++)
    sum += (data->planes+i)->x[j]*(data->planes+i)->x[j];
  len = sqrt(sum); 
  for (j=0 ; j<3 ; j++)
    (data->planes+i)->x[j] /= len;
  }

/* scale normal lengths with cell lengths */
for (i=0 ; i<data->num_planes ; i++)
  {
  sum = 0.0;
  for (j=0 ; j<3 ; j++)
    {
    tmp = fabs((data->planes+i)->x[j]*data->pbc[j]);
    sum += tmp*tmp;
    }
  (data->planes+i)->len *= sum;
  }

/* construct the hull */
data->num_vertices=0;
data->num_facets=0;
exec_cdd(data);

#if DEBUG_MORPH_CREATE
printf("-------------------------\n");
printf("       Input planes: %d\n",data->num_planes);
printf("Calculated vertices: %d\n",data->num_vertices);
printf("  Calculated facets: %d\n",data->num_facets);
printf("-------------------------\n");
#endif

if (!data->num_facets)
  {
/* cleanup!!! */
  delete_model(sysenv.active);
  show_text("Morphology calculation failed.\n");
  return(1);
  }

/* initialize */
init_objs(AXES_ON, data);
init_objs(INIT_COORDS, data);   /* setup non90 stuff */

/* calculate & label facet normals */
/* NB: must be done after INIT_COORDS, as needs model centroid */
facet_label(data);

/* proper rmax scaling & visibility calc */
init_objs(REDO_COORDS, data);
calc_bonds(data);
calc_mols(data);

pick_model(sysenv.active);
new_tree_item(sysenv.active, APPEND);

return(0);
}

