/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <ctype.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <strings.h>

#include "sginfo.h"
#include "gdis.h"

/* main structures */
extern struct sysenv_pak sysenv;

/* values for directions */
GtkWidget *image[MAX_DIALOGS][6];

/**********/
/* sginfo */
/**********/
static int str_ibegin(const char *s1, const char *s2) /* string ignore-case */
{                                                     /* begin              */
char u1, u2;

while (*s1 && *s2)
  {
  u1 = toupper(*s1++);
  u2 = toupper(*s2++);
  if (u1 < u2)
    return -1;
  else if (u1 > u2)
    return  1;
  }
if (*s2)
  return -1;
return 0;
}

/**********/
/* sginfo */
/**********/
/* TODO - we know what the input will be - trim */
gint BuildSgInfo(T_SgInfo *SgInfo, const gchar *SgName)
{
gint                VolLetter;
const T_TabSgName  *tsgn;


/* look for "VolA", "VolI", or "Hall" */

while (*SgName && isspace(*SgName)) SgName++;

VolLetter = -1;

if      (isdigit(*SgName))
  VolLetter = 'A';
else if (str_ibegin(SgName, "VolA") == 0)
  {
  VolLetter = 'A';
  SgName += 4;
  }
else if (str_ibegin(SgName, "VolI") == 0 || str_ibegin(SgName, "Vol1") == 0)
  {
  VolLetter = 'I';
  SgName += 4;
  }
else if (str_ibegin(SgName, "Hall") == 0)
  {
  VolLetter = 0;
  SgName += 4;
  }

while (*SgName && isspace(*SgName)) 
  SgName++;

/* default is "VolA" */

if (VolLetter == -1)
  VolLetter = 'A';

/* if we don't have a Hall symbol do a table look-up */

tsgn = NULL;

if (VolLetter)
  {
  tsgn = FindTabSgNameEntry(SgName, VolLetter);
  if (tsgn == NULL) return -1; /* no matching table entry */
  SgName = tsgn->HallSymbol;
  }

/* Allocate memory for the list of Seitz matrices and
   a supporting list which holds the characteristics of
   the rotation parts of the Seitz matrices
*/

SgInfo->MaxList = 192; /* absolute maximum number of symops */

SgInfo->ListSeitzMx
    = g_malloc(SgInfo->MaxList * sizeof (*SgInfo->ListSeitzMx));

if (SgInfo->ListSeitzMx == NULL) 
  {
  SetSgError("Not enough core");
  return -1;
  }

SgInfo->ListRotMxInfo
    = g_malloc(SgInfo->MaxList * sizeof (*SgInfo->ListRotMxInfo));

if (SgInfo->ListRotMxInfo == NULL) 
  {
  SetSgError("Not enough core");
  return -1;
  }

/* Initialize the SgInfo structure */

InitSgInfo(SgInfo);
SgInfo->TabSgName = tsgn; /* in case we know the table entry */

/* Translate the Hall symbol and generate the whole group */

ParseHallSymbol(SgName, SgInfo);
if (SgError != NULL)
  return -1;

  /* Do some book-keeping and derive crystal system, point group,
     and - if not already set - find the entry in the internal
     table of space group symbols
   */

return CompleteSgInfo(SgInfo);
}

/*****************************************/
/* generate positions of a complete cell */
/*****************************************/
#define DEBUG_GEN_POS 0
gint genpos(struct model_pak *data)
{
gint i, j, k, f, nt;
gint flag, match, m, n, ns, ix, primary, s, smin, new_flag;
gint iList, nTrV, iTrV, nLoopInv, iLoopInv, iMatrix;
gchar *cmd, *label;
gfloat dx, dy, dz, x[3];
const T_RTMx  *lsmx;
const gint *r, *t, *TrV;
T_RTMx SMx;
T_SgInfo SgInfo;

/* request info via number or name */
/* cell choice (TODO - neaten this ugly code) */
if (data->sginfo.cellchoice)
  {
  i = data->sginfo.cellchoice;
  if (data->sginfo.spacenum > 0)
    cmd = g_strdup_printf("%3d:%d",data->sginfo.spacenum,i);
  else
    cmd = g_strdup_printf("%s:%d",g_strstrip(data->sginfo.spacename),i);
  }
else
  {
  if (data->sginfo.spacenum > 0)
    cmd = g_strdup_printf("%3d",data->sginfo.spacenum);
  else
    cmd = g_strdup_printf("%s",g_strstrip(data->sginfo.spacename));
  }

#if DEBUG_GEN_POS
printf("retrieving as [%s]\n",cmd);
#endif

/* main call */
if (BuildSgInfo(&SgInfo, cmd) != 0)
  return(1);
g_free(cmd);

/* process returned space group name */
label = strdup(SgInfo.TabSgName->SgLabels);
for (i=0 ; i<strlen(label) ; i++)
  if (*(label+i) == '_')
    *(label+i) = ' ';

/* copy label to avoid any funny GULP characters */
i=strlen(label);
while(i>0)
  {
  if (*(label+i) == '=')
    {
    i++;
    break;
    }
  i--;
  }

/* fill in space group name */
g_free(data->sginfo.spacename);
data->sginfo.spacename = g_strdup_printf("%s",label+i);

/* fill in number (if an invalid value was supplied) */
if (data->sginfo.spacenum < 1)
  data->sginfo.spacenum = SgInfo.TabSgName->SgNumber;

/* crystal lattice type */
data->sginfo.lattice = SgInfo.XtalSystem;
g_free(data->sginfo.latticename);
switch(SgInfo.XtalSystem)
  {
  case XS_CUBIC:
    data->sginfo.latticename = g_strdup("Cubic");
    break;
  case XS_TETRAGONAL:
    data->sginfo.latticename = g_strdup("Tetragonal");
    break;
  case XS_ORTHOROMBIC:
    data->sginfo.latticename = g_strdup("Orthorombic");
    break;
  case XS_HEXAGONAL:
    data->sginfo.latticename = g_strdup("Hexagonal");
    break;
  case XS_TRIGONAL:
    data->sginfo.latticename = g_strdup("Trigonal");
    break;
  case XS_MONOCLINIC:
    data->sginfo.latticename = g_strdup("Monoclinic");
    break;
  case XS_TRICLINIC:
    data->sginfo.latticename = g_strdup("Triclinic");
    break;
  default:
    data->sginfo.latticename = g_strdup("Unknown");
  }

/* point group */
data->sginfo.pointgroup = SgInfo.PointGroup;

#if DEBUG_GEN_POS
printf("   Space group name: %s\n",data->sginfo.spacename);
printf(" Space group number: %d\n",data->sginfo.spacenum);
printf("       Lattice type: %c\n",SgInfo.LatticeInfo->Code);
printf("              Order: %d\n",SgInfo.OrderL);
printf("   Inversion center: ");
if (SgInfo.Centric == -1)
  printf("yes\n");
else
  printf("no\n");
#endif

/* OrderP/OrderL - which one??? */
data->sginfo.order = SgInfo.OrderL;
/* allocate for order number of pointers (to matrices) */
data->sginfo.matrix = (gfloat **) g_malloc(SgInfo.OrderL*sizeof(gfloat *));
data->sginfo.offset = (gfloat **) g_malloc(SgInfo.OrderL*sizeof(gfloat *));

iMatrix = 0;

nLoopInv = Sg_nLoopInv(&SgInfo);

nTrV = SgInfo.LatticeInfo->nTrVector;
 TrV = SgInfo.LatticeInfo->TrVector;

/* matrix counter */
m=0;
for (iTrV = 0; iTrV < nTrV; iTrV++, TrV += 3)
  {
  for (iLoopInv = 0; iLoopInv < nLoopInv; iLoopInv++)
    {
    if (iLoopInv == 0)
      f =  1;
    else
      f = -1;

    lsmx = SgInfo.ListSeitzMx;

/* loop over all matrices (order of the group) */
    for (iList = 0; iList < SgInfo.nList; iList++, lsmx++)
      {
      for (i = 0; i < 9; i++)
        SMx.s.R[i] = f * lsmx->s.R[i];
      for (i = 0; i < 3; i++)
        SMx.s.T[i] = iModPositive(f * lsmx->s.T[i] + TrV[i], STBF);

      r = SMx.s.R;
      t = SMx.s.T;

      *(data->sginfo.matrix+m) = (gfloat *) g_malloc(9*sizeof(gfloat));
      *(data->sginfo.offset+m) = (gfloat *) g_malloc(3*sizeof(gfloat));

      for (i = 0; i < 3; i++, t++)
        {
        for (j = 0; j < 3; j++, r++)
          {
/* mth general position */
          *(*(data->sginfo.matrix+m)+3*i+j) = (gfloat) *r;
          }
        nt = iModPositive(*t, STBF);
        if (nt >  STBF / 2)
          nt -= STBF;
/* offset matrix (3x1) */
        *(*(data->sginfo.offset+m)+i) = (gfloat) nt / (gfloat) STBF;
        }
      m++;
      }
    }
  }

/* number of matrices matches the order? */
if (m != data->sginfo.order)
  printf("Serious error in genpos()\n");

#if DEBUG_GEN_POS
printf("Constructed %d matrices.\n",m);
for (i=0 ; i<m ; i++)
  {
  printf("matrix %d :",i);
  for (j=0 ; j<9 ; j++)
    printf(" %4.1f",*(*(data->sginfo.matrix+i)+j));
  printf("\n");

  printf("offset %d :",i);
  for (j=0 ; j<3 ; j++)
    printf(" %4.1f",*(*(data->sginfo.offset+i)+j));
  printf("\n");
  }
#endif

/* for models that aren't 3D periodic (ie marvin surfaces) */
/* don't try to generate a (non existent) full cell */
if (data->periodic < 3)
  return(0);
/* if a pbc is zero, will cause info loops in pbc constrain */
for (i=0 ; i<3 ; i++)
  {
  if (data->pbc[i] == 0.0)
    {
    printf("Illegal cell length.\n");
    return(1);
    }
  }

/* index of added atoms */
n = primary = data->num_atoms;
/* NB: shells w/o cores won't be transformed */
ns = data->num_shells;

/* constrain asym unit to pbc */
/* NEW - only if not P1, as P1 may have atoms that were intended */
/* to be outside the cell (ie for viewing purposes) */
if (data->sginfo.spacenum > 1)
  pbc_constrain_atoms(data);

/* FIXME - hack to prevent core dump - num->atoms may have */
/* changed after pbc_constrain as atoms w/ partial occ. */
/* will have been deleted & thus lowered by mem_shrink */
n = primary = data->num_atoms;
ns = data->num_shells;

/* setup for site count (1 since we omit the trivial x,y,z) */
for (i=0 ; i<primary ; i++)
  (data->atoms+i)->sitecount = 1;

/* include an inversion (ie mult by -1) operation? */
if (SgInfo.Centric == -1)
  smin = -2;
else
  smin = 0;

/* FIXME - code can hang (or just run for a v long time?) */
/* if a bad space group (eg doesn't match pbc) is entered */

/* s = +1 or -1 for inversion of matrix elements */
for (s=1 ; s>smin ; s-=2)
  {
  for (i=0 ; i<primary ; i++)
    {
/* j=0 is trivial (x,y,z) */
    for (j=1 ; j<data->sginfo.order ; j++)
      {
/* get current (ith) coords */
      x[0] = s*(data->atoms+i)->x;
      x[1] = s*(data->atoms+i)->y;
      x[2] = s*(data->atoms+i)->z;
/* generate new position */
      vecmat(*(data->sginfo.matrix+j),x);
      x[0] += *(*(data->sginfo.offset+j)+0)*data->pbc[0];
      x[1] += *(*(data->sginfo.offset+j)+1)*data->pbc[1];
      x[2] += *(*(data->sginfo.offset+j)+2)*data->pbc[2];
/* constrain to cell */
      for (k=0 ; k<3 ; k++)
        {
        while (x[k] < 0.0)
          x[k] += data->pbc[k];
        while (x[k] > data->pbc[k])
          x[k] -= data->pbc[k];
        }
/* TODO - compare with all others (elem redundancy) and ... */
/* NB: all atom coords should be within the pbc defined unit cell */
      flag=0;
      for (k=0 ; k<n ; k++)
        {
        match=0;
/* x pbc corrected tolerance check */
        dx = fabs(x[0] - (data->atoms+k)->x);
        while (dx > data->pbc[0]/2.0)
          dx = fabs(dx - data->pbc[0]); 
        if (dx < POSITION_TOLERANCE)
          match++; 
/* y pbc corrected tolerance check */
        dy = fabs(x[1] - (data->atoms+k)->y);
        while (dy > data->pbc[1]/2.0)
          dy = fabs(dy - data->pbc[1]); 
        if (dy < POSITION_TOLERANCE)
          match++; 
/* z pbc corrected tolerance check */
        dz = fabs(x[2] - (data->atoms+k)->z);
        while (dz > data->pbc[2]/2.0)
          dz = fabs(dz - data->pbc[2]); 
        if (dz < POSITION_TOLERANCE)
          match++; 
/* all tolerances satisfied? */
        if (match == 3)
          {
/* must be same type to be deleted */
          if ((data->atoms+i)->atom_code == (data->atoms+k)->atom_code)
            flag++;
          }
        }
/* not a new atom - do next */
      if (flag)
        continue;
/* new position generated */
      (data->atoms+i)->sitecount++;
      new_flag=1;
/* TODO - somehow mark if in general or special position (via site count?) */
      if (n >= data->atom_limit)
        mem_grow(data, ATOM, 10);
      (data->atoms+n)->x = x[0]; 
      (data->atoms+n)->y = x[1]; 
      (data->atoms+n)->z = x[2]; 
      (data->atoms+n)->fx = x[0]/data->pbc[0]; 
      (data->atoms+n)->fy = x[1]/data->pbc[1]; 
      (data->atoms+n)->fz = x[2]/data->pbc[2]; 
      (data->atoms+n)->status = NORMAL;
      (data->atoms+n)->primary = FALSE;      /* not part of assym unit */
      (data->atoms+n)->orig = TRUE;        /* part of whole cell, though */
      (data->atoms+n)->sitecount = 0;
      (data->atoms+n)->sof = (data->atoms+i)->sof;
      (data->atoms+n)->offx = 0;
      (data->atoms+n)->offy = 0;
      strcpy((data->atoms+n)->label,(data->atoms+i)->label);
      strcpy((data->atoms+n)->element,(data->atoms+i)->element);
      strcpy((data->atoms+n)->tail,(data->atoms+i)->tail);
/* gets code & colour */
      elem_seek(n, ATOM, data);
/* also generate shell */
      if ((data->atoms+i)->has_shell)
        {
        ix = (data->atoms+i)->idx_shell;
        if (ns >= data->shell_limit)
          mem_grow(data, SHELL, 10);
/* get orig shell's coords & apply generator */
        x[0] = (data->shells+ix)->x;
        x[1] = (data->shells+ix)->y;
        x[2] = (data->shells+ix)->z;
        vecmat(*(data->sginfo.matrix+j),x);
        x[0] += *(*(data->sginfo.offset+j)+0)*data->pbc[0];
        x[1] += *(*(data->sginfo.offset+j)+1)*data->pbc[1];
        x[2] += *(*(data->sginfo.offset+j)+2)*data->pbc[2];
/* constrain to cell */
        for (k=0 ; k<3 ; k++)
          {
          while (x[k] < 0.0)
            x[k] += data->pbc[k];
          while (x[k] > data->pbc[k])
            x[k] -= data->pbc[k];
          }
/* save new shell */
        (data->shells+ns)->x = x[0]; 
        (data->shells+ns)->y = x[1]; 
        (data->shells+ns)->z = x[2]; 
        (data->shells+ns)->status = NORMAL;
        (data->shells+ns)->primary = FALSE;      /* not part of assym unit */
        (data->shells+ns)->orig = TRUE;        /* part of whole cell, though */
        (data->shells+ns)->sitecount = 0;
        (data->shells+ns)->offx = 0;
        (data->shells+ns)->offy = 0;
        strcpy((data->shells+ns)->label,(data->shells+ix)->label);
        strcpy((data->shells+ns)->element,(data->shells+ix)->element);
        strcpy((data->shells+ns)->tail,(data->shells+ix)->tail);
/* core-shell conection */
        (data->shells+ns)->has_core = TRUE;
        (data->shells+ns)->idx_core = n;
        (data->atoms+n)->has_shell = TRUE;
        (data->atoms+n)->idx_shell = ns;
/* setup */
        elem_seek(ns, SHELL, data);
/* next shell */
        ns++;
        }
/* next core */
      n++;
      }  /* loop over order of group */
    }    /* loop over asym unit */
  }    /* possible inversion center */

/* all done */
data->num_orig = data->num_atoms = n;
data->num_shells = ns;

/* unfortunately sitecount is not the site symmetry, */
/* since we would need to redo the loop that goes over */
/* the asym unit to go over the whole cell */ 
#if DEBUG_GEN_POS
printf("Atoms in assymetric unit: %d\n",primary);
printf("Atoms in whole unit: %d\n",n);
for (i=0 ; i<primary ; i++)
  printf("%3s multiplicity: %d\n",(data->atoms+i)->label
                                 ,(data->atoms+i)->sitecount);
#endif

return(0);
}

/************************/
/* force symmetry to P1 */
/************************/
void make_p1(struct model_pak *data)
{
gint i, j, k;

/* set name & number */
data->sginfo.spacenum = 1;
g_free(data->sginfo.spacename);
data->sginfo.spacename = g_strdup("P1");
/* all (non-deleted) atoms are now primary & non fractional */
mem_shrink(data);
for (i=data->num_atoms ; i-- ; )
  {
  (data->atoms+i)->orig = TRUE;
  (data->atoms+i)->primary = TRUE;
  }
/* if periodic images created - adjust cell params */
i = data->image_limit[1] + data->image_limit[0];
j = data->image_limit[3] + data->image_limit[2];
k = data->image_limit[5] + data->image_limit[4];
data->pbc[0] *= (gfloat) i;
data->pbc[1] *= (gfloat) j;
data->pbc[2] *= (gfloat) k;
/* rescale fractional coords to the new pbc */
if (data->fractional)
  {
  for (i=data->num_atoms ; i-- ; )
    {
    (data->atoms+i)->fx = (data->atoms+i)->x / data->pbc[0];
    (data->atoms+i)->fy = (data->atoms+i)->y / data->pbc[1];
    (data->atoms+i)->fz = (data->atoms+i)->z / data->pbc[2];
    }
  }
/* old (easy) way - pretend model is no longer fractional */
/*
data->fractional = FALSE;
*/
/* update cell param related data */
make_xlat(data);
make_cell(data);
/* generate symmetry data */
genpos(data);
/* NB: mem_shrink() -> redo connectivity */
init_objs(REDO_COORDS, data);
calc_bonds(data);
calc_mols(data);
}

/**************************************/
/* periodicity dialog hook to make_p1 */
/**************************************/
void remove_symmetry(gint id)
{
gint model;
struct model_pak *data;

/* get model ptr assoc. with this dialog */
model = sysenv.dialog[id].model;
data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* make P1 and update */
make_p1(data);
update_images(model, id, INITIAL);
wire_draw(SINGLE);
}

/***********************************/
/* periodic image creation routine */
/***********************************/
#define DEBUG_UPDATE_IMAGES 0
void update_images(gint model, gint dialog, gint mode)
{
gint a, b, c, i, j, n, num_cells, num_images, extra;
static gint current=-7;
struct model_pak *data;
struct atom_pak *dest;

/* checks */
data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* NEW - prevent multiple updates due to subsequent spinner updates */
if (current == dialog)
  {
#if DEBUG_UPDATE_IMAGES
printf("already active.\n");
#endif
  return;
  }
current = dialog;

#if DEBUG_UPDATE_IMAGES
for (i=0 ; i<sysenv.num_models ; i++)
  {
  printf("---------------------------\n");
  printf("model : %d\n",i);
  printf("a : %d - %d\n",data->image_limit[0],data->image_limit[1]);
  printf("b : %d - %d\n",data->image_limit[2],data->image_limit[3]);
  printf("c : %d - %d\n",data->image_limit[4],data->image_limit[5]);
  }
printf("---------------------------\n");
#endif

num_images = 0;
/* check mode */
switch(mode)
  {
  case RESTORE:
/* model switching - update the spin buttons */
    for (i=0 ; i<2*data->periodic ; i++)
      {
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(image[dialog][i]), 
                                data->image_limit[i]);
      gtk_signal_emit_by_name (GTK_OBJECT(image[dialog][i]), "changed");
      }
    return;
  case REFRESH:
/* get images from the spin button values */
    for (i=0 ; i<2*data->periodic ; i++)
      data->image_limit[i] = 
        gtk_spin_button_get_value_as_int(GTK_SPIN_BUTTON(image[dialog][i]));
  case CREATE:
/* from here - create images according to image_limit[] */
/* total number of displayed cells */
    num_cells = (data->image_limit[0]+data->image_limit[1])
              * (data->image_limit[2]+data->image_limit[3])
              * (data->image_limit[4]+data->image_limit[5]);
/* number of images is minus the default cell */
    num_images = num_cells-1;

/* CURRENT - more efficient updating */
    for (i=data->num_atoms ; i-- ; )
      if (!(data->atoms+i)->orig)
        (data->atoms+i)->status = DELETED;
    mem_shrink(data);

/* allocate for atoms */
    extra = num_cells*data->num_atoms - data->atom_limit;
    if (extra > 0)
      mem_grow(data, ATOM, extra);

#if DEBUG_UPDATE_IMAGES
/* FIXME - unfrag/pbc manip may screw num_orig up
printf("orig atoms = %d\n",data->num_orig);
*/
printf("orig atoms = %d\n",data->num_atoms);
printf("req images = %d\n",num_images);
printf("old alloc (atoms) = %d\n",data->atom_limit);
printf("new alloc (atoms) = %d\n",data->atom_limit+extra);
#endif

/* CURRENT - more efficient updating */
    dest = data->atoms + data->num_atoms;
    a = -data->image_limit[0];
    b = -data->image_limit[2];
    c = -data->image_limit[4];
    j = data->num_atoms;
    for (i=num_cells ; i-- ; )
      {
/* image increment */
      if (a == data->image_limit[1])
        {
        a = -data->image_limit[0];
        b++;
        if (b == data->image_limit[3])
          {
          b = -data->image_limit[2];
          c++;
          if (c == data->image_limit[5])
            break;
          }
        }
/* don't duplicate the original cell */
      if (a || b || c)
        {
#if DEBUG_UPDATE_IMAGES
printf("Creating %d,%d,%d\n",a,b,c);
#endif
/* create image */
        memcpy(dest, data->atoms, data->num_atoms*sizeof(struct atom_pak));
        j += data->num_atoms;
/* init atoms for this image */
        for (n=data->num_atoms ; n-- ; )
          {
          (dest+n)->x += a*data->pbc[0];
          (dest+n)->y += b*data->pbc[1];
          if (data->periodic == 3)
            (dest+n)->z += c*data->pbc[2];
          (dest+n)->status = NORMAL;
          (dest+n)->primary = FALSE;
          (dest+n)->orig = FALSE;
          }
/* next image */
        dest += data->num_atoms;
        }
      a++;
      }

#if DEBUG_UPDATE_IMAGES
printf("Wrote %d image atoms.\n",j-data->num_atoms);
#endif

/* mis-alloc'd? */
    if (j > data->atom_limit)
      printf("update_images(): serious memory allocation error.\n");
    else
      data->num_atoms = j;

    break;
  case INITIAL:
    for (i=data->num_atoms ; i-- ; )
      if (!(data->atoms+i)->orig)
        (data->atoms+i)->status = DELETED;
      else
        (data->atoms+i)->status = NORMAL;
    mem_shrink(data);

/* reset values to default */
    for (i=0 ; i<data->periodic ; i++)
      {
      data->image_limit[2*i] = 0;
      data->image_limit[2*i+1] = 1;
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(image[dialog][2*i]), 0);
      gtk_spin_button_set_value(GTK_SPIN_BUTTON(image[dialog][2*i+1]), 1);
      }
/* update the dialogue box */
    for (i=0 ; i<2*data->periodic ; i++)
      gtk_signal_emit_by_name (GTK_OBJECT(image[dialog][i]), "changed");
    break;
  }
current = -7;
}

/*********************/
/* spin button hooks */
/*********************/
/* TODO - combine/eliminate redundancy */
void gtk_refresh_images(GtkWidget *w, gint id)
{
gint model;
struct model_pak *data;

model = sysenv.dialog[id].model;
data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* atom colour/display updates */
update_images(model, id, REFRESH);
init_objs(REFRESH_COLOUR, data);
if (sysenv.mode[sysenv.active] == REGION_LITE)
  init_objs(REFRESH_LITE, data);
/* main coord updates */
init_objs(CENT_COORDS, data);
calc_bonds(data);
calc_mols(data);
pick_model(model);
}

void gtk_reset_images(GtkWidget *w, gint id)
{
gint model;
struct model_pak *data;

model = sysenv.dialog[id].model;
data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* atom colour/display updates */
update_images(model, id, INITIAL);
init_objs(REFRESH_COLOUR, data);
if (sysenv.mode[sysenv.active] == REGION_LITE)
  init_objs(REFRESH_LITE, data);
/* main coord updates */
init_objs(CENT_COORDS, data);
calc_bonds(data);
calc_mols(data);
pick_model(model);
}

/****************************/
/* display property updates */
/****************************/
void space_action(gint model, gint command)
{
gint i;
struct model_pak *data;

/* the usual */
data = model_ptr(model, RECALL);
if (!data)
  return;

switch(command)
  {
  case PBC_CONFINE_ATOMS:
    if (data->periodic)
      pbc_constrain_atoms(data);
    break;
  case PBC_CONFINE_MOLS:
    if (data->periodic)
      pbc_constrain_mols(data);
    break;
  case ASYM_TOGGLE:
    if (data->periodic)
      {
      data->asym_on ^= 1;
      if (data->asym_on)
        {
        for (i=0 ; i<data->num_atoms ; i++)
          if (!(data->atoms+i)->primary && 
              (data->atoms+i)->status != DELETED)
            (data->atoms+i)->status = HIDDEN;
        for (i=0 ; i<data->num_shells ; i++)
          if (!(data->shells+i)->primary &&
              (data->shells+i)->status != DELETED)
            (data->shells+i)->status = HIDDEN;
        }
      else
        {
        for (i=0 ; i<data->num_atoms ; i++)
          if (!(data->atoms+i)->primary && 
              (data->atoms+i)->status != DELETED)
            (data->atoms+i)->status = NORMAL;
        for (i=0 ; i<data->num_shells ; i++)
          if (!(data->shells+i)->primary &&
              (data->shells+i)->status != DELETED)
            (data->shells+i)->status = NORMAL;
        }
/*
      init_objs(REDO_COORDS, model);
      init_objs(REDO_BONDS, model);
      init_objs(REDO_MOLS, model);
      pick_model(model);
*/
      }
    break;
  default:
    printf("space_update(): unknown action requested.\n");
  }

/* common update */
init_objs(CENT_COORDS, data);
calc_bonds(data);
calc_mols(data);
wire_draw(SINGLE);
}

/****************************************************/
/* complete only one unit cell's worth of molecules */
/****************************************************/
#define DEBUG_UNFRAGMENT 0
void unfragment(struct model_pak *data)
{
gint i, j, k, flag, model, old;
gfloat x;

g_return_if_fail(data != NULL);
model = data->number;

old = data->num_atoms;

/* HIDDEN atoms can be duplicated, since they are not part of a mol */
/* unhide and recalc mols */
for (i=data->num_atoms ; i-- ; )
  if ((data->atoms+i)->orig)
    (data->atoms+i)->status = NORMAL;
data->asym_on = FALSE;
 
init_objs(REDO_COORDS, data);
calc_bonds(data);
calc_mols(data);

/* create extra unit cells to complete fringe (broken) mols */
/* FIXME? in theory we could have a mol that spans more than these limits */
data->image_limit[0] = 1;
data->image_limit[1] = 2;
data->image_limit[2] = 1;
data->image_limit[3] = 2;
if (data->periodic > 2)
  {
  data->image_limit[4] = 1;
  data->image_limit[5] = 2;
  }
else
  {
  data->image_limit[4] = 0;
  data->image_limit[5] = 1;
  }


/* dummy dialog id */
update_images(model, 0, CREATE);
/* reset */
data->image_limit[0] = 0;
data->image_limit[1] = 1;
data->image_limit[2] = 0;
data->image_limit[3] = 1;
data->image_limit[4] = 0;
data->image_limit[5] = 1;

/* CURRENT - remove duplicate molecules */
/* FIXME - if centroid is exactly on the boundary, may get problems */
init_objs(REDO_COORDS, data);
calc_bonds(data);
calc_mols(data);

#if DEBUG_UNFRAGMENT
printf("limits: %f %f %f\n",data->pbc[0],data->pbc[1],data->pbc[2]);
#endif

for (i=0 ; i<data->num_mols ; i++)
  {
  flag=0;
#if DEBUG_UNFRAGMENT
printf("[%d] ",i);
#endif
/* don't compare z if only 2D periodic */
  for (j=0 ; j<data->periodic ; j++)
    {
#if DEBUG_UNFRAGMENT
printf("%f ",(data->mols+i)->centroid[j]);
#endif
/* hack for -ve pbc */
    x = (data->mols+i)->centroid[j];
    if (data->pbc[j] < 0.0)
      x *= -1.0;
/* FIXME - this helps avoid 0.0/pbc duplication due to */
/* float comparison uncertainty, but it's a bit of a hack */
    x += FRACTION_TOLERANCE;
    if (x > 0.0 && x < fabs(data->pbc[j]))
      flag++;
    }

/* delete if pbc not satisfied */
  if (flag != data->periodic)
    {
#if DEBUG_UNFRAGMENT
printf(" deleted.");
#endif
    for (j=0 ; j<(data->mols+i)->num_atoms ; j++)
      {
      k = *((data->mols+i)->atom+j);
/* delete, DON'T hide - so that andrew's routine (after mem_shrink) */
/* gets one and only one unit cells worth of atoms */
      (data->atoms+k)->status = DELETED;
      }
    }
/* pbc satisifed, this molecule is now part of the unit cell */
  else
    {
    for (j=0 ; j<(data->mols+i)->num_atoms ; j++)
      {
      k = *((data->mols+i)->atom+j);
      (data->atoms+k)->orig = TRUE;
/* P1 symmetry -> also a primary atom */
      if (data->sginfo.spacenum == 1)
        (data->atoms+k)->primary = TRUE;
      }
    }
#if DEBUG_UNFRAGMENT
printf("\n");
#endif
  }
mem_shrink(data);

/* check */
if (data->num_atoms - old)
  printf("unfragment() ERROR: mismatch in number of atoms (%d/%d).\n",
                                                  data->num_atoms, old);
}

/****************/
/* button hooks */
/****************/
void pbc_confine_atoms(gint id)
{
space_action(sysenv.dialog[id].model, PBC_CONFINE_ATOMS);
}
void pbc_confine_mols(gint id)
{
space_action(sysenv.dialog[id].model, PBC_CONFINE_MOLS);
}
void asym_toggle(gint id)
{
space_action(sysenv.dialog[id].model, ASYM_TOGGLE);
}
#define DEBUG_COMPLETE_MOLS 0
void complete_mols(gint id)
{
gint model;
struct model_pak *data;

/* init */
model = sysenv.dialog[id].model;
data = model_ptr(model, RECALL);
g_return_if_fail(data != NULL);

/* NEW - call to complete */
unfragment(data);

/* update & redraw */
/* atom colour/display updates */
init_objs(REFRESH_COLOUR, data);
if (sysenv.mode[sysenv.active] == REGION_LITE)
  init_objs(REFRESH_LITE, data);
/* main coord updates */
init_objs(CENT_COORDS, data);
calc_bonds(data);
calc_mols(data);

#if DEBUG_COMPLETE_MOLS
printf("num atoms = %d\n",data->num_atoms);
printf("num mols = %d\n",data->num_mols);
#endif

/* draw - model is not nec = sysenv.active -> update all */
/* NEW - model may not be active - make it so (& update) */
pick_model(model);
}

/***************************/
/* Space Group info dialog */
/***************************/
void space_info(GtkWidget *w, struct model_pak *data)
{
gint i, j, k, l, m, rows, cols, need_sign, id;
gfloat *mat, value;
GString *info, *item;
GtkWidget *vbox, *hbox, *vbox1, *vbox2, *table, *frame, *button, *label;
GtkAdjustment *adj;
struct dialog_pak *sgid;

item = g_string_new(NULL);
info = g_string_new(NULL);

/* model checks */
g_return_if_fail(data != NULL);
g_return_if_fail(data->id != NODATA);
g_return_if_fail(data->periodic > 1);

/* NEW - active this model (avoid confusion?) */
pick_model(data->number);

/* retrieve a suitable dialog */
if ((id = request_dialog(data->number, SGINFO)) < 0)
  return;
sgid = &sysenv.dialog[id];

/* setup the dialog */
sgid->win = gtk_dialog_new();
gtk_window_set_title (GTK_WINDOW (sgid->win), "Space Group Info");
gtk_signal_connect(GTK_OBJECT(sgid->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);

/* dual pane layout */
hbox = gtk_hbox_new(FALSE,0);
gtk_container_add(GTK_CONTAINER(GTK_BOX(GTK_DIALOG(sgid->win)->vbox)),hbox);

/* left & right vboxes */
vbox1 = gtk_vbox_new (FALSE, 0);
vbox2 = gtk_vbox_new (FALSE, 0);
gtk_box_pack_start(GTK_BOX(hbox), vbox1, TRUE, TRUE, 0);
gtk_box_pack_start(GTK_BOX(hbox), vbox2, FALSE, FALSE, 0);

/* LEFT FRAME - MAIN INFO */
g_string_sprintf(info,"%s",data->basename);

frame = gtk_frame_new (info->str);
gtk_box_pack_end (GTK_BOX (vbox1), frame, TRUE, TRUE, 0);
gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
vbox = gtk_vbox_new (FALSE, 5);
gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
gtk_container_add (GTK_CONTAINER(frame),vbox);

/* table for nice spacing */
table = gtk_table_new(2, 12, FALSE);
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox)),table);

label = gtk_label_new ("Space Group name:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,0,1);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);
g_string_sprintf(info,"%s",data->sginfo.spacename);
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("Space Group number:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);

/* NEW - include cell choice */
if (data->sginfo.cellchoice)
  g_string_sprintf(info,"%d : %d",data->sginfo.spacenum
                                 ,data->sginfo.cellchoice);
else
  g_string_sprintf(info,"%d",data->sginfo.spacenum);
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,1,2);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("Crystal system:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);
g_string_sprintf(info,"%s",g_strstrip(data->sginfo.latticename));
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,2,3);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("Point group:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,3,4);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);
i = PG_Index(data->sginfo.pointgroup);
g_string_sprintf(info,"%s",PG_Names[i]);
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,3,4);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("Atoms in asymmetric cell:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,4,5);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);
g_string_sprintf(info,"%d",data->num_asym);
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,4,5);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("Atoms in full cell:       ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,5,6);
gtk_misc_set_alignment(GTK_MISC(label), 1.0f, 0.0f);
g_string_sprintf(info,"%d",data->num_orig);
label = gtk_label_new (g_strstrip(info->str));
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,5,6);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

/* NEW - cell parameters */
label = gtk_label_new ("a        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,6,7);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%5.2f",data->pbc[0]);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,6,7);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
label = gtk_label_new ("b        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,7,8);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%5.2f",data->pbc[1]);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,7,8);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
label = gtk_label_new ("c        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,8,9);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%5.2f",data->pbc[2]);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,8,9);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

label = gtk_label_new ("alpha        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,9,10);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%6.2f",180.0*data->pbc[3]/PI);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,9,10);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
label = gtk_label_new ("beta        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,10,11);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%6.2f",180.0*data->pbc[4]/PI);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,10,11);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
label = gtk_label_new ("gamma        ");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,11,12);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);
g_string_sprintf(info,"%6.2f",180.0*data->pbc[5]/PI);
label = gtk_label_new (info->str);
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,11,12);
gtk_misc_set_alignment(GTK_MISC(label), 0.5f, 0.0f);

/* BOTTOM FRAME */
frame = gtk_frame_new ("General positions");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(sgid->win)->vbox),frame,TRUE,TRUE,0);
gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
vbox = gtk_vbox_new (FALSE, 5);
gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
gtk_container_add (GTK_CONTAINER(frame),vbox);
/* table for nice spacing */
/* more intelligent dimension choice? */
cols = 4;
rows = data->sginfo.order / cols;
table = gtk_table_new(cols, rows, FALSE);
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox)),table);

/* order of the group (ie mth matrix) */
m=0;
/* loop over rows */
while (m<data->sginfo.order)
  {
/* get row(i) and column(j) */
  i = m / cols;
  j = m % cols;
/* construct the three generator elements */
  mat = *(data->sginfo.matrix+m);
  g_string_sprintf(info," ");
  for (k=0 ; k<3 ; k++)
    {
/* offset elements */
    need_sign = FALSE;
    value = *(*(data->sginfo.offset+m)+k);
    if (fabs(2.0*value-1.0) < FRACTION_TOLERANCE)
      {
      g_string_append(info,"1/2");
      need_sign = TRUE;
      }
    if (fabs(3.0*value-1.0) < FRACTION_TOLERANCE)
      {
      g_string_append(info,"1/3");
      need_sign = TRUE;
      }
    if (fabs(3.0*value+1.0) < FRACTION_TOLERANCE)
      {
      g_string_append(info,"-1/3");
      need_sign = TRUE;
      }
    if (fabs(4.0*value-1.0) < FRACTION_TOLERANCE)
      {
      g_string_append(info,"1/4");
      need_sign = TRUE;
      }
    if (fabs(4.0*value+1.0) < FRACTION_TOLERANCE)
      {
      g_string_append(info,"-1/4");
      need_sign = TRUE;
      }
/* matrix elements */
    for (l='x' ; l<='z' ; l++)
      {
      if (*mat > 0.0)
        {
        if (l == 'z')
          {
          if (need_sign)
            g_string_sprintf(item,"+%c",l);
          else
            g_string_sprintf(item,"%c",l);
          }
        else
          {
          if (need_sign)
            g_string_sprintf(item,"+%c,",l);
          else
            g_string_sprintf(item,"%c,",l);
          }
        g_string_append(info,item->str);
        }
      if (*mat < 0.0)
        {
        if (l == 'z')
          g_string_sprintf(item,"-%c",l);
        else
          g_string_sprintf(item,"-%c,",l);
        g_string_append(info,item->str);
        }
      mat++;
      }
    }
/* write the generator to the table */
  label = gtk_label_new(info->str);
  gtk_table_attach_defaults(GTK_TABLE(table),label,j,j+1,i,i+1);
  m++;
  }

/* RIGHT FRAME - PERIODIC IMAGE CONSTRUCTION */
frame = gtk_frame_new ("Periodic image construction");
gtk_box_pack_start (GTK_BOX (vbox2), frame, FALSE, TRUE, 0);

gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
vbox = gtk_vbox_new (FALSE, 5);
gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
gtk_container_add (GTK_CONTAINER (frame), vbox);
/* use table for nice spacing */
table = gtk_table_new(3, 4, FALSE);
gtk_container_add(GTK_CONTAINER(GTK_BOX(vbox)),table);

/* image direction labels */
label = gtk_label_new ("-");
gtk_table_attach_defaults(GTK_TABLE(table),label,1,2,0,1);
label = gtk_label_new ("+");
gtk_table_attach_defaults(GTK_TABLE(table),label,2,3,0,1);

/* x direction */
label = gtk_label_new ("a");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,1,2);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[0], 0, 5, 1, 1, 1);
image[id][0] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][0]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][0], 1, 2, 1, 2);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[1], 1, 5, 1, 1, 1);
image[id][1] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][1]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][1], 2, 3, 1, 2);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);

/* y direction */
label = gtk_label_new ("b");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,2,3);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[2], 0, 5, 1, 1, 1);
image[id][2] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][2]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][2], 1, 2, 2, 3);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[3], 1, 5, 1, 1, 1);
image[id][3] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][3]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][3], 2, 3, 2, 3);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);

/* z direction */
if (data->periodic == 3)
  {
label = gtk_label_new ("c");
gtk_table_attach_defaults(GTK_TABLE(table),label,0,1,3,4);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[4], 0, 5, 1, 1, 1);
image[id][4] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][4]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][4], 1, 2, 3, 4);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);

adj = (GtkAdjustment *) gtk_adjustment_new
                           (data->image_limit[5], 1, 5, 1, 1, 1);
image[id][5] = gtk_spin_button_new (adj, 0, 0);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(image[id][5]), GTK_SHADOW_OUT);
gtk_table_attach_defaults(GTK_TABLE(table), image[id][5], 2, 3, 3, 4);
gtk_signal_connect (GTK_OBJECT (adj), "value_changed",
                    GTK_SIGNAL_FUNC (gtk_refresh_images), (gpointer) id);
  }

/* reset button */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
button = gtk_button_new_with_label ("  Reset  ");
gtk_box_pack_start (GTK_BOX (hbox), button, TRUE, FALSE, 0);
gtk_signal_connect (GTK_OBJECT (button), "clicked",
                    GTK_SIGNAL_FUNC (gtk_reset_images), (gpointer) id);

/* RIGHT FRAME - VIEWING TOGGLES */
frame = gtk_frame_new (NULL);
gtk_box_pack_end (GTK_BOX (vbox2), frame, TRUE, TRUE, 0);
gtk_container_set_border_width (GTK_CONTAINER (frame), 5);
vbox = gtk_vbox_new (TRUE, 5);
gtk_container_set_border_width (GTK_CONTAINER (vbox), 5);
gtk_container_add (GTK_CONTAINER (frame), vbox);

/* view only asym */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
label = gtk_label_new ("Asymmetric/full cell");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
button = gtk_button_new_with_label (" X ");
gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (asym_toggle),
                           (gpointer) id);

/* FIXME (broken) constrain atoms to pbc */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
label = gtk_label_new (" Constrain atoms to PBC ");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
button = gtk_button_new_with_label (" X ");
gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (pbc_confine_atoms),
                           (gpointer) id);

/* complete 'broken' mols */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
label = gtk_label_new (" Constrain mols to PBC ");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
button = gtk_button_new_with_label (" X ");
gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (pbc_confine_mols),
                           (gpointer) id);

/* complete 'broken' mols */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
label = gtk_label_new (" Unfragment mols ");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
button = gtk_button_new_with_label (" X ");
gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (complete_mols),
                           (gpointer) id);

/* force to P1 symmetry */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start (GTK_BOX (vbox), hbox, FALSE, FALSE, 0);
label = gtk_label_new (" Convert to P1 ");
gtk_box_pack_start (GTK_BOX (hbox), label, TRUE, TRUE, 0);
button = gtk_button_new_with_label (" X ");
gtk_box_pack_end (GTK_BOX (hbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (remove_symmetry),
                           (gpointer) id);

/* Do the exit button */
button = gtk_button_new_with_label ("   Close   ");
gtk_box_pack_start (GTK_BOX (GTK_DIALOG (sgid->win)->action_area), button,
                                         TRUE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (close_dialog),
                           (gpointer) id);

/* done, draw, exit */
gtk_widget_show_all(sgid->win);
wire_draw(ALL);

g_string_free(info, TRUE);
g_string_free(item, TRUE);
}

