/*
Copyright (C) 2000 by Sean David Fleming

s.fleming@wbmt.tudelft.nl

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

The GNU GPL can also be found at http://www.gnu.org
*/

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <math.h>

#include "gdis.h"

/* re-entrant surface search step (Ang) */
#define LIFT_STEP 0.01
/* re-entrant surface search sweep (radians) */
#define SWEEP_STEP 0.05

/* main pak structures */
extern struct sysenv_pak sysenv;
extern struct elem_pak elements[ATOMDATA];

/* parameters are adjustable via a dialogue */
gint ix[3] = {0,1,2};   /* indep. variables x,y,z */

/********************************/
/* grid to cartesian conversion */
/********************************/
void grid2cart(gfloat *grid, gfloat *x)
{
/* decode grid scan plane format to x,y,z format */
if (ix[0] == 1)
  {
/* yz */
  *(x+0) = *(grid+2);
  *(x+1) = *(grid+0);
  *(x+2) = *(grid+1);  
  }
else
  {
/* xy */
  if (ix[1] == 1)
    {
    *(x+0) = *(grid+0);
    *(x+1) = *(grid+1);
    *(x+2) = *(grid+2);  
    }
/* xz */
  else
    {
    *(x+0) = *(grid+0);
    *(x+1) = *(grid+2);
    *(x+2) = *(grid+1);  
    }
  }
}

/****************************************************/
/* make a grid representation of a connolly surface */
/****************************************************/
/* CURRENT - latmat conversion (ie correct pbc handling) NEEDS CHECKING */
#define DEBUG_CALCCON 0
void calc_surf(struct model_pak *data)
{
gint i, j, k, m, n, ai, bi, num_exp, swap;
gint *atom_exp, grid, touch, flag, flag2;
gfloat da, db, dc, pa, pb, pc, prad, rad, len, height, d2, r2, td2, ta, tb;
gfloat a[3], b[3], g[3], p[3], x[3], pos[3], min[3], maxh, step[3];
gfloat qa, qb, qc;
gfloat angle, area, gamma, atot, psa, gsa, *area_exp, *area_pac;

/* checks */
g_return_if_fail(data != NULL);
if (!sysenv.num_models)
  return;
if (data->fractional)
  {
  show_text("Sorry, can't do this for fractional coordinates yet.");
  return;
  }

/* setup */
prad = data->csurf.prad;
grid = data->csurf.grid;

/* attempt to match atom type with database */
#if DEBUG_CALCCON
printf("Constructing grid (%dx%d) with probe of radius: %f\n",grid,grid,prad);
#endif

/* allocation */
g_free(data->csurf.touch);
g_free(data->csurf.h);
g_free(data->csurf.pa);
g_free(data->csurf.pb);
g_free(data->csurf.pc);
g_free(data->csurf.px);
g_free(data->csurf.py);
g_free(data->csurf.rx);
g_free(data->csurf.ry);
g_free(data->csurf.rz);
data->csurf.touch = (gint *) g_malloc(grid*grid*sizeof(gint));
data->csurf.h = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.pa = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.pb = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.pc = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.px = (gint *) g_malloc(grid*grid*sizeof(gint));
data->csurf.py = (gint *) g_malloc(grid*grid*sizeof(gint));
data->csurf.rx = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.ry = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));
data->csurf.rz = (gfloat *) g_malloc(grid*grid*sizeof(gfloat));

/* match min/max to displayed pbc cell */
/* start at origin (see make_cell()) */
min[0] = min[1] = min[2] = 0.0;
/* supplement with 'height' scan variable ONLY */
maxh = -999999.999;
min[ix[2]] = 999999.999;
for (n=0 ; n<data->num_atoms ; n++)
  {
  if ((data->atoms+n)->status == DELETED)
    continue;
  x[0] = (data->atoms+n)->x;
  x[1] = (data->atoms+n)->y;
  x[2] = (data->atoms+n)->z;
/* CURRENT  - latmat? */
  vecmat(data->latmat, x);

/* 'height' only */
  if (x[ix[2]] < min[ix[2]])
    min[ix[2]] = x[ix[2]];
  if (x[ix[2]] > maxh)
    maxh = x[ix[2]];
  }
/* ensure full 'height' scan by adding a 'vertical' safety zone */
/* (if max is too low - probe will not reach the top of high atoms) */
if (!data->fractional)
  maxh += (prad + 2.0);

/* compute step size required */
step[0] = (data->pbc[0]) / (grid-1);
step[1] = (data->pbc[1]) / (grid-1);
step[2] = (data->pbc[2]) / (grid-1);

/* ab is scan plane - c reps surface 'height' values */
#if DEBUG_CALCCON
printf("a limits: %9.2f (%9.2f)\n",min[ix[0]],step[ix[0]]);
printf("b limits: %9.2f (%9.2f)\n",min[ix[1]],step[ix[1]]);
printf("c limits: %9.2f to %9.2f\n",min[ix[2]],maxh);
#endif


/* scan the required plane */
k=0;
ta=tb=0;
for (i=0 ; i<grid ; i++)
  {
  for (j=0 ; j<grid ; j++)
    {
/* generate actual grid point coordinates */
    p[0] = (gfloat) i*step[ix[0]];
    p[1] = (gfloat) j*step[ix[1]];
    p[2] = 0.0;
    grid2cart(p,g);
    vecmat(data->latmat,g);
    g[0] += min[ix[0]];
    g[1] += min[ix[1]];
    g[2] += min[ix[2]];
/* find highest touched (periodic!) atom at this point */
    touch = REENTRANT;
    len = 1.0;
    for (n=0 ; n<data->num_atoms ; n++)
      {
      if ((data->atoms+n)->status == DELETED)
        continue;
/* get atom's data */
      rad = elements[(data->atoms+n)->atom_code].vdw;
/* do periodic images */
      for (ai=-1 ; ai<2 ; ai++)
        {
        for (bi=-1 ; bi<2 ; bi++)
          {
/* if not periodic - only do ai=bi=0 */
          if (!data->periodic)
            if (ai || bi)
              continue;
/* pbc */
          p[0] = (gfloat) ai*data->pbc[ix[0]];
          p[1] = (gfloat) bi*data->pbc[ix[1]];
          p[2] = 0.0;
          grid2cart(p,pos);
/* current atom's location */
          pos[0] += (data->atoms+n)->x;
          pos[1] += (data->atoms+n)->y;
          pos[2] += (data->atoms+n)->z;
/* CURRENT - latmat? */
/* apply lattice vectors */
          vecmat(data->latmat,pos); 
/* compute ab plane projected separation */
          da = g[0] - pos[ix[0]];
          db = g[1] - pos[ix[1]];
          r2 = rad*rad - da*da - db*db;
/* smaller than atom's radius? */
          if (r2 > 0.0)
            {
/* yes, calculate height on atom's surface (NB: assume +ve is UP) */
            height = pos[ix[2]] + sqrt(r2); 
            if (height > g[2])
              {
              g[2] = height;
              touch = n;
              len = rad;
/* save touched atom's translational info!!! */
              ta = (gfloat) ai;
              tb = (gfloat) bi;
              }
            }
          }
        }
      }
/* was an atom found? */
/* FIXME - hack to prevent gcc warning */
    pa=pb=pc=0.0;
    if (touch < 0)
      {
#if DEBUG_CALCCON
printf("Warning: hole found in structure. (%f,%f,%f)\n",g[0],g[1],g[2]);
#endif
      flag=1;
      }
    else
      {
/* pbc translation component of atom touched */
      p[0] = ta*data->pbc[ix[0]];
      p[1] = tb*data->pbc[ix[1]];
      p[2] = 0.0;
      grid2cart(p,pos);
/* calculate probe position require to touch point: pos[] */
      pos[0] += (data->atoms+touch)->x;
      pos[1] += (data->atoms+touch)->y;
      pos[2] += (data->atoms+touch)->z;
/* CURRENT - latmat? */
      vecmat(data->latmat,pos); 
/* vector from touched atom centre to touch pt */
      da = g[0] - pos[ix[0]];
      db = g[1] - pos[ix[1]];
      dc = g[2] - pos[ix[2]];
/* touch point */
      pa = g[0];
      pb = g[1];
      pc = g[2];
/* probe center coords */
      pa += da * prad / len;
      pb += db * prad / len;
      pc += dc * prad / len;
/* does probe intersect any other atoms? */
      flag=0;
      for (n=0 ; n<data->num_atoms ; n++)
        {
        if (n == touch)
          continue; 
        if ((data->atoms+n)->status == DELETED)
          continue;
/* check against n & surrounding images of n */
        for (ai=-1 ; ai<2 ; ai++)
          {
          for (bi=-1 ; bi<2 ; bi++)
            {
/* if not periodic - only do ai=bi=0 */
            if (!data->periodic)
              if (ai || bi)
                continue;
/* pbc */
            p[0] = (gfloat) ai*data->pbc[ix[0]];
            p[1] = (gfloat) bi*data->pbc[ix[1]];
            p[2] = 0.0;
            grid2cart(p,pos);
/* current atom's location */
            pos[0] += (data->atoms+n)->x;
            pos[1] += (data->atoms+n)->y;
            pos[2] += (data->atoms+n)->z;
/* CURRENT - latmat? */
/* apply lattice vectors */
            vecmat(data->latmat,pos); 
/* touching required distance (squared) */
            rad = elements[(data->atoms+n)->atom_code].vdw;
            td2 = (rad+prad)*(rad+prad);
/* actual separation */
            da = pa - pos[ix[0]];
            db = pb - pos[ix[1]];
            dc = pc - pos[ix[2]];
            d2 = da*da + db*db + dc*dc;
            if (d2 < td2)
              {
              flag=2;
              goto skip;
              }
            }    /* image scan */
          }      /* image scan */
        }        /* probe intersection scan */
      }          /* if probe touch */

/* was the probe's position ok? */
skip:;
    switch(flag)
      {
      case 0:
/* yes, record actual touching position */
        *(data->csurf.touch+k) = touch; 
        *(data->csurf.h+k) = g[2];
/* record probe's center - for post-scan */
        *(data->csurf.pa+k) = pa;
        *(data->csurf.pb+k) = pb;
        *(data->csurf.pc+k) = pc;
        break;
      case 1:
/* no - type hole */
        *(data->csurf.touch+k) = HOLE; 
        *(data->csurf.h+k) = min[ix[2]];
        break;
      case 2:
/* no - type 'normal' re-entrant surface */
        *(data->csurf.touch+k) = REENTRANT;
        break;
      }
/* next grid/surface point */
    k++;
    }  /* grid loop (j) */
  }    /* grid loop (i) */


/***********************/
/* post scan 1 - holes */
/***********************/
/* current grid point index */
k=0;
for (i=0 ; i<grid ; i++)
  {
  for (j=0 ; j<grid ; j++)
    {
/* only interested in holes */
    if (*(data->csurf.touch+k) == HOLE)
      {
      p[0] = (gfloat) i*step[ix[0]];
      p[1] = (gfloat) j*step[ix[1]];
      p[2] = 0.0;
      grid2cart(p,g);
/* CURRENT */
      vecmat(data->latmat,g);
      g[0] += min[ix[0]];
      g[1] += min[ix[1]];
      g[2] += min[ix[2]];
/* scan loop */
      flag=0;
      for (n=0 ; n<data->num_atoms ; n++)
        {
        if ((data->atoms+n)->status == DELETED)
          continue;
/* check against n & surrounding images of n */
        for (ai=-1 ; ai<2 ; ai++)
          {
          for (bi=-1 ; bi<2 ; bi++)
            {
/* if not periodic - only do ai=bi=0 */
            if (!data->periodic)
              if (ai || bi)
                continue;
/* pbc */
            p[0] = ai*data->pbc[ix[0]];
            p[1] = bi*data->pbc[ix[1]];
            p[2] = 0.0;
            grid2cart(p,pos);
/* CURRENT - latmat? */
/* apply lattice vectors */
            vecmat(data->latmat,pos); 
/* current atom's location */
            pos[0] += (data->atoms+n)->x;
            pos[1] += (data->atoms+n)->y;
            pos[2] += (data->atoms+n)->z;
/* touching required distance (squared) */
            rad = elements[(data->atoms+n)->atom_code].vdw;
            td2 = (rad+prad)*(rad+prad);
/* dist to grid point */
            da = pos[ix[0]] - g[0];
            db = pos[ix[1]] - g[1];
/* close enough? */
            r2 = td2 - da*da - db*db;
            if (r2 > 0.0)
              {
/* bottom of probe surface */
              height = pos[ix[2]] + sqrt(r2);
/* is it the lowest so far? */
              if (height > g[2])
                {
                g[2] = height; 
                flag++;
                }
              }
            }  /* pbc loop bi */
          }    /* pbc loop ai */
        }      /* search loop n*/
/* record - was a 'proper' height found? */
      if (flag)
        {
/* this is a special hole */
        *(data->csurf.touch+k) = VALLEY;
        *(data->csurf.pa+k) = g[0];
        *(data->csurf.pb+k) = g[1];
        *(data->csurf.pc+k) = g[2];
        *(data->csurf.h+k) = g[2] - prad;
        }
      }        /* if hole */
    k++;
    }          /* grid loop j */
  }            /* grid loop i */

/************************************/
/* post scan 2 - re-entrant surface */
/************************************/
/* current grid point index */
k=0;
num_exp=0;
atom_exp = (gint *) g_malloc(data->num_atoms * sizeof(gint));
qa=qb=qc=0.0;           
for (i=0 ; i<grid ; i++)
  {
  for (j=0 ; j<grid ; j++)
    {
/* if contact */
    if (*(data->csurf.touch+k) > 0)
      {
      flag=0;
      for(m=0 ; m<num_exp ; m++)
        if (*(atom_exp+m) == *(data->csurf.touch+k))
          flag++;
/* already recorded as exposed? */
      if (!flag)
        {
        *(atom_exp+num_exp) = *(data->csurf.touch+k);
        num_exp++;
        }
      }

/* skip if not re-entrant (ie -1) */
    if (*(data->csurf.touch+k) == REENTRANT)
      {
      p[0] = (gfloat) i*step[ix[0]];
      p[1] = (gfloat) j*step[ix[1]];
      p[2] = 0.0;
      grid2cart(p,g);
/* CURRENT */
      vecmat(data->latmat,g);
      g[0] += min[ix[0]];
      g[1] += min[ix[1]];
      g[2] = 10.0 + maxh;
/* scan for contact surfaces close enough to generate a height */
/* and record the lowest of these - n is scan index */
      flag=0;
flag2=0;
      for (n=0 ; n<grid*grid ; n++)
        {
/* only want surfaces with attached probe centers */
        if (*(data->csurf.touch+n) == REENTRANT ||
            *(data->csurf.touch+n) == HOLE)
          continue;
/* check against n & surrounding images of n */
        for (ai=-1 ; ai<2 ; ai++)
          {
          for (bi=-1 ; bi<2 ; bi++)
            {
/* if not periodic - only do ai=bi=0 */
            if (!data->periodic)
              if (ai || bi)
                continue;
/* required translation */
    p[0] = ai*data->pbc[ix[0]]; 
    p[1] = bi*data->pbc[ix[1]]; 
    p[2] = 0.0;
/* decode to cartesian */
    grid2cart(p,x);
/* CURRENT */
/* apply lattice vectors */
    vecmat(data->latmat,x); 
/* generate PBC xlat'd probe center coords */
    pa = x[ix[0]] + *(data->csurf.pa+n);
    pb = x[ix[1]] + *(data->csurf.pb+n);
    pc = x[ix[2]] + *(data->csurf.pc+n);
/* decode */
/*
    p[0] += *(data->csurf.pa+n);
    p[1] += *(data->csurf.pb+n);
    p[2] += *(data->csurf.pc+n);
    grid2cart(&p[0], &x[0]);
*/
/* dist to grid point */
            da = pa - g[0];
            db = pb - g[1];
/* close enough? */
            td2 = prad*prad - da*da - db*db;
            if (td2 > 0.0)
              {
flag2++;
/* bottom of probe surface */
              height = pc - sqrt(td2);
/* is it the lowest so far? */
              if (height < g[2])
                {
                g[2] = height; 
/* candidate probe centre for this re-entrant surface pt. */
                qa = pa;
                qb = pb;
                qc = pc;
                flag=1;
                }
              }
            }  /* pbc loop bi */
          }    /* pbc loop ai */
        }      /* search loop n*/

/* record - was a 'proper' height found? */
      if (flag)
        {
        *(data->csurf.h+k) = g[2];  /* yes */
        *(data->csurf.pa+k) = qa;
        *(data->csurf.pb+k) = qb;
        *(data->csurf.pc+k) = qc;
        }
      else
        {
#if DEBUG_CALCCON
printf("Warning, no valid probe positions (%d candidates).\n",flag2);
#endif
        *(data->csurf.touch+k) = HOLE;
        *(data->csurf.h+k) = min[ix[2]];
        }
      }      /* if re-entrant */
/* next grid point */
    k++;
    }        /* grid loop j */
  }          /* grid loop i */

#if DEBUG_CALCCON
/* touch grid putput */
/*
k=0;
for (i=0 ; i<grid ; i++)
  {
  for (j=0 ; j<grid ; j++)
    {
    touch = *(data->csurf.touch+k);
    if (touch < 0)
      {
      printf("*   ");
      }
    else
      {
      printf("%3d ",touch);
      }
    k++;
    }
  printf("\n");
  }
*/
#endif

/*************************************/
/* post scan 3 - surface integration */
/*************************************/
area_exp = (gfloat *) g_malloc(num_exp*sizeof(gfloat));
area_pac = (gfloat *) g_malloc(num_exp*sizeof(gfloat));
for (i=0 ; i<num_exp ; i++)
  {
  *(area_exp+i) = 0.0;
  *(area_pac+i) = 0.0;
  }
flag=0;
atot=0.0;

/* project scan plane (ab) surface area */
/* a vector */
a[0] = a[1] = a[2] = 0.0;
a[ix[0]] = data->pbc[ix[0]];
vecmat(data->latmat,a);
/* b vector */
b[0] = b[1] = b[2] = 0.0;
b[ix[1]] = data->pbc[ix[1]];
vecmat(data->latmat,b);
/* get area via cross prod */
crossprod(g,a,b);
gsa = psa = VEC3MAG(g);

/* grid point ab projected surace area */
gsa /= (grid-1)*(grid-1);

k=0;
for (i=0 ; i<grid ; i++)
  {
  for (j=0 ; j<grid ; j++)
    {
/* skip if true hole */
    if (*(data->csurf.touch+k) == HOLE)
      {
      k++;
      flag++;
      continue;
      }
/* get surface pt. (touch) */
    p[0] = (gfloat) i*step[ix[0]];
    p[1] = (gfloat) j*step[ix[1]];
    p[2] = 0.0;
    grid2cart(p,g);
    g[0] += min[ix[0]];
    g[1] += min[ix[1]];
    g[2] = *(data->csurf.h+k);
    vecmat(data->latmat,g);
/* get associated probe centre */
    p[0] = *(data->csurf.pa+k);
    p[1] = *(data->csurf.pb+k);
    p[2] = *(data->csurf.pc+k);
    grid2cart(p,x);
/* surface normal is vec from touch to probe centre */
    p[0] = x[0] - g[0];
    p[1] = x[1] - g[1];
    p[2] = x[2] - g[2];
/* often a good check */
#if DEBUG_CALCCON
    r2 = sqrt(p[0]*p[0] + p[1]*p[1] + p[2]*p[2]);
    if (fabs(r2-prad) > 0.01)
      printf("Error: bad probe center coordinates (k=%i,dist=%f).\n",k,r2);
#endif
/* get angle between this and the verticle normal (ie c) */
    g[0] = 0.0;
    g[1] = 0.0;
    g[2] = 1.0;
    grid2cart(g,x);
/* compute molec. surf. area at this grid point */
    angle = via(&p[0],&x[0],3);
    gamma = fabs(cos(angle));
/* FIXME - crude hack for (near) vertical surface element */
    if (gamma > 0.01)
      area = gsa / gamma;
    else
      area = gsa / 0.01;
/* if contact, assign area to assoc. atom */
    for (n=0 ; n<num_exp ; n++)
      if (*(data->csurf.touch+k) == *(atom_exp+n))
        *(area_exp+n) += area;
/* add to total area */
    atot += area;
/* next grid point */
    k++;
    }
  }

/* output status/summary */
if (data->csurf.rsf)
  {
  if (flag)
    printf("Area calculation warning: surface has %d holes.\n",flag);
  printf("Found %d exposed atoms.\n",num_exp);
/*
  printf(" Projected surface area: %f\n",psa);
*/
  printf("   Net VdW surface area: %f\n",atot);
/* POSE - % of self exposed */
/* compute % accessibility */
  area=0.0;
  for (n=0 ; n<num_exp ; n++)
    {
/* atom index */
    i = *(atom_exp+n);
/* element reference */
    j = (data->atoms+i)->atom_code;
    rad = elements[j].vdw;
    pa = 4.0*PI*rad*rad;
    if (pa > 0.001)
      ta = *(area_exp+n)/pa;
    else
      ta = 0.0;
/* percentage accessibility */
    *(area_pac+n) = 100.0*ta;
/* net contact surface */
    area += *(area_exp+n);
    }

/* further summary */
  printf("   Contact surface area: %f\n",area);
  printf("Re-entrant surface area: %f\n",atot-area);

/* sort the accessibilities */
  swap=1;
  while(swap)
    {
    swap=0;
    for (n=0 ; n<num_exp-1 ; n++)
      {
/* if order is bad - swap */
      if (*(area_pac+n) < *(area_pac+n+1))
        {
        ta = *(area_pac+n);
        tb = *(area_exp+n);
        i = *(atom_exp+n);
        *(area_pac+n) = *(area_pac+n+1);
        *(area_exp+n) = *(area_exp+n+1);
        *(atom_exp+n) = *(atom_exp+n+1);
        *(area_pac+n+1) = ta;
        *(area_exp+n+1) = tb;
        *(atom_exp+n+1) = i;
        swap++;
        }
      }
    }
/* print them out */
  for (n=0 ; n<num_exp ; n++)
    {
    i = *(atom_exp+n);
    printf("[%2s] %5.2f%% of atom accessible (raw area %f)\n",
          (data->atoms+i)->element, *(area_pac+n), *(area_exp+n));
    }
  }

/* setup for display */
data->csurf_on = TRUE;
data->csurf.grid = grid;
data->csurf.ix[0] = ix[0];
data->csurf.ix[1] = ix[1];
data->csurf.ix[2] = ix[2];
data->csurf.min[0] = min[0];
data->csurf.min[1] = min[1];
data->csurf.min[2] = min[2];
data->csurf.step[0] = step[0];
data->csurf.step[1] = step[1];
data->csurf.step[2] = step[2];
/* calculate pixel coords & redraw */
update_coords(REFRESH, data);
cent_coords(data);
update_coords(REFRESH, data);
wire_draw(SINGLE);

/* status */
if (!data->periodic)
  show_text("Warning: non periodic model, surface may contain edge artifacts.\n");
else
  show_text("2D vdW surface generated.");

/* cleanup & exit */
g_free(atom_exp);
g_free(area_exp);
g_free(area_pac);
return;
}

/***************************/
/* global spin update hook */
/***************************/
gint change_spin(GtkWidget *w, gpointer *spin)
{
gint id;
gfloat r;
struct model_pak *data=NULL;

/* checks & setup */
g_return_val_if_fail(spin != NULL, FALSE);
r = SPIN_FVAL(GTK_SPIN_BUTTON(spin));
data = (struct model_pak *) gtk_object_get_data(GTK_OBJECT(spin), "ptr");
g_return_val_if_fail(data != NULL, FALSE);
id = GPOINTER_TO_INT(gtk_object_get_data(GTK_OBJECT(spin), "id"));

/* mode dependent spinner update */
switch(id)
  {
/* construct surface */
  case MILLER_H:
    data->surface.miller[0] = (gint) r;
    break;
  case MILLER_K:
    data->surface.miller[1] = (gint) r;
    break;
  case MILLER_L:
    data->surface.miller[2] = (gint) r;
    break;
  case SHIFT:
    data->surface.shift = r;
    break;
  case REGION1:
    data->surface.region[0] = (gint) r;
    break;
  case REGION2:
    data->surface.region[1] = (gint) r;
    break;
/* contact surface */
  case GRID:
    data->csurf.grid = r;
    break;
  case OFFSET:
    data->csurf.offset = r;
    init_objs(CENT_COORDS, data);
    wire_draw(SINGLE);
    break;
/* update for both (FIXME - problems with this?) */
  case PROBE:
    data->csurf.prad = r;
    data->mysurf[0].prad = r;
    break;
  case SURF_ACCURACY:
    data->mysurf[0].shell_size = (gint) r;
    break;
  default:
    printf("unknown mode.\n");
  }

return(FALSE);
}

/********************/
/* simple hide hook */
/********************/
void hide_con(struct model_pak *data)
{
data->csurf_on = FALSE;
wire_draw(SINGLE);
}

/**************************/
/* results summary toggle */
/**************************/
void rsf_toggle(struct model_pak *data)
{
data->csurf.rsf ^= 1;
}

/*********************/
/* select scan plane */
/*********************/
void switch_plane(gint plane)
{
switch(plane)
  {
  case PLANE_XY:
    ix[0] = 0;
    ix[1] = 1;
    ix[2] = 2;
    break;
  case PLANE_YZ:
    ix[0] = 1;
    ix[1] = 2;
    ix[2] = 0;
    break;
  case PLANE_XZ:
    ix[0] = 0;
    ix[1] = 2;
    ix[2] = 1;
    break;
  }
}

/***************************/
/* Molecular surface setup */
/***************************/
void surf_dialog()
{
gint id;
GSList *surf_plane;
GtkWidget *frame, *vbox, *button, *spin;
GtkAdjustment *adj;
struct dialog_pak *surf_dialog;
struct model_pak *data;

/* checks */
if (!sysenv.num_models)
  return;
if ((id = request_dialog(sysenv.active, SURF)) < 0)
  return;
surf_dialog = &sysenv.dialog[id];

/* get the model data for all subsequent operations */
data = model_ptr(sysenv.active, RECALL);
g_return_if_fail(data != NULL);

/* default values */
data->csurf.grid = 40;        /* square grid side length (points) */
data->csurf.offset = 2.0;     /* drawing offset (Angs) */
data->csurf.prad = 0.97;      /* probe radius (Angs) */
data->csurf.rsf = FALSE;         /* results summary flag */

/* create dialog */
surf_dialog->win = gtk_dialog_new();
gtk_signal_connect(GTK_OBJECT (surf_dialog->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);
gtk_window_set_title(GTK_WINDOW(surf_dialog->win), "VdW surface");

/* radio buttons - scan which surface (xy,yz,xz) */
frame = gtk_frame_new ("Scan plane");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
/* do the first (DEFAULT MODE) radio button */
button = gtk_radio_button_new_with_label (NULL, "xy");
gtk_signal_connect_object(GTK_OBJECT (button), "clicked",
                          GTK_SIGNAL_FUNC (switch_plane),
                         (gpointer) PLANE_XY);
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

/* make a radio group */
surf_plane = gtk_radio_button_group (GTK_RADIO_BUTTON (button));
/* do the next button */
button = gtk_radio_button_new_with_label (surf_plane, "yz");
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (switch_plane),
                           (gpointer) PLANE_YZ);
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);
/* subsequent button... */
button = gtk_radio_button_new_with_label(
         gtk_radio_button_group (GTK_RADIO_BUTTON (button)), "xz");
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (switch_plane),
                           (gpointer) PLANE_XZ);
gtk_box_pack_start (GTK_BOX (vbox), button, TRUE, TRUE, 0);

/* grid side length */
frame = gtk_frame_new ("Grid side length");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
/* create spin box */
adj = (GtkAdjustment *) gtk_adjustment_new(data->csurf.grid, 5, 1000, 5, 10, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), TRUE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(vbox), spin, FALSE, TRUE,0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) GRID);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* surface drawing offset */
frame = gtk_frame_new ("Drawing offset");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),
                   frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
/* create spin box */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->csurf.offset, 0.0, 10.0, 0.5, 1.0, 0);
spin = gtk_spin_button_new(adj, 0.5, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start (GTK_BOX (vbox), spin, FALSE, TRUE,0);
/* update hook */
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) OFFSET);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* probe type */
frame = gtk_frame_new ("Probe radius");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->csurf.prad, 0.1, 10.0, 0.01, 0.05, 0);
spin = gtk_spin_button_new(adj, 0.01, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX (vbox), spin, FALSE, TRUE,0);
/* update hook */
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) PROBE);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* re-entrant surface flag */
frame = gtk_frame_new (NULL);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),
                   frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
button = gtk_check_button_new_with_label ("Print results summary");
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (rsf_toggle), (gpointer) data);
gtk_box_pack_start(GTK_BOX (vbox), button, TRUE, TRUE, 0);

/* make, hide, close - terminating buttons */
button = gtk_button_new_with_label ("Make");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->action_area),
                                    button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(calc_surf), (gpointer) data);

button = gtk_button_new_with_label ("Hide");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->action_area),
                                    button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(hide_con), (gpointer) data);

button = gtk_button_new_with_label ("Close");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->action_area),
                                    button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(close_dialog),
                         (gpointer) id);
/* done */
gtk_widget_show_all(surf_dialog->win);
}

/* CURRENT */
/* It's all yours Andrew... */
gint generate_surface(struct model_pak *src, struct model_pak *dest)
{
return(0);
}

/*************************/
/* construct the surface */
/*************************/
void make_surface(struct model_pak *data)
{
gint new;
struct model_pak *surf;

g_return_if_fail(data != NULL);

printf("Generating surface for model: %d\n",data->number);
/* NEW - complete molecules */
unfragment(data);
init_objs(REFRESH_COLOUR, data);
init_objs(CENT_COORDS, data);
calc_bonds(data);
calc_mols(data);

/* allocate & init for surface data */
new = sysenv.num_models;
surf = model_ptr(new, ASSIGN);
g_return_if_fail(surf != NULL);
sysenv.mode[new] = FREE;
surf->id = MARVIN;

memcpy(&surf->surface, &data->surface, sizeof(struct surface_pak));

printf("Miller (%d,%d,%d)\n",surf->surface.miller[0]
                            ,surf->surface.miller[1]
                            ,surf->surface.miller[2]);
printf("Shift %f\n",surf->surface.shift);
printf("Region sizes (%d,%d)\n",surf->surface.region[0]
                               ,surf->surface.region[1]);

/* TODO - valid shift/cut loop, region size??? */
/* call Andrew's routine */
if (generate_surface(data, surf))
  printf("Returned true.\n");
else
  printf("Returned false.\n");

/* TODO - model coord init */
}

/******************************************/
/* NEW - surface creation/cut elimination */
/******************************************/
void surface_dialog()
{
GtkWidget *surfwin, *vbox, *hbox, *label, *spin, *button;
GtkAdjustment *adj;
struct model_pak *data;

/* act on active model only */
data = model_ptr(sysenv.active, RECALL);
g_return_if_fail(data != NULL);

/* create a dialog */
surfwin = gtk_dialog_new();
gtk_window_set_title(GTK_WINDOW (surfwin), "Surface creator");

vbox = gtk_vbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surfwin)->vbox), vbox, TRUE, TRUE, 10);

/* use table? */

/* miller adjustment */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

label = gtk_label_new(" Miller ");
gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

/* miller - h */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.miller[0], -5, 5, 1, 1, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc data */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) MILLER_H);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* miller - k */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.miller[1], -5, 5, 1, 1, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc data */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) MILLER_K);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* miller - l */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.miller[2], -5, 5, 1, 1, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc data */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) MILLER_L);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* shift adjustment */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

label = gtk_label_new("  Shift  ");
gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.shift, 0.0, 1.0, 0.1, 0.1, 0);
spin = gtk_spin_button_new(adj, 0.1, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc model ptr to spinner */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) SHIFT);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);


/* region size adjustment */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(vbox), hbox, TRUE, TRUE, 0);

label = gtk_label_new(" Region sizes ");
gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);

/* region 1a */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.region[0], 0, 15, 1, 1, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc data */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) REGION1);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* region 2a */
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->surface.region[1], 0, 20, 1, 1, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE, 0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
/* NEW - assoc data */
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) REGION2);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* terminating buttons */
hbox = gtk_hbox_new(FALSE,0);
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surfwin)->action_area),
                                       hbox, FALSE, TRUE, 0);

button = gtk_button_new_with_label (" Generate ");
gtk_box_pack_start(GTK_BOX(hbox), button, FALSE, TRUE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (make_surface),
                          (gpointer) data);

button = gtk_button_new_with_label ("  Close  ");
gtk_box_pack_start(GTK_BOX(hbox), button, TRUE, FALSE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(gtk_widget_destroy),
                          GTK_OBJECT(surfwin));

/* done */
gtk_widget_show_all(surfwin);
}

struct sphere_pak
{
gint num_points;
gfloat *x;
gfloat *y;
gfloat *z;
};

/*******************************************/
/* calc 3D molecular/solvent acc. surfaces */
/*******************************************/
#define DEBUG_MYSURF 0
void mysurf(struct model_pak *data)
{
gint i, j, k, n, s;
gint num_unique, max, strips, nump, flag;
gint *unique_list;
gfloat x, y, z, cx, cy, cz, dx, dy, dz, da, r, theta, phi, r2, rcut;
gfloat div, dtheta, dphi, st, ct;
struct sphere_pak *spheres;

#if DEBUG_MYSURF
printf("scanning: %s\n",data->filename);
#endif

/* scan for all unique atom codes */
num_unique = 0;
unique_list = g_malloc(data->num_atoms * sizeof(gint));
for (i=0 ; i<data->num_atoms ; i++)
  {
  flag=0;
  for (j=0 ; j<num_unique ; j++)
    {
    if (*(unique_list+j) == (data->atoms+i)->atom_code)
      {
      flag++;
      break;
      }
    }
  if (!flag)
    {
    *(unique_list+num_unique) = (data->atoms+i)->atom_code;
    num_unique++;
    }
  }

#if DEBUG_MYSURF
printf("Found %d unique atom codes: ",num_unique);
for (i=0 ; i<num_unique ; i++)
  printf("%d ",*(unique_list+i));
printf("\n");
#endif

/* TODO - triangle tessellation for more uniform dot density */

/* split evenly between theta & phi */
/* FIXME - shouldn't da depend on r (vdw radius) ? */
/* actually, want shell_size to vary, with a const da for all r */
da = 4.0*PI/data->mysurf[0].shell_size;
dtheta = sqrt(da);
strips = (gint) (PI/dtheta);
dtheta = PI/strips;

/* magic number - how to properly calc? */
/* ie sum of DISCRETE vals of sin */
div = 2.0 / dtheta;

#if DEBUG_MYSURF
/* these may need adjustment (bit of a fudge) */
printf("sphere strips = %d\n",strips);
printf("div = %f\n",div);
#endif

/*  calc actual num of monopoles */
max = 0;
for (i=0 ; i<=strips ; i++)
  {
  theta = i*dtheta;
/* number of points for this value of theta */
  nump = (gint) (data->mysurf[0].shell_size * sin(theta) / div);
  if (nump < 1) 
    nump = 1;
  max += nump;
  }

#if DEBUG_MYSURF
printf("Requested shell points: %d, actual points: %d\n",data->mysurf[0].shell_size,max);
#endif

/* digitize spheres (each radius) at required density */
spheres = g_malloc(num_unique*sizeof(struct sphere_pak));

for (n=0 ; n<num_unique ; n++)
  {
  r = elements[*(unique_list+n)].vdw;
/* alloc for sphere points */
  (spheres+n)->x = g_malloc(max*sizeof(gfloat)); 
  (spheres+n)->y = g_malloc(max*sizeof(gfloat)); 
  (spheres+n)->z = g_malloc(max*sizeof(gfloat)); 
/* index for sphere points */
  k = 0;
  for (i=0 ; i<=strips ; i++)
    {
    theta = i*dtheta;
    st = sin(theta);
    ct = cos(theta);
/* number of points for this value of theta */
    nump = (gint) (data->mysurf[0].shell_size * st) / div;
    if (!nump)
      nump = 1;
    dphi = 2.0 * PI / nump;
    for (j=0 ; j<nump ; j++)
      {
      phi = j*dphi;
/* convert spherical coords to cartesian */
      (spheres+n)->x[k] = (gfloat) r*st*cos(phi);
      (spheres+n)->y[k] = (gfloat) r*st*sin(phi);
      (spheres+n)->z[k] = (gfloat) r*ct;
      k++;
      }
    }
  (spheres+n)->num_points = k;
#if DEBUG_MYSURF
/* allocation check - FIXME - why do we only sometimes get a miscalc? */
  if (k != max)
    printf("mysurf() warning: sphere mis-allocation.\n");
  printf("digitized sphere %d: points (%d/%d)\n",n,k,max);
#endif
  }

/* TODO - construct near neighbour lists - elim unecessary comparisons */

/* upper bound on memory */
max *= data->num_atoms;
data->mysurf[SOLSURF].num_points = 0;
data->mysurf[SOLSURF].max_points = max;
data->mysurf[SOLSURF].touch = g_malloc(max * sizeof(gint));
data->mysurf[SOLSURF].px = g_malloc(max * sizeof(gint));
data->mysurf[SOLSURF].py = g_malloc(max * sizeof(gint));
data->mysurf[SOLSURF].x = g_malloc(max * sizeof(gfloat));
data->mysurf[SOLSURF].y = g_malloc(max * sizeof(gfloat));
data->mysurf[SOLSURF].z = g_malloc(max * sizeof(gfloat));
data->mysurf[MOLSURF].num_points = 0;
data->mysurf[MOLSURF].max_points = max;
data->mysurf[MOLSURF].touch = g_malloc(max * sizeof(gint));
data->mysurf[MOLSURF].px = g_malloc(max * sizeof(gint));
data->mysurf[MOLSURF].py = g_malloc(max * sizeof(gint));
data->mysurf[MOLSURF].x = g_malloc(max * sizeof(gfloat));
data->mysurf[MOLSURF].y = g_malloc(max * sizeof(gfloat));
data->mysurf[MOLSURF].z = g_malloc(max * sizeof(gfloat));

/* pass 1 - contact surface */
k=0;
for (i=0 ; i<data->num_atoms ; i++)
  {
/* find the right digitized sphere */
  s=0;
  while (s<num_unique)
    if (*(unique_list+s++) == (data->atoms+i)->atom_code)
      break;
  s--;
/* loop over all points */
  for (n=0 ; n<(spheres+s)->num_points ; n++)
    {
/* current point */
    x = (data->atoms+i)->x + (spheres+s)->x[n];
    y = (data->atoms+i)->y + (spheres+s)->y[n];
    z = (data->atoms+i)->z + (spheres+s)->z[n];
/* is this point contained by any other atom? */
/* TODO - near neighbour lists to reduce this loop */
    flag=0;
    for (j=0 ; j<data->num_atoms ; j++)
      {
      dx = x - (data->atoms+j)->x;
      dy = y - (data->atoms+j)->y;
      dz = z - (data->atoms+j)->z;
      r2 = dx*dx + dy*dy + dz*dz; 
      rcut = elements[(data->atoms+j)->atom_code].vdw - POSITION_TOLERANCE;
      if (r2 < rcut*rcut)
        {
        flag++;
        break;
        }
      }
/* if not contained by another atom */
    if (!flag)
      {
/* vector from atom to molsurf, normalize */
      cx = x - (data->atoms+i)->x;
      cy = y - (data->atoms+i)->y;
      cz = z - (data->atoms+i)->z;
/* normalize - TODO - maybe precalc & mult by 1/r (mult is faster than div) */
      cx /= elements[(data->atoms+i)->atom_code].vdw;
      cy /= elements[(data->atoms+i)->atom_code].vdw;
      cz /= elements[(data->atoms+i)->atom_code].vdw;
/* scale with probe radius */
      cx *= data->mysurf[0].prad;
      cy *= data->mysurf[0].prad;
      cz *= data->mysurf[0].prad;
/* get probe centre (solsurf) */
      cx += x;
      cy += y;
      cz += z;
/* does it overlap any other atom? */
      flag=0;
      for (j=0 ; j<data->num_atoms ; j++)
        {
        dx = cx - (data->atoms+j)->x;
        dy = cy - (data->atoms+j)->y;
        dz = cz - (data->atoms+j)->z;
        r2 = dx*dx + dy*dy + dz*dz; 
rcut = elements[(data->atoms+j)->atom_code].vdw + data->mysurf[0].prad - POSITION_TOLERANCE;
        if (r2 < rcut*rcut)
          {
          flag++;
          break;
          }
        }
      if (!flag)
        {
        data->mysurf[SOLSURF].touch[k] = i;
        data->mysurf[SOLSURF].x[k] = cx;
        data->mysurf[SOLSURF].y[k] = cy;
        data->mysurf[SOLSURF].z[k] = cz;
        data->mysurf[SOLSURF].num_points++;
        data->mysurf[MOLSURF].touch[k] = i;
        data->mysurf[MOLSURF].x[k] = (data->atoms+i)->x + (spheres+s)->x[n];
        data->mysurf[MOLSURF].y[k] = (data->atoms+i)->y + (spheres+s)->y[n];
        data->mysurf[MOLSURF].z[k] = (data->atoms+i)->z + (spheres+s)->z[n];
        data->mysurf[MOLSURF].num_points++;
        k++;
        }
      }
    }
  }

/* TODO */
/* note which points touch more than one atom -> use for re-entrant surf */
/* post scan - solsurf coords that are identical (w/i tolerance) are */
/* in contact with more than one atom & therefore define re-entrant surf */

/* shrink the mysurf points alloc from max_points to num_points */

/* done all j, points remaining are part of contact surface */
/* PERCENTAGE of remaining points (x SA = 4.pi.r2) gives area */

/* pairwise molsurf */

/* cube fill better? (get volume as well) */
/* cube fill may use solvent acc. probe positions for the re-entrant */
/* part of the calculation (ie restrict w/all probes touching >1 atom) */

#if EXP
i=0;
j=1;
/* sphere dimensions & distances */
dx = (data->atoms+i)->x - (data->atoms+j)->x;
dy = (data->atoms+i)->y - (data->atoms+j)->y;
dz = (data->atoms+i)->z - (data->atoms+j)->z;
r_ij = sqrt(dx*dx + dy*dy + dz*dz);
r_i = elements[(data->atoms+i)->atom_code].vdw;
r_j = elements[(data->atoms+j)->atom_code].vdw;
r_p = data->mysurf[0].prad
r_ip = r_p + r_i;
r_jp = r_p + r_j;
/* cosine rule to get angle */
cosa = (r_ij*r_ij + r_ip*r_ip - r_jp*r_jp)/(2.0*r_ij*r_ip);
a = arcos(cosa);
b = PI - a;
/* probe position - anywhere on circle, so we choose y=0 */
p[0] = r_ip * cos(b);
p[1] = 0.0;
p[2] = r_ip * sin(b);

/* scoot along rij */
for (x[2]=0.0 ; x[2]<r_ij ; x[2]+=0.1)
  {
/* disk radius */
  r = p[0] - sqrt(r_p*r_p - (p[2]-z)*(p[2]-z));
/* TODO - digitized disks? */
/* TODO - smoother */
  for (theta=0.0 ; theta<2.0*PI ; theta += 0.2)
    {
    x[0] = r * cos(theta);
    x[1] = r * sin(theta);
/* rotate x,y,z to match actual i->j vector */

/* valid point? */

/* store as part of surf */

    }
  }
#endif

/* clean up */
for (i=0 ; i<num_unique ; i++)
  {
  g_free((spheres+i)->x);
  g_free((spheres+i)->y);
  g_free((spheres+i)->z);
  }
g_free(spheres);
g_free(unique_list);

/* redraw */
init_objs(CENT_COORDS, data);
wire_draw(SINGLE);
}

/********************************/
/* surface type display toggles */
/********************************/
void toggle_molsurf(gpointer *button)
{
struct model_pak *data;

data = (struct model_pak *) gtk_object_get_data(GTK_OBJECT(button), "ptr");
data->molsurf_on ^= 1;
/* redraw */
init_objs(CENT_COORDS, data);
wire_draw(SINGLE);
}

void toggle_solsurf(gpointer *button)
{
struct model_pak *data;

data = (struct model_pak *) gtk_object_get_data(GTK_OBJECT(button), "ptr");
data->solsurf_on ^= 1;
/* redraw */
init_objs(CENT_COORDS, data);
wire_draw(SINGLE);
}

/******************************************************************/
/* dot sphere molecular and solvent accessible surface calculator */
/******************************************************************/
void surf2_dialog(void)
{
gint id;
GtkWidget *frame, *vbox, *hbox, *button, *spin;
GtkAdjustment *adj;
struct dialog_pak *surf_dialog;
struct model_pak *data;

/* checks */
if (!sysenv.num_models)
  return;
if ((id = request_dialog(sysenv.active, SURF)) < 0)
  return;
surf_dialog = &sysenv.dialog[id];

/* get the model data for all subsequent operations */
data = model_ptr(sysenv.active, RECALL);
g_return_if_fail(data != NULL);

/* default parameters */
/* NB: MOLSURF *must* use these as well */
data->mysurf[SOLSURF].shell_size = 1000; /* shell digitization */
data->mysurf[SOLSURF].prad = 0.97;       /* probe radius (Angs) */

/* create dialog */
surf_dialog->win = gtk_dialog_new();
gtk_signal_connect(GTK_OBJECT (surf_dialog->win), "destroy",
                   GTK_SIGNAL_FUNC(event_close_dialog), (gpointer) id);
gtk_window_set_title(GTK_WINDOW(surf_dialog->win), "VdW surface");

/* number of points to digitize the spheres with */
frame = gtk_frame_new ("Dot sphere size");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
hbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), hbox);
/* create spin box */
adj = (GtkAdjustment *) gtk_adjustment_new
      (data->mysurf[0].shell_size, 100, 2000, 100, 100, 0);
spin = gtk_spin_button_new(adj, 0, 0);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX(hbox), spin, FALSE, TRUE,0);
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) SURF_ACCURACY);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* probe size */
frame = gtk_frame_new ("Probe radius");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
hbox = gtk_hbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), hbox);
adj = (GtkAdjustment *) gtk_adjustment_new
                        (data->mysurf[0].prad, 0.1, 10.0, 0.01, 0.05, 0);
spin = gtk_spin_button_new(adj, 0.01, 2);
gtk_spin_button_set_wrap(GTK_SPIN_BUTTON(spin), FALSE);
gtk_spin_button_set_shadow_type(GTK_SPIN_BUTTON(spin), GTK_SHADOW_OUT);
gtk_box_pack_start(GTK_BOX (hbox), spin, FALSE, TRUE,0);
/* update hook */
gtk_signal_connect(GTK_OBJECT(adj), "value_changed",
                   GTK_SIGNAL_FUNC(change_spin), (gpointer) spin);
gtk_object_set_data(GTK_OBJECT(spin), "id", (gpointer) PROBE);
gtk_object_set_data(GTK_OBJECT(spin), "ptr", (gpointer) data);

/* surface display type toggles */
frame = gtk_frame_new("Surface type");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->vbox),frame,FALSE,TRUE,0); 
gtk_container_set_border_width (GTK_CONTAINER(frame), 5);
vbox = gtk_vbox_new(TRUE, 0);
gtk_container_add(GTK_CONTAINER(frame), vbox);
button = gtk_check_button_new_with_label ("Contact (TODO re-entrant)");
gtk_box_pack_end (GTK_BOX (vbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (toggle_molsurf),
                           (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);

button = gtk_check_button_new_with_label (" Solvent accessible ");
gtk_box_pack_end (GTK_BOX (vbox), button, FALSE, FALSE, 0);
gtk_signal_connect_object (GTK_OBJECT (button), "clicked",
                           GTK_SIGNAL_FUNC (toggle_solsurf),
                           (gpointer) button);
gtk_object_set_data(GTK_OBJECT(button), "ptr", (gpointer) data);
/* default - on */
gtk_signal_emit_by_name (GTK_OBJECT (button), "clicked");

/* make, hide, close - terminating buttons */
button = gtk_button_new_with_label ("Calc");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->action_area),
                                    button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(mysurf), (gpointer) data);

button = gtk_button_new_with_label ("Close");
gtk_box_pack_start(GTK_BOX(GTK_DIALOG(surf_dialog->win)->action_area),
                                    button, FALSE, TRUE, 0);
gtk_signal_connect_object(GTK_OBJECT(button), "clicked",
                          GTK_SIGNAL_FUNC(close_dialog),
                         (gpointer) id);

/* done */
gtk_widget_show_all(surf_dialog->win);
}

